@@ -59,7 +59,7 @@ typedef struct note
   enum headtype noteheadtype; /**< Holds note head type.  */
   gint x_off;  /**< Holds x offset from default position.  */
   gint y_off; /**< Holds y offset from default position.  */
- 
+  GString *directive; /**< LilyPond text to be inserted after the note */
 }
 note;
 
@@ -273,6 +273,7 @@ typedef struct lyric
 typedef struct lilydirective
 {
   GString *directive;
+  gboolean locked;/**< If true the directive cannot be deleted easily */
 }
 lilydirective;
 
