@@ -94,11 +94,12 @@ typedef enum input_mode {
   INPUTCLASSIC = 1<<0, /* classic mode */
   INPUTEDIT = 1<<1, /* edit mode */
   INPUTINSERT = 1<<2, /* insert mode */
-#define ENTRY_TYPE_MASK (~(INPUTNORMAL|INPUTREST|INPUTBLANK))
+#define ENTRY_TYPE_MASK (~(INPUTNORMAL|INPUTREST|INPUTBLANK|INPUTRHYTHM))
   INPUTNORMAL = 1<<3, /* entry type notes */
   INPUTREST = 1<<4, /* entry type rests */
   INPUTBLANK = 1<<5,/* entry type non-printing rests */
-  TRAVERSE = 1<<6 /* read-only */
+  TRAVERSE = 1<<6, /* read-only */
+  INPUTRHYTHM = 1<<7, /*Input rhythms without pitches*/
 }input_mode;
 
 /**
@@ -134,21 +135,23 @@ typedef struct
   /* Mudelaobjunion u; has become a pointer*/
   gpointer object; /**< the structures pointed to are given in denemo_objects.h */
   gboolean isinvisible; /**< If  TRUE it will be rendered white and will not be seen. If it is set FALSE then it will be drawn normally. */
-}DenemoObject;
+} DenemoObject;
 
 
 /**
- * Enumeration for a staffs context. 
- * Currently only four to chose from,  may change if Lilypond 
- * adds any more
+ * Control of LilyPond context
+ * Allows for e.g.  Piano context within staff group context by using bit fields
  */
 typedef enum 
 {
-  DENEMO_NONE=1,
-  DENEMO_PIANO,
-  DENEMO_GROUP,
-  DENEMO_CHOIR
-}DenemoContext;
+  DENEMO_NONE = 0,
+  DENEMO_PIANO_START =  1<<0,
+  DENEMO_PIANO_END = 1<<1,
+  DENEMO_GROUP_START = 1<<2,
+  DENEMO_GROUP_END = 1<<3,
+  DENEMO_CHOIR_START = 1<<4,
+  DENEMO_CHOIR_END = 1<<5
+} DenemoContext;
 
 
 /**
@@ -165,21 +168,21 @@ typedef GList measurenode;
 typedef struct 
 {
   measurenode *measures; /**< This is a pointer to each measure in the staff */
-  gint sclef; /**< Set initial clef see denemo_objects.h clefs */
-  gint skey; /**< Set the inital keysig. >0=sharps <0 = flats */  
+  gint sclef; /**< The initial clef see denemo_objects.h clefs */
+  gint skey; /**< The inital keysig. >0=sharps <0 = flats */  
   gint skey_isminor;  /**< Values 0 = major, 1 = minor, 2 = mode */
-  gint skeyaccs[7]; /**< Value used for current keysignature -7=Cb... -1=F, 0=C, 1=G...7=C# in a chain of fifths */
+  gint skeyaccs[7]; /**< The initial keysignature. Values -7=Cb... -1=F, 0=C, 1=G...7=C# in a chain of fifths */
   gint stime1; /**< This is the initial timesig numerator */
   gint stime2; /**< This is the initial timesig denominator */
-  gint leftmost_clefcontext; /**< The first clef used in a staff */
-  gint leftmost_keysigcontext; /**< The first or initial keysig used in a staff. */
-  gint leftmost_keywidth; /**< Space allowed for initial keysig  */
-  gint leftmost_time1context; /**< This is the initial timesig numerator */
-  gint leftmost_time2context; /**< This is the initial timesig denominator */
+  gint leftmost_clefcontext; /**< The clef for the leftmost measure visible in the window*/
+  gint leftmost_keysigcontext; /**< The keysig at thestart of the leftmost measure visible in the window*/
+  gint leftmost_keywidth; /**< Space allowed for leftmost keysig  */
+  gint leftmost_time1context; /**< The leftmost visible timesig numerator */
+  gint leftmost_time2context; /**< The leftmost visible timesig denominator */
 
-  gint leftmost_keyaccs[7]; /**< Rirst keysignature used in the staff */
-  gint leftmost_stem_directive; /**< Rirst stem directive used in the staff */
-  DenemoContext context;   /**< New variable to define the context in which this staff appears */
+  gint leftmost_keyaccs[7]; /**<  The keysig at thestart of the leftmost measure visible in the window */
+  gint leftmost_stem_directive; /**< Stem directive at start of leftmost visible measure */
+  DenemoContext context;   /**< The Lilypond context in which this staff appears */
   /*
    * Staff Parameters
    * Added Adam Tee 27/1/2000, 2001 
@@ -197,11 +200,15 @@ typedef struct
   GString *denemo_name; /**< denemo_name gets copied into lily_name */
   GString *lily_name; /**< this is the name of the staff that is export to lilypond */
   GString *midi_instrument; /**< midi instrument name used for the staff when exported via midi */
-
+  gboolean midi_prognum_override; /**< override to allow manually setting prognum + channel */
+  guint8 midi_prognum; /**< midi prognum assigned to the staff voice */
+  guint8 midi_channel; /**< midi channel assigned to the staff voice */
+  
   gint space_above; /**< space above the staff used in the denemo gui */
   gint space_below; /**< space below the staff used in the denemo gui */
   gboolean haslyrics; /**< TRUE if the staff has had lyrics attached */
   gboolean hasfigures; /**<TRUE if the staff has had figures attached. Only one staff should have this set */
+  gboolean hasfakechords; /**<TRUE if the staff has had chord symbols attached. Only one staff should have this set */
   gint voicenumber; /**< presently set to 2 for any non-primary voices; we might want to
    * change that, though */
   measurenode ** is_parasite; /**< points to address of host staff's measures 
@@ -212,8 +219,10 @@ typedef struct
   GList *tone_store; /**< list of pitches and durations used a source for
 			the notes in this staff
 			the data are tone* */
-  gchar *lilybefore, *lilyafter;/**< Lilypond directives to be issued before and after the staff */
-
+  GString *staff_prolog;/**< Customised version of the LilyPond prolog defining the music of this staff */
+  GString *lyrics_prolog;/**< Customised version of the LilyPond prolog defining the lyrics of this staff */
+  GString *figures_prolog;/**< Customised version of the LilyPond prolog defining the figured bass of this staff */
+  GString *fakechords_prolog;/**< Customised version of the LilyPond prolog defining the chord symbols of this staff */
 }DenemoStaff;
 
 /* The ->data part of each staffnode points to a staff structure */
@@ -229,37 +238,30 @@ typedef struct staff_info
 }
 staff_info;
 
-
-/**
- * Union for the two types of keypress related functions
- */
-union func_union
+typedef enum
 {
-  GtkFunction nocallback;  /**< Not associated with a menued item */
-  GActionCallback callback; /**< Associated with a menued item */
-};
-
-/**
- * This structure is what's stored in the keymap data structure, both
- * in the array and in the hash tables.  Note that some redundant
- * information is stored in either case, but either one structure or
- * the other will need or find it useful to have each piece of information. 
- */
-typedef struct KeybindingInfo
-{
-  gint keyval;
-  gint state;
-  gint command_number;
-  gint callback_action;
-  union func_union func;
-}KeybindingInfo;
+	KeymapEntry,
+	KeymapToggleEntry,
+	KeymapRadioEntry
+}KeymapCommandType;
 
 typedef struct DenemoKeymap
 {
-  GList **commands;
-  GHashTable *quick_lookup_hashes;
-}keymap;
+  //command information store
+  GtkListStore *commands; // ListStore for the commands
 
+  //reference for easy access
+  GHashTable *idx_from_name; //hashtable linking the name of a command to
+							 //its index in the ListStore (values are guint *)
+
+  GHashTable *idx_from_keystring; //hashtable linking the string representing
+                                  //a keypress to the index of its command
+								  //The keystring is the output of
+								  //dnm_accelerator_name()
+  
+  //additional information to be able to retrieve actions from command names
+  gchar *action_group_name;
+}keymap;
 
 #define MAX_HISTORY 10
 /**
@@ -287,6 +289,7 @@ typedef struct DenemoPrefs
   GString *csoundorcfile; /**< Path to .orc file used for csound playback */
   gboolean rtcs; /**< Real time csound */
   GString *pdfviewer; /**< PDF viewer */
+  GString *imageviewer; /**< Image Viewer */
   GString *texteditor; /**< texteditor for editing scripts and lilypond files */
   GString *denemopath; /**< path were denemo files are saved */
   GQueue *history; /**< Queue to contain recently opened files */
@@ -296,12 +299,13 @@ typedef struct DenemoPrefs
 }DenemoPrefs;
 
 /**
- * Contains all the lilypond header information
+ * Contains the lilypond header information for the movements, plus markup between movements.
  *
  */
-typedef struct LilypondHeaderInfo
+
+typedef struct LilypondHeaderFields
 {
-/* Mudela file header information */
+/* LilyPond movement header and markup information */
   GString *title;
   GString *subtitle;
   GString *poet;
@@ -316,11 +320,21 @@ typedef struct LilypondHeaderInfo
   GString *copyright;
   GString *footer;
   GString *tagline;
-/*  GString *; */
+  GString *extra;
+  /* lilypond before and after each \score block     */
+  GString *lilypond_before;
+  GString *lilypond_after;
+}LilypondHeaderFields;
 
-}LilypondHeaderInfo;
 
 
+typedef enum 
+{
+  REPLACE_SCORE,
+  ADD_STAFFS,
+  ADD_MOVEMENTS
+} ImportType;
+
 typedef enum 
 {
   WOODWIND,
@@ -386,44 +400,51 @@ typedef struct Bookmark
 }Bookmark;
 
 /** 
- * Contains paper score configuration
+ * Control of the LilyPond output for the whole musical score DenemoGUI
  *
  */
-typedef struct DenemoScoreConfig
+typedef struct DenemoLilyControl
 {
   GString *papersize;
   gint fontsize;
-  GString	*lilyversion;
+  GString *lilyversion;
   gboolean orientation;
+  gboolean excerpt;
+  GString *lilypond; /**< Lilypond directive for all music in the movements */
 	
-}DenemoScoreConfig;
+} DenemoLilyControl;
 
+typedef struct DenemoScoreblock {
+  GString *scoreblock;/**< text of the scoreblock */
+  gboolean visible;/**< Whether the scoreblock should be used by default */
+} DenemoScoreblock;
+
+/** 
+ * The (singleton) root object for the program
+ *
+ */
+struct DenemoRoot
+{
+  GList *guis; /**< the list of DenemoGUI objects, representing pieces of music
+		  simultaneously open */
+  DenemoPrefs prefs;  /**< Preferences stored on exit and re-loaded on startup */
+  gint autosaveid;/**< autosave timer id: only one musical score is being autosaved at present */
+  gint accelerator_status; /**< if the accelerators have been saved, or extra ones for special keys defined  */
+}  Denemo; /**< The root object. */
 
 /*
- *It contains
- * within it a DenemoPrefs *, and when Denemo supports editing
- * multiple scores simultaneously, each score's scoreinfo structure will
- * contain a pointer to the same single DenemoPrefs structure.
- *
- * I originally hit on this idea as a hack -- everything in Denemo was
- * coded to pass around struct scoreinfo *s, and I didn't want to
- * change this around -- but then I realized that it was a reasonably
- * good idea.  Basically all operations need access to the scoreinfo
- * structure, and a handful need access to the DenemoPrefs structure as
- * well. Only the preferences dialog needs access to _just_ the
- * DenemoPrefs structure.  So, why not keep things simple and always pass
- * around scoreinfos? */
- 
-/**
- * DenemoScore contains the information relating to a musical score
+ *  DenemoScore structure representing a single movement of a piece of music.
+ *  A movement corresponds with a single \score{} block in the LilyPond language
+ *  that is,  uninterrupted music on a set of staffs, preceded by a title.
  */
+ 
 typedef struct DenemoScore
 {
   gboolean readonly; /**< Indicates if the file is readonly or not */
   GList *curlilynode; /**< the node of the lily parse tree on display 
 			 in textwindow */
   GList *lily_file; /**< root of lily file parse, see lilyparser.y etc  */
-  DenemoScoreConfig *config; /**< contains the paper configuration of the score*/
+
  
   gint leftmeasurenum; /**< start at 1 */
   gint rightmeasurenum;/**< start at 1 */
@@ -465,11 +486,7 @@ typedef struct DenemoScore
    * if the cursor advances to the next measure from the next "insert chord"
    * operation */
   gint curmeasure_stem_directive;
- 
-  /* support for rhythm patterns */
-  GList *rhythms;/**< list of RhythmPattern s */
-  GList *currhythm; /**< currently in use element of rhythms */
-  GList *rstep; /**< step within RhythmPattern->rsteps, the current element of the current rhythm pattern */
+
 
   gboolean is_grace_mode;
   /* Is there a figured bass present, is so this relates the bass
@@ -486,9 +503,8 @@ typedef struct DenemoScore
   gint lastmeasuremarked;
   gint firstobjmarked;
   gint lastobjmarked;
+  LilypondHeaderFields headerinfo;/**< Lilypond header fields and markup for this movement */
 
-  gboolean haschanged;
-  gboolean is_slured; /** Temp field for testing for a slur*/
 
   /* Fields used for MIDI playback */
   gint tempo;
@@ -496,23 +512,13 @@ typedef struct DenemoScore
   gint end;
   gint stafftoplay;
 
-  LilypondHeaderInfo *headerinfo;
-  
-  /* Filename */
-  GString *filename;
-  GString *autosavename;
 
- 
   
   GList *savebuffer;
   /* Holds List of Plugin Names */
   
 
-  /* a list of scoreinfos for each scoreblock in the (lilypond) file
-     the last one is always the one currently at the address hardwired
-     into the display functions.
-     NULL if just one scoreblock in file */
-  GList *scoreblocks;
+
 
   /*list of undo data */
   GQueue *undodata;
@@ -520,24 +526,25 @@ typedef struct DenemoScore
   gint undo_redo_mode;
 
   
-  DenemoScoreConfig *sconfig;
+
  
   GList *bookmarks;
-  
+  gint currentbookmark;
   GList *Instruments;
   
 }DenemoScore;
 
 /**
- * DenemoGUI contains all the GUI information as 
- * well as a pointer to the DenemoScore structure.
- * 
- * It is done this way to allow abstraction from GUI specific 
- * and Score specific functions
+ * DenemoGUI representing a musical score, with associated top level
+ * GUI and a list of movements (DenemoScore) and a pointer to the current
+ * movement. 
  */
 typedef struct DenemoGUI
 {
-		
+  /* window state */
+  gint width;
+  gint height;
+  gboolean maximized;
   /* Fields used fairly directly for drawing */
   GtkWidget *window;
   GtkWidget *scorearea;
@@ -553,28 +560,47 @@ typedef struct DenemoGUI
   GtkWidget *ModelessMenu;/**< Menu to give the note editing facilities when used without modes */
 
 
-
-
   GtkWidget *statusbar;
   gint status_context_id;
-  GtkTextBuffer *textbuffer;   /**< text of lily input string corresponding to curobj */
-  GtkWidget *textwindow; /**< top level window containing lily textwindow */
-  GtkWidget *textview; /**< text entry containing lily text to be edited */
-  GtkWidget *musicdatabutton; /**< button for editing music data textually */
-  gulong sigid; /**< signal handler id for when text is changed */
-  GtkWidget *musictext; /**< status bar with text of lily input string 
-			   corresponding to music item at cursor */
+  GtkWidget *textwindow; /**< LilyPond output window */
+  GtkTextBuffer *textbuffer;   /**< buffer for LilyPond text */
+  GtkTextView *textview; /**< LilyPond output text view */
+
+  
   GtkUIManager *ui_manager;  /**< UI manager */
   GList *plugins;
   gint plugincounter;
-  DenemoPrefs *prefs;  /**< And the DenemoPrefs pointer */
+
   GtkWidget* articulation_palette; /**< Articulation palette window */
   gboolean pitch_recognition;/**< Whether pitch recognition off/on */
   input_mode mode; /**< Input mode for Score */
   GtkWidget *progressbar;
+
+
+
+  GList *movements;   /**< a list of DenemoScore, NULL if just one movement */
+  DenemoScore *si;  /**< the (current)  movement in the musical score controlled by this gui */
+  DenemoLilyControl lilycontrol; /**< Control of the LilyPond output for whole musical score */
+  GList *custom_scoreblocks; /**< List of customized texts for LilyPond output, replaces standard score blocks, elements are DenemoScoreblock * */
+  GString *custom_prolog; /**< Customized text for LilyPond output, replaces standard prolog */
+  gpointer lilystart, lilyend; /**<range of lilytext  */
+  GString **target; /**< pointer to target string for modification in lilytext  */
+  GList *anchors;/**< anchors in the LilyPond text at points where that can be edited */
+
+  GString *filename;/**< the filename to save to */
+  GString *autosavename;/**< the filename to autosave to, full path */
+
+ 
+  guint changecount;/**< number of edits since last save */
+  guint lilysync;/**< value of changecount when the Lily text was last refreshed */
+
+ 
+  /* support for rhythm patterns */
+  GList *rhythms;/**< list of RhythmPattern s */
+  GList *currhythm; /**< currently in use element of rhythms */
+  GList *rstep; /**< step within RhythmPattern->rsteps, the current element of the current rhythm pattern */
   struct RhythmPattern *singleton_rhythms[256]; /**< rhythm patterns for the EntryToolbar of this GUI */
   struct RhythmPattern *prevailing_rhythm; /**< one of singleton_rhythms used for entering notes */
-  DenemoScore *si;
 }DenemoGUI;
 
 
