@@ -405,13 +405,13 @@ create_articulation_widget (DenemoGUI * si)
  * Callback to hide articulation palette
  */
 static gboolean
-hide_palette (GtkWidget * widget, GdkEvent * event, DenemoGUI * si)
+hide_palette (GtkWidget * widget, GdkEvent * event, DenemoGUI * gui)
 {
   GtkWidget *toggle_palette;
 
-  if (si->articulation_palette)
+  if (gui->articulation_palette)
     {
-      toggle_palette = gtk_ui_manager_get_widget (si->ui_manager,
+      toggle_palette = gtk_ui_manager_get_widget (gui->ui_manager,
 						  "/MainMenu/ViewMenu/ToggleArticulationPalette");
       gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (toggle_palette),
 				      FALSE);
