@@ -20,7 +20,7 @@ static gchar *string_barlines[6] = { "Ordinary", "Double",
 
 struct callbackdata
 {
-  DenemoScore *si;
+  DenemoGUI *gui;
   GtkWidget *combo;
 };
 
@@ -92,7 +92,7 @@ insert_barline (GtkAction * action, DenemoGUI * gui)
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
 		      okbutton, TRUE, TRUE, 0);
 
-  cbdata.si = gui->si;
+  cbdata.gui = gui;
   cbdata.combo = combobox;
   /* Signal connection */
   gtk_signal_connect (GTK_OBJECT (okbutton), "clicked",
@@ -132,13 +132,13 @@ void
 add_barline (GtkWidget * widget, gpointer data)
 {
   struct callbackdata *cbdata = (struct callbackdata *) data;
-  DenemoScore *si = cbdata->si;
+  DenemoScore *si = cbdata->gui->si;
 
   gchar *thetext =
     (gchar *)
     gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (cbdata->combo)->entry));
 
-  object_insert (si, newbarline (barlinefromname (thetext)));
+  object_insert (cbdata->gui, newbarline (barlinefromname (thetext)));
 
 }
 
