@@ -247,11 +247,12 @@ static note *new_note(gint mid_c_offset, gint enshift, gint dclef) {
  * Add note to the current chord
  * The note will not get added if it is
  * present already
+ * return note added or NULL if not added
  */
-void
+note *
 addtone (DenemoObject * thechord, gint mid_c_offset, gint enshift, gint dclef)
 {
-  note *newnote;
+  note *newnote = NULL;
 
   if (!g_list_find_custom
       (((chord *) thechord->object)->notes, GINT_TO_POINTER (mid_c_offset),
@@ -278,6 +279,7 @@ addtone (DenemoObject * thechord, gint mid_c_offset, gint enshift, gint dclef)
       ((chord *) thechord->object)->sum_mid_c_offset += mid_c_offset;
       ((chord *) thechord->object)->numnotes++;
     }
+  return newnote;
 }
 
 void dnm_addtone (DenemoObject * thechord, gint mid_c_offset, gint enshift, gint dclef) {
