@@ -22,7 +22,7 @@ objnode *nearestnote (DenemoObject * thechord, gint mid_c_offset);
 
 void modify_note (chord *thechord,  gint mid_c_offset, gint enshift, gint dclef);
 
-void addtone (DenemoObject * mudelaobj, gint mid_c_offset, gint enshift,
+note* addtone (DenemoObject * mudelaobj, gint mid_c_offset, gint enshift,
 	      gint dclef);
 
 void removetone (DenemoObject * mudelaobj, gint mid_c_offset, gint dclef);
