@@ -1,9 +1,9 @@
-/* clefdialog.cpp
+/* clefdialog.c
  * callback that creates "change staff" dialog box
  * user for further action
 
  * for Denemo, a gtk+ frontend to GNU Lilypond
- * (c) 1999-2005  Matthew Hiller,Adam Tee
+ * (c) 1999-2005, 2008  Matthew Hiller,Adam Tee, Richard Shann
  */
 
 #include <gtk/gtk.h>
@@ -145,11 +145,11 @@ clef_change (DenemoGUI * gui, actiontype action)
 	{
 	  dnm_setinitialclef (gui->si, curstaffstruct, clef);
 
-
+	  score_status(gui, TRUE);
 	}
       else
 	{
-	  object_insert (gui->si, dnm_newclefobj (clef));
+	  object_insert (gui, dnm_newclefobj (clef));
 
 	}
       displayhelper (gui);
