@@ -219,8 +219,9 @@ jumpcursor (gint cursor_y, gint fromnote, gint tonote)
  *  into the score
  */
 void
-object_insert (DenemoScore * si, DenemoObject * mudela_obj_new)
+object_insert (DenemoGUI * gui, DenemoObject * mudela_obj_new)
 {
+  DenemoScore *si = gui->si;
   unre_data *undo = (unre_data *) g_malloc (sizeof (unre_data));
 
   declarecurmudelaobj;
@@ -258,7 +259,7 @@ object_insert (DenemoScore * si, DenemoObject * mudela_obj_new)
 
   update_undo_info (si, undo);
 
-
+  score_status(gui, TRUE);
   si->markstaffnum = 0;
 }
 
@@ -312,6 +313,8 @@ measureright (DenemoGUI * gui)
 gboolean
 swapmovements (GtkAction *action, DenemoGUI * gui)
 {
+  if(!confirm_insertstaff_custom_scoreblock(gui))
+    return;
   GList *this = g_list_find( gui->movements, gui->si);
   if(this->prev) {
     GList * prev = this->prev;
@@ -328,7 +331,7 @@ swapmovements (GtkAction *action, DenemoGUI * gui)
     prev->next = next;
     prev->prev = this;
     gchar *str = g_strdup_printf("This movement is now number %d in the score", 1+g_list_index(gui->movements, gui->si));
-    warningdialog(str);//FIXME use an informative not a warning
+    infodialog(str);
     g_free(str);
   } else
     warningdialog("There is no previous movement to swap with");
@@ -340,26 +343,107 @@ swapmovements (GtkAction *action, DenemoGUI * gui)
 gboolean
 swapstaffs (GtkAction *action, DenemoGUI * gui)
 {
+  if(!confirm_insertstaff_custom_scoreblock(gui))
+    return;
   if (gui->si->currentstaff && gui->si->currentstaff->prev)
     {
-      gpointer *temp;
+      DenemoStaff *temp;
       temp = gui->si->currentstaff->data;
-      gui->si->currentstaff->data = gui->si->currentstaff->prev->data;
-      gui->si->currentstaff->prev->data = temp;
-      gui->si->currentstaffnum--;
-      gui->si->currentstaff = gui->si->currentstaff->prev;
+      if(temp->context==DENEMO_NONE ||
+	confirm("A context is set on this staff", "You will need to alter the staff->properties->context of this and the previous staff; Proceed?")) {
+	  
+	  gui->si->currentstaff->data = gui->si->currentstaff->prev->data;
+	  gui->si->currentstaff->prev->data = temp;
+	  gui->si->currentstaffnum--;
+	  gui->si->currentstaff = gui->si->currentstaff->prev;
+	  setcurrentprimarystaff (gui->si);
+	  setcurrents (gui->si);
+	  move_viewport_up (gui);
+	  score_status(gui, TRUE);
+	  return TRUE;
+	}
+    }
+  else
+    warningdialog("There is no previous staff to swap with");
+  return FALSE;
+}
+
+/**
+ * 
+ *
+ */
+gboolean
+splitstaffs (GtkAction *action, DenemoGUI * gui)
+{
+  if(!confirm_insertstaff_custom_scoreblock(gui))
+    return;
+  if (gui->si->currentstaff && gui->si->currentstaff->next)
+    {
+      DenemoStaff *thestaff = (DenemoStaff *)gui->si->currentstaff->data;
+      DenemoStaff *nextstaff = (DenemoStaff *)gui->si->currentstaff->next->data;
+      if((thestaff->voicenumber==1) && (nextstaff->voicenumber!=1))
+	nextstaff->voicenumber=1;
+      else
+	warningdialog("There is no voice below this one on this staff");
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
       move_viewport_up (gui);
       score_status(gui, TRUE);
       return TRUE;
     }
+  else
+    warningdialog("There is no voice to split from");
   return FALSE;
 }
 
 
+/**
+ * 
+ *
+ */
+gboolean
+joinstaffs (GtkAction *action, DenemoGUI * gui)
+{
+  if(!confirm_insertstaff_custom_scoreblock(gui))
+    return;
+  if (gui->si->currentstaff && gui->si->currentstaff->prev)
+    {
+      DenemoStaff *thestaff = (DenemoStaff *)gui->si->currentstaff->data;
+      DenemoStaff *prevstaff = (DenemoStaff *)gui->si->currentstaff->prev->data;
+      thestaff->voicenumber=2;
+      setcurrentprimarystaff (gui->si);
+      setcurrents (gui->si);
+      move_viewport_up (gui);
+      score_status(gui, TRUE);
+      return TRUE;
+    }
+  else
+    warningdialog("There is no staff above to move this staff into");
+  return FALSE;
+}
 
 
+/**
+ * Move si->currentstaff up an one voice, return TRUE if successful
+ *
+ */
+gboolean
+voiceup (DenemoGUI * gui)
+{
+  if(!gui->si->currentstaff)
+    return FALSE;
+  if (gui->si->currentstaff && (((DenemoStaff *)(gui->si->currentstaff->data))->voicenumber==2))  {
+      gui->si->currentstaffnum--;
+      gui->si->currentstaff = gui->si->currentstaff->prev;
+      setcurrentprimarystaff (gui->si);
+      setcurrents (gui->si);
+      move_viewport_down (gui);
+      return TRUE;
+    } else
+      warningdialog("This is the first voice");
+  return FALSE;
+}
+
 /**
  * Move si->currentstaff up an one staff/voice, return TRUE if successful
  *
@@ -367,7 +451,11 @@ swapstaffs (GtkAction *action, DenemoGUI * gui)
 gboolean
 staffup (DenemoGUI * gui)
 {
-  if (gui->si->currentstaff && gui->si->currentstaff->prev)
+  if(!gui->si->currentstaff)
+    return FALSE;
+  while (((DenemoStaff *)(gui->si->currentstaff->data))->voicenumber!=1)
+    voiceup(gui);
+  if (gui->si->currentstaff->prev)
     {
       gui->si->currentstaffnum--;
       gui->si->currentstaff = gui->si->currentstaff->prev;
@@ -375,11 +463,32 @@ staffup (DenemoGUI * gui)
       setcurrents (gui->si);
       move_viewport_up (gui);
       return TRUE;
-    }
+    }else
+      warningdialog("This is the first staff");
   return FALSE;
 }
 
 
+/**
+ * Move si->currentstaff down an one staff/voice, return TRUE if successful
+ *
+ */
+gboolean
+voicedown (DenemoGUI * gui)
+{
+  if(!gui->si->currentstaff)
+    return FALSE;
+  if (gui->si->currentstaff->next && ((DenemoStaff *)(gui->si->currentstaff->next->data))->voicenumber==2) {
+      gui->si->currentstaffnum++;
+      gui->si->currentstaff = gui->si->currentstaff->next;
+      setcurrentprimarystaff (gui->si);
+      setcurrents (gui->si);
+      move_viewport_down (gui);
+      return TRUE;
+    } else
+      warningdialog("This is the last voice");
+  return FALSE;
+}
 /**
  * Move si->currentstaff down an one staff/voice, return TRUE if successful
  *
@@ -387,6 +496,10 @@ staffup (DenemoGUI * gui)
 gboolean
 staffdown (DenemoGUI * gui)
 {
+  if(!gui->si->currentstaff)
+    return FALSE;
+  while (gui->si->currentstaff->next && ((DenemoStaff *)(gui->si->currentstaff->next->data))->voicenumber==2)
+    voicedown(gui);
   if (gui->si->currentstaff->next)
     {
       gui->si->currentstaffnum++;
@@ -394,9 +507,12 @@ staffdown (DenemoGUI * gui)
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
       move_viewport_down (gui);
-    }
+    }else
+      warningdialog("This is the last staff");
 }
 
+
+
 /**
  * move the cursor one position to the left
  *
@@ -537,9 +653,7 @@ shiftcursor (DenemoGUI  *gui, gint note_value)
 	gui->si->cursor_y = mid_c_offset;
 	tonechange(gui->si, FALSE);
       } else {/* single-note chord - change the note */
-      DenemoStaff *curstaff = ((DenemoStaff*)gui->si->currentstaff->data);
-      find_leftmost_staffcontext (curstaff, gui->si);
-      int dclef = curstaff->leftmost_clefcontext;	    
+      gint dclef = find_prevailing_clef(gui->si);	    
       modify_note((chord*)theobj->object, mid_c_offset, gui->si->curmeasureaccs[note_value], dclef);
       showwhichaccidentals ((objnode *) gui->si->currentmeasure->data,
 			    gui->si->curmeasurekey, gui->si->curmeasureaccs);
@@ -555,7 +669,7 @@ shiftcursor (DenemoGUI  *gui, gint note_value)
       do {
 	if(g) {
 	  for(h = ((RhythmElement*)g->data)->functions;h;h=h->next) {
-	    nextmeasure (gui->si, TRUE);	
+	    insertion_point (gui->si);	
 	    gui->si->cursoroffend = FALSE;
 	    ((GtkFunction)h->data)(gui);
 	    displayhelper(gui);
@@ -586,7 +700,7 @@ insert_rhythm_pattern(DenemoGUI  *gui) {
     do {
       if(g) {
 	for(h = ((RhythmElement*)g->data)->functions;h;h=h->next) {
-	  nextmeasure (gui->si, TRUE);
+	  insertion_point (gui->si);
 	  gui->si->cursoroffend = FALSE;
 	  if(!code_is_a_duration(modifier_code(h->data)))
 	    cursorleft(gui);
@@ -604,22 +718,18 @@ insert_rhythm_pattern(DenemoGUI  *gui) {
 }
 
 /**
- * badly named and incorrectly described (below)
- * this is really a "find_or_create_insertion_point" function
- * that is, if the current measure is full it moves into the next one provided it is empty
- * (or creates a new one and moves into it)
- * if it is not full, or if the next measure is not empty it does nothing.
- * param ALL add any extra measure to all staffs if the current staff has a full set of measures
-old misleading comment:
- * Goto the next measure 
- * Goto the next measure in the score.  A new measure will
- * be created if required
- * @param si pointer to the scoreinfo structure
- * @param all apply to all staffs or not
+ * insertion_point()
+ * chooses/creates a good insertion point.
+ * if the cursor is at the end of a full measure:
+ *      creates a new measure and makes it the current one.
+ * if the cursor is at the end of a full measure before an empty measure:
+ *      it makes that empty measure current. 
+ * 
  */
 void
-nextmeasure (DenemoScore * si, gboolean all /*=TRUE */ )
+insertion_point (DenemoScore * si)
 {
+
   gboolean next_measure;
 
   /* First, check to see if the insertion'll cause the cursor to
@@ -634,12 +744,12 @@ nextmeasure (DenemoScore * si, gboolean all /*=TRUE */ )
     {
       if (!si->currentmeasure->next)
 	{
-
+	  gboolean all = TRUE;//add to all measures
 	  g_debug ("Appending a new measure\n");
 
 	  /* Add a measure and make it currentmeasure */
 	  if(!(all && si->currentstaff && si->currentstaff && g_list_length(((DenemoStaff *) si->currentstaff->data)->measures) == g_list_length(si->measurewidths)))
-	    all = FALSE; // add only to current staff if it is short
+	    all = FALSE; // add only to current staff if it is shorter than some other staff
 	  si->currentmeasure =
 	    dnm_addmeasures (si, si->currentmeasurenum, 1, all);
 	}
@@ -663,10 +773,10 @@ nextmeasure (DenemoScore * si, gboolean all /*=TRUE */ )
  * @param rest specifies a note is a rest
  */
 void
-dnm_insertchord (DenemoScore * si, gint duration, input_mode mode, 
+dnm_insertchord (DenemoGUI * gui, gint duration, input_mode mode, 
 		 gboolean rest)
 {
-
+  DenemoScore *si = gui->si;
   DenemoObject *mudela_obj_new;
   int prognum;
   DenemoStaff *curstaffstruct;
@@ -675,23 +785,15 @@ dnm_insertchord (DenemoScore * si, gint duration, input_mode mode,
     changeduration(si, duration);
     return;
   }
-  nextmeasure (si, FALSE);
+  insertion_point (si);
   
   /* Now actually create the chord */
   mudela_obj_new = newchord (duration, 0, 0);
-  if ( (mode & INPUTBLANK) && (rest != TRUE))
-    {
-      addtone (mudela_obj_new, si->cursor_y,
-	       si->cursoraccs[si->staffletter_y], si->cursorclef);
-      mudela_obj_new->isinvisible = TRUE;
-    }
-  else if ((mode & INPUTNORMAL) && (rest != TRUE))
+  if ((mode & INPUTNORMAL) && (rest != TRUE))
     addtone (mudela_obj_new, si->cursor_y, si->cursoraccs[si->staffletter_y],
 	     si->cursorclef);
-
-
-
-
+  if ((mode & INPUTBLANK) || (gui->mode & INPUTBLANK))
+    mudela_obj_new->isinvisible = TRUE;
 
   if (si->is_grace_mode)
     ((chord *) mudela_obj_new->object)->is_grace = TRUE;
@@ -702,7 +804,7 @@ dnm_insertchord (DenemoScore * si, gint duration, input_mode mode,
      which will in turn invoke update_hscrollbar, so we
      don't need to invoke that here.  */
 
-  object_insert (si, mudela_obj_new);
+  object_insert (gui, mudela_obj_new);
   curstaffstruct = (DenemoStaff *) si->currentstaff->data;
   prognum = select_program (curstaffstruct->midi_instrument->str);
   /*playnotes (si->prefs->immediateplayback, *(chord *) mudela_obj_new->object,
@@ -715,11 +817,12 @@ dnm_insertchord (DenemoScore * si, gint duration, input_mode mode,
  * @param type the type of tuplet to insert
  */
 void
-dnm_inserttuplet (DenemoScore * si, tuplet_type type)
+dnm_inserttuplet (DenemoGUI * gui, tuplet_type type)
 {
+  DenemoScore *si = gui->si;
   DenemoObject *mudela_obj_new;
 
-  nextmeasure (si, FALSE);
+  insertion_point (si);
   switch (type)
     {
     case DUPLET:
@@ -745,10 +848,10 @@ dnm_inserttuplet (DenemoScore * si, tuplet_type type)
       break;
     }
   //g_print ("Cursor pos %d (Before tup open)\n", si->cursor_x);
-  object_insert (si, mudela_obj_new);
+  object_insert (gui, mudela_obj_new);
   //g_print ("Cursor pos %d (After tup open, before tup close)\n",	   si->cursor_x);
   /* Add the closing bracket */
-  object_insert (si, newtupclose ());
+  object_insert (gui, newtupclose ());
   //g_print ("Cursor pos %d (After tup close)\n", si->cursor_x);
   si->cursor_x--;
   //g_print ("Cursor pos %d( After move back)\n", si->cursor_x);
@@ -762,17 +865,18 @@ dnm_inserttuplet (DenemoScore * si, tuplet_type type)
  * @param si pointer to the scoreinfo structure
  */
 void
-insertgrace (DenemoScore * si)
+insertgrace (DenemoGUI * gui)
 {
+  DenemoScore *si = gui->si;
   DenemoObject *mudela_obj_new;
-  nextmeasure (si, FALSE);
+  insertion_point (si);
 
 
   mudela_obj_new = newgracestart ();
 
-  object_insert (si, mudela_obj_new);
+  object_insert (gui, mudela_obj_new);
 
-  object_insert (si, newgraceend ());
+  object_insert (gui, newgraceend ());
   si->cursor_x--;
   si->currentobject =
     g_list_nth ((objnode *) si->currentmeasure->data, si->cursor_x);
@@ -863,8 +967,9 @@ displayhelper (DenemoGUI * gui)
  */
 
 void
-incrementenshift (DenemoScore * si, gint direction)
+incrementenshift (DenemoGUI * gui, gint direction)
 {
+  DenemoScore *si = gui->si;
   declarecurmudelaobj;
   int prognum;
   DenemoStaff *curstaffstruct;
@@ -892,6 +997,7 @@ incrementenshift (DenemoScore * si, gint direction)
       data->staffnum = si->currentstaffnum;
       data->action = ACTION_CHANGE;
       update_undo_info (si, data);
+      score_status(gui, TRUE);
     }
 }
 /**
@@ -1066,11 +1172,7 @@ delete_staff_before (GtkAction * action, DenemoGUI * gui)
 {
   DenemoScore *si = gui->si;
   if (staffup(gui)) {
-    deletestaff (si);
-      setcurrents (si);
-      find_xes_in_all_measures (si);
-      si->markstaffnum = 0;
-    displayhelper (gui);
+    deletestaff (gui, TRUE);
   }
 }
 
@@ -1085,11 +1187,7 @@ delete_staff_after (GtkAction * action, DenemoGUI * gui)
 {
   DenemoScore *si = gui->si;
   if (staffdown(gui)) {
-    deletestaff (si);
-      setcurrents (si);
-      find_xes_in_all_measures (si);
-      si->markstaffnum = 0; 
-    displayhelper (gui);
+    deletestaff (gui, TRUE);
   }
 }
 
@@ -1103,11 +1201,7 @@ void
 delete_staff_current (GtkAction * action, DenemoGUI * gui)
 {
   DenemoScore *si = gui->si;
-  deletestaff (si);
-      setcurrents (si);
-      find_xes_in_all_measures (si);
-      si->markstaffnum = 0;
-  displayhelper (gui);
+  deletestaff (gui, TRUE);
 }
 
 
@@ -1141,7 +1235,6 @@ void deletemeasureallstaffs(DenemoGUI * gui)
   setcurrents (si);
   score_status(gui, TRUE);
   si->markstaffnum = 0;
-  
   isoffleftside (gui);
   displayhelper(gui);
 }
@@ -1180,10 +1273,12 @@ dnm_deletemeasure (DenemoScore * si)
 static void
 remove_object (measurenode * cur_measure, objnode * cur_objnode)
 {
-  cur_measure->data = g_list_remove_link ((objnode *) cur_measure->data,
+  if(cur_measure->data) {
+    cur_measure->data = g_list_remove_link ((objnode *) cur_measure->data,
 					  cur_objnode);
-  freeobject ((DenemoObject *) cur_objnode->data);
-  g_list_free_1 (cur_objnode);
+    freeobject ((DenemoObject *) cur_objnode->data);
+    g_list_free_1 (cur_objnode);
+  }
 }
 
 
@@ -1252,6 +1347,10 @@ dnm_deleteobject (DenemoScore * si)
     }
   }
 
+  if(curmudelaobj->type==LILYDIRECTIVE && ((lilydirective *)curmudelaobj->object)->locked)
+    if(!confirm("This LilyPond insert is locked","Really delete it?"))
+      return;
+
 
   if (si->undo_redo_mode == UNDO)
     {
@@ -1304,8 +1403,10 @@ dnm_deleteobject (DenemoScore * si)
 	    {
 	      curmeasure = g_list_nth (firstmeasurenode (curstaff),
 				       si->currentmeasurenum - 1);
-	      remove_object (curmeasure, (objnode *) curmeasure->data);
-	      beamsandstemdirswholestaff ((DenemoStaff *) curstaff->data);
+	      if(curmeasure){
+		remove_object (curmeasure, (objnode *) curmeasure->data);
+		beamsandstemdirswholestaff ((DenemoStaff *) curstaff->data);
+	      }
 	    }
 	  reset_cursor_stats (si);
 	  find_xes_in_all_measures (si);
@@ -1349,12 +1450,23 @@ dnm_deleteobject (DenemoScore * si)
  * @return none
  */
 void
-insertclone (DenemoScore * si)
+insertclone (DenemoGUI *gui)
 {
+  DenemoScore *si = gui->si;
   declarecurmudelaobj;
 
   if (curmudelaobj && curmudelaobj->type == CHORD)
-    object_insert (si, dnm_clone_object (curmudelaobj));
+    object_insert (gui, dnm_clone_object (curmudelaobj));
+}
+
+
+void tolastobject (DenemoGUI *gui)
+{
+  while (gui->si->currentobject && (gui->si->currentobject->next))
+    {
+      gui->si->currentobject = gui->si->currentobject->next;
+      gui->si->cursor_x++;
+    }
 }
 
 /**
@@ -1370,6 +1482,8 @@ toend (GtkAction * action, DenemoGUI * gui)
     gui->si->rightmeasurenum =
     g_list_length (((DenemoStaff *) gui->si->currentstaff->data)->measures);
   setcurrents (gui->si);
+  tolastobject(gui);
+  cursorright(gui);
   find_leftmost_allcontexts (gui->si);
   update_hscrollbar (gui);
   displayhelper (gui);
@@ -1405,7 +1519,7 @@ tohome (GtkAction * action, DenemoGUI * gui)
 void
 stem_directive_insert (GtkAction * action, DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_stem_directive_new (DENEMO_STEMBOTH));
+  object_insert (gui, dnm_stem_directive_new (DENEMO_STEMBOTH));
   /* This sets beams and stem directions in the measure, but that's
    * not sufficient */
 
@@ -1612,7 +1726,7 @@ auto_save_document_timeout (DenemoGUI *gui)
   //g_print ("Auto save file name %s\n", gui->autosavename->str);
   if (si->lily_file)
     {
-      exportlilypond (gui->autosavename->str, gui, 0, 0, 1);
+      exportlilypond (gui->autosavename->str, gui, TRUE);
     }
   else
     {
