@@ -37,6 +37,11 @@ measureright (DenemoGUI  *gui);
 
 gboolean
 swapstaffs (GtkAction *action, DenemoGUI  *gui);
+gboolean
+splitstaffs (GtkAction *action, DenemoGUI  *gui);
+gboolean
+joinstaffs (GtkAction *action, DenemoGUI  *gui);
+
 gboolean
 swapmovements (GtkAction *action, DenemoGUI  *gui);
 
@@ -46,6 +51,12 @@ staffup (DenemoGUI  *gui);
 gboolean
 staffdown (DenemoGUI  *si);
 
+gboolean
+voiceup (DenemoGUI  *si);
+
+gboolean
+voicedown (DenemoGUI  *si);
+
 void
 cursorleft (DenemoGUI  *gui);
 
@@ -71,7 +82,7 @@ void
 tonechange (DenemoScore  *si, gboolean remove);
 
 void
-incrementenshift (DenemoScore  *si, gint direction);
+incrementenshift (DenemoGUI  *gui, gint direction);
 
 void
 change_stem_directive (DenemoScore *si, enum stemdirections amount);
@@ -84,7 +95,7 @@ void
 appendmeasures (DenemoScore  *si, gint number);
 
 void
-insertclone (DenemoScore  *si);
+insertclone (DenemoGUI  *gui);
 
 void
 toend (GtkAction *action, DenemoGUI  *gui);
@@ -97,7 +108,7 @@ void
 inserttuplet (DenemoScore  *si, tuplet_type type);
 
 void
-object_insert (DenemoScore  *si, DenemoObject *mudela_obj_new);
+object_insert (DenemoGUI  *gui, DenemoObject *mudela_obj_new);
 
 void
 stem_directive_insert (GtkAction *action, DenemoGUI *gui);
@@ -108,7 +119,7 @@ toggle_begin_slur (DenemoGUI *gui);
 void
 toggle_end_slur (DenemoGUI  *gui);
 
-void insertgrace(DenemoScore  *si);
+void insertgrace(DenemoGUI  *gui);
 
 void
 toggle_start_crescendo (DenemoGUI *gui);
@@ -122,7 +133,7 @@ toggle_start_diminuendo (DenemoGUI *gui);
 void
 toggle_end_diminuendo (DenemoGUI *gui);
 
-void nextmeasure(DenemoScore  *si, gboolean all);
+void insertion_point (DenemoScore  *si);
 
 void
 caution (DenemoScore  *si);
