@@ -184,37 +184,7 @@ determineClef (gint type, gchar ** clefName)
     }
 }
 
-/**
- * Determine the staff context as it should
- * be written to the XML file
- */
-static gchar *
-determinestaffcontext (DenemoContext context)
-{
-  gchar *tmp;
-  switch (context)
-    {
-    case DENEMO_NONE:
-      tmp = "Staff";
-      break;
 
-    case DENEMO_PIANO:
-      tmp = "PianoStaff";
-      break;
-
-    case DENEMO_GROUP:
-      tmp = "StaffGroup";
-      break;
-
-    case DENEMO_CHOIR:
-      tmp = "ChoirStaff";
-      break;
-    default:
-      tmp = "";
-      break;
-    }
-  return tmp;
-}
 
 /**
  * Determine the duration as it should be written to the XML file.  duration
@@ -520,11 +490,11 @@ newXMLNoteHead (xmlNodePtr parent, xmlNsPtr ns, enum headtype noteHeadType)
  * Denemo XML file to the given file.
  */
 void
-exportXML (gchar * thefilename, DenemoScore * si, gint start, gint end)
+exportXML (gchar * thefilename, DenemoGUI *gui, gint start, gint end)
 {
   GString *filename = g_string_new (thefilename);
   xmlDocPtr doc;
-  xmlNodePtr scoreElem, stavesElem, voicesElem, voiceElem;
+  xmlNodePtr scoreElem, mvmntElem, stavesElem, voicesElem, voiceElem;
   xmlNodePtr measuresElem, measureElem, objElem, prevTieElem;
   xmlNodePtr curElem, parentElem;
   xmlNsPtr ns;
@@ -564,69 +534,83 @@ exportXML (gchar * thefilename, DenemoScore * si, gint start, gint end)
   doc->xmlRootNode = scoreElem =
     xmlNewDocNode (doc, NULL, (xmlChar *) "score", NULL);
   ns = xmlNewNs (doc->xmlRootNode, (xmlChar *) DENEMO_XML_NAMESPACE, NULL);
-  xmlSetProp (scoreElem, (xmlChar *) "version", (xmlChar *) "1.0");
+  xmlSetProp (scoreElem, (xmlChar *) "version", (xmlChar *) "2.0");
   /* FIXME: Put comment here ("Denemo XML file generated by..."). */
-
-  /* Output the score-info element and its children. */
-
-  parentElem = xmlNewChild (scoreElem, ns, (xmlChar *) "paper-info", NULL);
-  xmlNewChild (parentElem, ns, (xmlChar *) "lilypondversion",
-	       si->sconfig->lilyversion->str);
-  newXMLIntChild (parentElem, ns, (xmlChar *) "fontsize",
-		  si->sconfig->fontsize);
-  xmlNewChild (parentElem, ns, (xmlChar *) "papersize",
-	       si->sconfig->papersize->str);
-
-  newXMLIntChild (parentElem, ns, (xmlChar *) "orientation",
-		  si->sconfig->orientation);
+  /* lilycontrol for the whole musical score */
 
 
-  parentElem = xmlNewChild (scoreElem, ns, (xmlChar *) "edit-info", NULL);
+ parentElem = xmlNewChild (scoreElem, ns, (xmlChar *) "lilycontrol", NULL);
+#define NEWCHILD(field) if(gui->lilycontrol.field->len) \
+                       xmlNewChild (parentElem, ns, (xmlChar *) #field,\
+                      (xmlChar *) gui->lilycontrol.field->str)
+ NEWCHILD(papersize);
+ NEWCHILD(lilyversion);
+ NEWCHILD(lilypond);
+ newXMLIntChild (parentElem, ns, (xmlChar *) "fontsize",  gui->lilycontrol.fontsize);
+ newXMLIntChild (parentElem, ns, (xmlChar *) "orientation",  gui->lilycontrol.orientation);
+   GList *custom;
+   for(custom=g_list_last(gui->custom_scoreblocks);custom;custom=custom->prev) {
+     xmlNewChild (scoreElem, ns, "custom_scoreblock", (xmlChar *)((GString*)(((DenemoScoreblock*)custom->data)->scoreblock)->str));
+     if(((DenemoScoreblock*)custom->data)->visible)
+       xmlNewChild (scoreElem, ns, "visible_scoreblock", NULL);
+   }
+  GList *g;
+  for(g=gui->movements;g;g=g->next) {
+    DenemoScore *si = g->data;
+
+  mvmntElem = xmlNewChild (scoreElem, ns, (xmlChar *) "movement", NULL);
+  parentElem = xmlNewChild (mvmntElem, ns, (xmlChar *) "edit-info", NULL);
   newXMLIntChild (parentElem, ns, (xmlChar *) "staffno", si->currentstaffnum);
   newXMLIntChild (parentElem, ns, (xmlChar *) "cursorposition",
 		  si->cursor_x - 1);
 
 
-  parentElem = xmlNewChild (scoreElem, ns, (xmlChar *) "score-info", NULL);
+  parentElem = xmlNewChild (mvmntElem, ns, (xmlChar *) "score-info", NULL);
   curElem = xmlNewChild (parentElem, ns, (xmlChar *) "tempo", NULL);
   newXMLFraction (xmlNewChild (curElem, ns, (xmlChar *) "duration", NULL), ns,
 		  1, 4);
   newXMLIntChild (curElem, ns, (xmlChar *) "bpm", si->tempo);
   xmlNewChild (parentElem, ns, (xmlChar *) "title",
-	       (xmlChar *) si->headerinfo->title->str);
+	       (xmlChar *) si->headerinfo.title->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "subtitle",
-	       (xmlChar *) si->headerinfo->subtitle->str);
+	       (xmlChar *) si->headerinfo.subtitle->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "composer",
-	       (xmlChar *) si->headerinfo->composer->str);
+	       (xmlChar *) si->headerinfo.composer->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "poet",
-	       (xmlChar *) si->headerinfo->poet->str);
+	       (xmlChar *) si->headerinfo.poet->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "meter",
-	       (xmlChar *) si->headerinfo->meter->str);
+	       (xmlChar *) si->headerinfo.meter->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "opus",
-	       (xmlChar *) si->headerinfo->opus->str);
+	       (xmlChar *) si->headerinfo.opus->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "arranger",
-	       (xmlChar *) si->headerinfo->arranger->str);
+	       (xmlChar *) si->headerinfo.arranger->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "instrument",
-	       (xmlChar *) si->headerinfo->instrument->str);
+	       (xmlChar *) si->headerinfo.instrument->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "dedication",
-	       (xmlChar *) si->headerinfo->dedication->str);
+	       (xmlChar *) si->headerinfo.dedication->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "piece",
-	       (xmlChar *) si->headerinfo->piece->str);
+	       (xmlChar *) si->headerinfo.piece->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "head",
-	       (xmlChar *) si->headerinfo->head->str);
+	       (xmlChar *) si->headerinfo.head->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "copyright",
-	       (xmlChar *) si->headerinfo->copyright->str);
+	       (xmlChar *) si->headerinfo.copyright->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "footer",
-	       (xmlChar *) si->headerinfo->footer->str);
+	       (xmlChar *) si->headerinfo.footer->str);
   xmlNewChild (parentElem, ns, (xmlChar *) "tagline",
-	       (xmlChar *) si->headerinfo->tagline->str);
+	       (xmlChar *) si->headerinfo.tagline->str);
+  xmlNewChild (parentElem, ns, (xmlChar *) "extra",
+	       (xmlChar *) si->headerinfo.extra->str);
+  xmlNewChild (parentElem, ns, (xmlChar *) "markup_before",
+	       (xmlChar *) si->headerinfo.lilypond_before->str);
+  xmlNewChild (parentElem, ns, (xmlChar *) "markup_after",
+	       (xmlChar *) si->headerinfo.lilypond_after->str);
 
   /* Output each (primary) staff, and store the IDs in a hash table. */
   fraction = 1.0 / (gdouble) g_list_length (si->thescore);
   fraction /= 3;
 	
 	
-  stavesElem = xmlNewChild (scoreElem, ns, (xmlChar *) "staves", NULL);
+  stavesElem = xmlNewChild (mvmntElem, ns, (xmlChar *) "staves", NULL);
   for (curStaff = si->thescore; curStaff != NULL; curStaff = curStaff->next)
     {
       curStaffStruct = (DenemoStaff *) curStaff->data;
@@ -644,23 +628,51 @@ exportXML (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			  curStaffStruct->transposition);
 	  xmlNewChild (curElem, ns, (xmlChar *) "instrument",
 		       (xmlChar *) curStaffStruct->midi_instrument->str);
-	  xmlNewChild (curElem, ns, (xmlChar *) "context",
-		       (xmlChar *) determinestaffcontext (curStaffStruct->
-							  context));
+	  newXMLIntChild (curElem, ns, (xmlChar *) "midi_prognum_override",
+	               curStaffStruct->midi_prognum_override);
+	  newXMLIntChild (curElem, ns, (xmlChar *) "midi_prognum",
+	               curStaffStruct->midi_prognum);
+	  newXMLIntChild (curElem, ns, (xmlChar *) "midi_channel",
+	               curStaffStruct->midi_channel);
+	  
+
+#define PUTCHILD(A,B) if(curStaffStruct->context & A) xmlNewChild (curElem, ns, (xmlChar *) "context", B);
+          PUTCHILD(DENEMO_PIANO_START, PIANO_START_STRING);
+          PUTCHILD(DENEMO_PIANO_END, PIANO_END_STRING);
+          PUTCHILD(DENEMO_CHOIR_START, CHOIR_START_STRING);
+          PUTCHILD(DENEMO_CHOIR_END, CHOIR_END_STRING);
+          PUTCHILD(DENEMO_GROUP_START, GROUP_START_STRING);
+          PUTCHILD(DENEMO_GROUP_END, GROUP_END_STRING);
+#undef PUTCHILD
 	  newXMLIntChild (curElem, ns, (xmlChar *) "space_above",
 			  curStaffStruct->space_above);
 	  newXMLIntChild (curElem, ns, (xmlChar *) "space_below",
 			  curStaffStruct->space_below);
 	  newXMLIntChild (curElem, ns, (xmlChar *) "hasfigures",
 			  curStaffStruct->hasfigures);
-	  xmlNewChild (curElem, ns, (xmlChar *) "lilybefore",
-		       (xmlChar *) curStaffStruct->lilybefore);
+	  newXMLIntChild (curElem, ns, (xmlChar *) "haslyrics",
+			  curStaffStruct->haslyrics);
+	  newXMLIntChild (curElem, ns, (xmlChar *) "hasfakechords",
+			  curStaffStruct->hasfakechords);
+	  
+	  if(curStaffStruct->staff_prolog && curStaffStruct->staff_prolog->len)
+	     xmlNewChild (curElem, ns, (xmlChar *) "staff-prolog",
+		       (xmlChar *) curStaffStruct->staff_prolog->str);
+	  if(curStaffStruct->lyrics_prolog && curStaffStruct->lyrics_prolog->len)
+	     xmlNewChild (curElem, ns, (xmlChar *) "lyrics-prolog",
+		       (xmlChar *) curStaffStruct->lyrics_prolog->str);
+	  if(curStaffStruct->figures_prolog && curStaffStruct->figures_prolog->len)
+	     xmlNewChild (curElem, ns, (xmlChar *) "figures-prolog",
+		       (xmlChar *) curStaffStruct->figures_prolog->str);
+	  if(curStaffStruct->fakechords_prolog && curStaffStruct->fakechords_prolog->len)
+	     xmlNewChild (curElem, ns, (xmlChar *) "fakechords-prolog",
+		       (xmlChar *) curStaffStruct->fakechords_prolog->str);
 	}
     }
 
   /* Output each voice. */
 
-  voicesElem = xmlNewChild (scoreElem, ns, (xmlChar *) "voices", NULL);
+  voicesElem = xmlNewChild (mvmntElem, ns, (xmlChar *) "voices", NULL);
   for (curStaff = si->thescore; curStaff != NULL; curStaff = curStaff->next)
     {
       curStaffStruct = (DenemoStaff *) curStaff->data;
@@ -724,7 +736,7 @@ exportXML (gchar * thefilename, DenemoScore * si, gint start, gint end)
       newXMLTimeSignature (parentElem, ns, curTime1, curTime2);
 
       /* Write out the measures. */
-
+      gboolean warning_given = FALSE;//No warning yet about empty measures
       measuresElem =
 	xmlNewChild (voiceElem, ns, (xmlChar *) "measures", NULL);
       for (curMeasure = curStaffStruct->measures; curMeasure != NULL;
@@ -1115,6 +1127,10 @@ exportXML (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			      newXMLNoteHead (curElem, ns,
 					      curNote->noteheadtype);
 			    }
+			  if(curNote->directive && curNote->directive->len)
+			     xmlNewChild (curElem, ns,
+					  (xmlChar *) "directive",
+					  (xmlChar *)curNote->directive->str);
 			}
 		    }
 
@@ -1270,6 +1286,8 @@ exportXML (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			xmlNewChild (measureElem, ns, (xmlChar *) "lily-directive",
 				     (xmlChar *) ((lilydirective *) curObj->object)->
 				     directive->str);
+		  xmlSetProp (objElem, (xmlChar *) "locked",
+			      (xmlChar *) (((lilydirective *) curObj->object)->locked?"true":"false"));
 
 		  break;
 		case BARLINE:
@@ -1287,12 +1305,18 @@ exportXML (gchar * thefilename, DenemoScore * si, gint start, gint end)
 
 	  if (emptyMeasure)
 	    {
+#if 0
 	      curElem =
 		xmlNewChild (measureElem, ns, (xmlChar *) "rest", NULL);
 	      xmlSetProp (curElem, (xmlChar *) "show", (xmlChar *) "false");
 	      newXMLFraction (xmlNewChild
 			      (curElem, ns, (xmlChar *) "duration", NULL), ns,
 			      curTime1, curTime2);
+	      if(!warning_given)
+		if(strcmp(filename->str, gui->autosavename->str))
+		warningdialog("Empty measure - putting a non-printing rest into it");
+	      warning_given = TRUE;
+#endif
 	    }
 	}			/* end for each measure in voice */
 
@@ -1303,7 +1327,7 @@ exportXML (gchar * thefilename, DenemoScore * si, gint start, gint end)
       g_list_free (diminElemStack);
       g_free (lastBeamStartXMLID);
     }				/* end for each voice in score */
-
+  }// for each movement
   /* Save the file. */
 
   if (xmlSaveFile (filename->str, doc) < 0)
