@@ -5,10 +5,8 @@
 
 #include <denemo/denemo.h>
 
-//void 
-//insertfigure(GtkWidget *widget, gpointer data);
 void
-seperate_fakechord_elements (gchar *fakechord, DenemoObject *curObj);
+separate_fakechord_elements (gchar *fakechord, DenemoObject *curObj);
 
 void 
 fakechord_insert(GtkAction *action, DenemoGUI *gui);
