@@ -14,8 +14,6 @@
 #include "exportabc.h"
 #include "exportlilypond.h"
 #include "file.h"
-#include "frogio.h"
-#include "frogdefs.h"
 #include "moveviewport.h"
 #include "staffops.h"
 #include "scoreops.h"
@@ -48,10 +46,9 @@ typedef enum
 { DENEMO_FORMAT = 0,
   DNM_FORMAT,
   MUDELA_FORMAT,
+  PNG_FORMAT,
   ABC_FORMAT,
-  JTF_FORMAT,
   MIDI_FORMAT,
-  JTF_POLY_FORMAT,
   CSOUND_FORMAT
 }
 FileFormatNames;
@@ -81,10 +78,9 @@ static struct FileFormatData supported_export_file_formats[] = {
   {"*.denemo", N_("Denemo XML format (*.denemo)"), ".denemo"},
   {"*.dnm", N_("Denemo XML format (*.dnm)"), ".dnm"},
   {"*.ly", N_("Lilypond (*.ly)"), ".ly"},
+  {"*.png", N_("png image format (*.png)"), ".png"},
   {"*.abc", N_("ABC (*.abc)"), ".abc"},
-  {"*.jtf", N_("Unnamed file format (*.jtf)"), ".jtf"},
   {"*.mid", N_("Midi (*.mid)"), ".mid"},
-  {"*.jtf", N_("Unnamed file format (*.jtf) Poly"), ".jtf"},
   {"*.sco", N_("CSound Score File (*.sco)"), ".sco"}
 };
 
@@ -106,6 +102,7 @@ struct callbackdata
 /* directory last used for saving */
 static gchar *file_selection_path = NULL;
 static gchar *system_template_path = NULL;
+static gchar *system_example_path = NULL;
 static gchar *local_template_path = NULL;
 static gchar *default_template_path = NULL;
 
@@ -230,7 +227,7 @@ gint
 open_for_real (gchar * filename, DenemoGUI * gui, gboolean template, ImportType type)
 {
   gint result;
-
+  result = 1;//FAILURE
   if (strcmp (filename + strlen (filename) - 7, ".denemo") == 0)
     result = importXML (filename, gui, type);
   else if (strcmp (filename + strlen (filename) - 4, ".dnm") == 0)
@@ -240,8 +237,6 @@ open_for_real (gchar * filename, DenemoGUI * gui, gboolean template, ImportType
   else if (strcmp (filename + strlen (filename) - 4, ".mid") == 0 ||
 	   strcmp (filename + strlen (filename) - 5, ".midi") == 0)
     result = importMidi (filename, gui);
-  else if (strcmp (filename + strlen (filename) - 4, ".jtf") == 0)
-    result = froginput (filename, gui->si);
   if (result == 0)
     {
       if(!template) {// not a template
@@ -259,6 +254,8 @@ open_for_real (gchar * filename, DenemoGUI * gui, gboolean template, ImportType
       gtk_widget_queue_draw (gui->scorearea);
       gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
       gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
+      gui->lilysync = G_MAXUINT;//FIXME move these two lines into a function, they force refresh of lily text
+      refresh_lily_cb(NULL, gui);
     }
   return result;
 }
@@ -328,8 +325,9 @@ create_filename (const gchar * file_name, gint *format_id)
 }
 
 /**
- * File save callback called by fileselsave callback
- * param file_name is full path to file with extension
+ * File save called by fileselsave callback
+ * param file_name is full path to file possibly with extension
+ * FIXME duplicate code with file_save
  */
 static void
 filesel_save (DenemoGUI * gui, const gchar * file_name, gint format_id, gboolean template)
@@ -340,10 +338,9 @@ filesel_save (DenemoGUI * gui, const gchar * file_name, gint format_id, gboolean
 	    (int) G_N_ELEMENTS (supported_export_file_formats));
 
   DenemoScore *si = gui->si;
-  // Append file extension extension
+  // Append file extension if needed
   gchar *file = NULL;
   gchar *basename = NULL;
-
   file = create_filename(file_name, &format_id);
 #ifdef DEBUG
   g_print("Saving to file %s", file);
@@ -367,29 +364,26 @@ filesel_save (DenemoGUI * gui, const gchar * file_name, gint format_id, gboolean
 	  };
 	case MUDELA_FORMAT:
 	  {
-	    exportlilypond (file, gui, 0, 0, 1);
+	    gui->si->markstaffnum = 0;
+	    exportlilypond (file, gui, TRUE);
 	    break;
 	  };
+	case PNG_FORMAT:
+	  {
+	    gui->lilycontrol.excerpt = TRUE;
+            exportlilypond (file, gui, TRUE);
+	    break;
+	  }
 	case ABC_FORMAT:
 	  {
 	    exportabc (file, gui, 0, 0);
 	    break;
 	  };
-	case JTF_FORMAT:
-	  {
-	    filesave (file, si, 0, 0, 0);
-	    break;
-	  };
 	case MIDI_FORMAT:
 	  {
 	    exportmidi (file, si, 0, 0);
 	    break;
 	  };
-	case JTF_POLY_FORMAT:
-	  {
-	    filesave (file, si, 0, 0, 1);
-	    break;
-	  };
 	case CSOUND_FORMAT:
 	  {
 	    exportcsound (file, si, 0, 0);
@@ -401,11 +395,13 @@ filesel_save (DenemoGUI * gui, const gchar * file_name, gint format_id, gboolean
 	
 	/*export parts as lilypond files*/
 	if(Denemo.prefs.saveparts)
-		export_lilypond_parts(file,gui,0,0);
-	
-	
-      score_status(gui, FALSE);
-      si->readonly = FALSE;
+		export_lilypond_parts(file,gui);
+	if(gui->lilysync==gui->changecount)
+	  gui->lilysync = 0;//still in sync
+	gui->changecount = 0;
+	score_status(gui, FALSE);
+
+	si->readonly = FALSE;
     }
   g_free(basename);
   g_free(file);
@@ -424,21 +420,37 @@ init_local_path(void) {
 	}
 }
 
+typedef enum {
+  LOCAL,
+  SYSTEM,
+  EXAMPLE
+} TemplateType;
+
 /*
  * Sets the file_selection_path to the templates directory and 
  * calls file_open to create the file selection dialog
- * LOCAL whether to use the local templates or systemwide ones
+ * LOCAL whether to use the local templates or systemwide templates or examples
  * does nothing if unable to access templates
  */
 static void
-template_open (DenemoGUI * gui, gboolean local)
+template_open (DenemoGUI * gui, TemplateType local)
 {
-  if(local) {
+  if(local==LOCAL) {
     if(local_template_path==NULL) {
       init_local_path();
     }
     default_template_path = local_template_path;   
-  } else { 
+  } else if(local==EXAMPLE){
+    if(system_example_path==NULL) {
+      system_example_path = g_build_filename (get_data_dir (), "examples", NULL);
+      GDir *denemo_path = g_dir_open(system_example_path, 0, NULL);
+      if(denemo_path == NULL) {
+	warningdialog ("No examples directory in installation");
+	system_example_path = NULL;
+      }
+    }
+    default_template_path = system_example_path;
+  } else{ 
     if(system_template_path==NULL) {
       system_template_path = g_build_filename (get_data_dir (), "templates", NULL);
       GDir *denemo_path = g_dir_open(system_template_path, 0, NULL);
@@ -463,16 +475,34 @@ template_open (DenemoGUI * gui, gboolean local)
  */
 void
 system_template_open_with_check (GtkAction * action, DenemoGUI * gui) {
-  if (gui->haschanged)
+  if (gui->changecount)
     {
       if (confirmbox (gui))
 	{
-	  template_open (gui, FALSE);
+	  template_open (gui, SYSTEM);
 	}
     }
   else
     {
-      template_open (gui, FALSE);
+      template_open (gui, SYSTEM);
+    }
+}
+
+/*
+ * Open system template file callback function 
+ */
+void
+system_example_open_with_check (GtkAction * action, DenemoGUI * gui) {
+  if (gui->changecount)
+    {
+      if (confirmbox (gui))
+	{
+	  template_open (gui, EXAMPLE);
+	}
+    }
+  else
+    {
+      template_open (gui, EXAMPLE);
     }
 }
 /*
@@ -480,16 +510,16 @@ system_template_open_with_check (GtkAction * action, DenemoGUI * gui) {
  */
 void
 local_template_open_with_check (GtkAction * action, DenemoGUI * gui) {
-  if (gui->haschanged)
+  if (gui->changecount)
     {
       if (confirmbox (gui))
 	{
-	  template_open (gui, TRUE);
+	  template_open (gui, LOCAL);
 	}
     }
   else
     {
-      template_open (gui, TRUE);
+      template_open (gui, LOCAL);
     }
 }
 
@@ -502,15 +532,17 @@ local_template_open_with_check (GtkAction * action, DenemoGUI * gui) {
 void
 file_open_with_check (GtkAction * action, DenemoGUI * gui)
 {
-  if (gui->haschanged)
+  if (gui->changecount)
     {
       if (confirmbox (gui))
 	{
+	  deletescore (NULL, gui);
 	  file_open (gui, FALSE, REPLACE_SCORE);
 	}
     }
   else
     {
+      deletescore (NULL, gui);
       file_open (gui, FALSE, REPLACE_SCORE);
     }
 }
@@ -522,6 +554,8 @@ file_open_with_check (GtkAction * action, DenemoGUI * gui)
  */
 void
 file_add_movements(GtkAction * action, DenemoGUI * gui){
+  if(!confirm_insertstaff_custom_scoreblock(gui))
+    return;
   file_open(gui, FALSE, ADD_MOVEMENTS);
   score_status(gui, TRUE);
 }
@@ -531,6 +565,8 @@ file_add_movements(GtkAction * action, DenemoGUI * gui){
  */
 void
 file_add_staffs(GtkAction * action, DenemoGUI * gui){
+  if(!confirm_insertstaff_custom_scoreblock(gui))
+    return;
   file_open(gui, FALSE, ADD_STAFFS);
   score_status(gui, TRUE);
 }
@@ -671,7 +707,8 @@ file_save (GtkWidget * widget, DenemoGUI * gui)
 	};
       case MUDELA_FORMAT:
 	{
-	  exportlilypond (gui->filename->str, gui, 0, 0, 1);
+	  gui->lilycontrol.excerpt = TRUE;
+	  exportlilypond (gui->filename->str, gui, TRUE);
 	  break;
 	};
       case ABC_FORMAT:
@@ -679,16 +716,6 @@ file_save (GtkWidget * widget, DenemoGUI * gui)
 	  exportabc (gui->filename->str, gui, 0, 0);
 	  break;
 	};
-      case JTF_POLY_FORMAT:
-	{
-	  filesave (gui->filename->str, si, 0, 0, 1);
-	  break;
-	};
-      case JTF_FORMAT:
-	{
-	  filesave (gui->filename->str, si, 0, 0, 0);
-	  break;
-	};
       case MIDI_FORMAT:
 	{
 	  exportmidi (gui->filename->str, si, 0, 0);
@@ -708,9 +735,12 @@ file_save (GtkWidget * widget, DenemoGUI * gui)
       };
    /*Save parts as lilypond files*/   
    if(Denemo.prefs.saveparts)
-	export_lilypond_parts(gui->filename->str,gui,0,0);
+	export_lilypond_parts(gui->filename->str,gui);
   
   denemo_warning (gui, guess_file_format (gui->filename->str));
+  if(gui->lilysync==gui->changecount)
+    gui->lilysync = 0;//still in sync
+  gui->changecount = 0;
   score_status(gui, FALSE);
 }
 
@@ -823,37 +853,28 @@ file_saveas (DenemoGUI * gui, gboolean template)
 void
 file_newwrapper (GtkAction * action, DenemoGUI * gui)
 {
-  if (gui->haschanged)
+  if (gui->changecount)
     {
       if (confirmbox (gui))
 	{
-	  deletescore (NULL, gui);
+	  deletescore(NULL, gui);
 	}
     }
   else
     {
-      deletescore (NULL, gui);
+      deletescore(NULL, gui);
     };
 }
 
 /**
  * Delete the given score and create a new one
+ * with one movement and empty music data
  *
  */
 void
 deletescore (GtkWidget * widget, DenemoGUI * gui)
 {
-  GList *g = gui->movements;
-  if(g==NULL)
-    free_score(gui->si);
-  else {
-    for(;g;g=g->next) {
-      free_score (g->data);
-      if(g->data!=gui->si)
-	g_free(g->data);
-    }
-  gui->movements = NULL;
-  }
+  free_gui(gui);
   score_status(gui, FALSE);
   if(gui->filename) {
     g_string_free(gui->filename, TRUE);
@@ -871,6 +892,8 @@ deletescore (GtkWidget * widget, DenemoGUI * gui)
   gtk_widget_queue_draw (gui->scorearea);
   gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
   gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
+  gui->lilysync = G_MAXUINT;
+  refresh_lily_cb(NULL, gui);
 }
 
 void
@@ -880,8 +903,8 @@ dnm_deletescore (GtkWidget * widget, DenemoGUI * gui){
 
 
 /**
- * Try to suggest the format of a given file, after its file name extension. A
- * more powerful function could be written to guess the format after the
+ * Try to suggest the format of a given file, from file name extension. A
+ * more powerful function could be written to guess the format from the
  * file contents 
  */
 gint
@@ -952,7 +975,7 @@ replace_existing_file_dialog (const gchar * filename,
   gboolean r = (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES);
   gtk_widget_destroy (dialog);
   g_free (file);
-  g_print ("Yes dialog is %d\n", r);
+  //g_print ("Yes dialog is %d\n", r);
   return r;
 }
 
@@ -968,5 +991,5 @@ file_savepartswrapper (GtkAction * action, DenemoGUI * gui)
       file_saveas (gui, FALSE);
     }
 
-  export_lilypond_parts (gui->filename->str, gui, 0, 0);
+  export_lilypond_parts (gui->filename->str, gui);
 }
