@@ -30,13 +30,13 @@ about (GtkAction * action, gpointer callback_data)
 				 _("The GNU graphical score editor"));
   gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (dialog), VERSION);
   gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (dialog),
-				"http://savannah.gnu.org/cvs/?group=denemo");
+				"http://denemo.org");
   gtk_about_dialog_set_website_label (GTK_ABOUT_DIALOG (dialog),
 				      _("Denemo website"));
   gtk_about_dialog_set_license (GTK_ABOUT_DIALOG (dialog),
 				_
 				("(c) 1999 - 2008 Matthew Hiller, Adam Tee, and others.\n\n\
-http://www.gnu.org/software/denemo/denemo.html\n\n\
+http://www.denemo.org\n\n\
 This program is licensed under the terms of the GNU\n\
 General Public License and is provided with absolutely\n\
 NO WARRANTY; see the file COPYING for details."));
