@@ -4,12 +4,11 @@
  * (c) 2003-2005 AJAnderson
  *
  */
-
-#include <denemo/denemo.h>
-#include "importmidi.h"
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
+#include <denemo/denemo.h>
+#include "importmidi.h"
 
 /* 	TODO
  *  
@@ -33,12 +32,11 @@ importMidi (gchar * filename, DenemoGUI * gui)
   midicallback *mididata = (midicallback *)g_malloc0(sizeof(midicallback));
   mididata->notestack = NULL;
   mididata->chordnotes = NULL;
-  mididata->si = gui->si;
+  mididata->gui = gui;
   mididata->PPQN = 200;
   mididata->bartime = 0;
   mididata->lastoff = 0;
-  mididata->trackplus = 0;
-  
+  mididata->track = 0;
   gint ret = 0;			// (-1 on failure)
   gint data = 0;
   FILE *fp = 0;
@@ -71,6 +69,7 @@ importMidi (gchar * filename, DenemoGUI * gui)
 #endif  
 	  readtrack (fp, mididata);
 	  tracks--;
+	  mididata->track++;
 	    };
 	};
       if (tracks == 0)
@@ -79,9 +78,8 @@ importMidi (gchar * filename, DenemoGUI * gui)
 
   fclose (fp);
     
-  g_list_foreach(mididata->final_list, addnote, mididata);
+  g_list_foreach(mididata->final_list, (GFunc) addnote, mididata);
   g_free(mididata);
-  ret = 1;
   return ret;
 }
 
@@ -154,7 +152,7 @@ readheader (FILE * fp, midicallback *mididata)
 void
 readtrack (FILE * fp, midicallback *mididata)
 {
-  DenemoScore *si = mididata->si; 
+  DenemoScore *si = mididata->gui->si; 
   gint stat;
   gint data1;
   gint data2;
@@ -164,20 +162,7 @@ readtrack (FILE * fp, midicallback *mididata)
   gint event = 0;
 
   tlength = readBytes (fp, 4);
-
-
-  if (mididata->trackplus != 0)
-    {
-      si->currentstaffnum++;
-      newstaff (si, ADDFROMLOAD, DENEMO_NONE);
-      si->currentstaff = g_list_last (si->thescore);
-      mididata->lastoff = 0;
-      si->cursor_x = 0;
-      mididata->bartime = 0;
-      si->currentmeasurenum = 1;
-
-    }
-
+ 
   while (tlength != 0)
     {
       dtime = readVariable (fp);
@@ -266,14 +251,14 @@ readtrack (FILE * fp, midicallback *mididata)
 	      {
 		data2 = readBytes (fp, 1);
 		tlength--;
-		donoteoff (mididata, data1, time);
+		donoteoff (mididata, (int *) data1, (int *) time);
 		break;
 	      }
 	    case 9:		/*note on */
 	      {
 		data2 = readBytes (fp, 1);
 		tlength--;
-		donoteon (mididata, data1, data2, time);
+		donoteon (mididata, (int *) data1, (int *) data2, (int *) time);
 		break;
 	      }
 	    default:
@@ -298,10 +283,6 @@ readtrack (FILE * fp, midicallback *mididata)
 	    };
 	}
     }
-  mididata->trackplus++;
-#ifdef DEBUG
-  printf ("\ntrackplus = %i\n", mididata->trackplus);
-#endif
 }
 
 /**
@@ -312,14 +293,12 @@ void
 dotimesig (FILE * fp, midicallback *mididata)
 {
   /*only does initial TS */
-  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->si->currentstaff->data;
+  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->gui->si->currentstaff->data;
 
   curstaffstruct->stime1 = readBytes (fp, 1);
   curstaffstruct->stime2 = (gint) pow (2, (readBytes (fp, 1)));
 
-
   mididata->barlength = mididata->PPQN * 4 * curstaffstruct->stime1 / curstaffstruct->stime2;
-
   readBytes (fp, 2);		/*skip last two characters */
 }
 
@@ -339,12 +318,10 @@ dokeysig (FILE * fp, midicallback *mididata)
   if (key > 7)
     key = key - 256;		/*get flat key num, see keysigdialog.cpp */
 
-  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->si->currentstaff->data;
+  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->gui->si->currentstaff->data;
   curstaffstruct->skey = key;
   curstaffstruct->skey_isminor = isminor;
   dnm_setinitialkeysig (curstaffstruct, key, isminor);
-
-
 }
 
 void
@@ -355,14 +332,13 @@ dotempo (FILE * fp, midicallback *mididata)
   while (n++ < 3)
     x = ((x & 0x007FFFFF) << 8) + (gint) readBytes (fp, 1);
   tempo = (gint) (6.0e7 / (double) x);
-  //printf("\ntempo = %i\n",tempo);
-  mididata->si->tempo = tempo;
+  mididata->gui->si->tempo = tempo;
 }
 
 void
 dotrackname (FILE * fp, midicallback *mididata, gint x)
 {
-  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->si->currentstaff->data;
+  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->gui->si->currentstaff->data;
   GString *temp = g_string_new(""); 
   while (x-- > 0)
 	temp = g_string_append_c(temp, (gchar) readBytes (fp, 1));
@@ -372,11 +348,10 @@ dotrackname (FILE * fp, midicallback *mididata, gint x)
   g_string_free (temp, FALSE);
 }
 
-
 void
 doinstrname (FILE * fp,  midicallback *mididata, gint x)
 {
-  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->si->currentstaff->data;
+  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->gui->si->currentstaff->data;
   GString *temp = g_string_new("");  
   while (x-- > 0)
 	temp = g_string_append_c(temp, (gchar) readBytes (fp, 1));
@@ -385,15 +360,12 @@ doinstrname (FILE * fp,  midicallback *mididata, gint x)
   g_string_free (temp, FALSE);
 }
 
-
 static nstack *
-stack (gint *pitch, gint *timeon, gint *duration, gint *measure)
+stack (gint *pitch, gint *timeon, gint *duration, gint *measure, gint *staffnum)
 {
   nstack *mystack = (nstack *)g_malloc0(sizeof(nstack));
   mystack->pitch = pitch;
   mystack->timeon = timeon;
-  mystack->duration = duration;
-  mystack->measure = measure;
   return mystack;
 }
 
@@ -402,7 +374,7 @@ stack (gint *pitch, gint *timeon, gint *duration, gint *measure)
  */
 
 void
-donoteon (midicallback *mididata, gint pitchon, gint attack, gint timeon)
+donoteon (midicallback *mididata, gint *pitchon, gint *attack, gint *timeon)
 {
   nstack *noteon;
   if (attack == 0)
@@ -412,59 +384,88 @@ donoteon (midicallback *mididata, gint pitchon, gint attack, gint timeon)
   else
     {
       /* add a note to the stack */
-      noteon = stack(pitchon,timeon,NULL,NULL);
+      noteon = stack(pitchon,timeon,NULL,NULL, NULL);
       mididata->notestack = g_list_append(mididata->notestack, noteon);
 #ifdef DEBUG
 	      printf ("\npitchon = %i timeon = %i\n", pitchon, timeon);
 #endif
     }
 }
-gint CompareNotes(GList *tmp, gint pitchoff)
+
+gint CompareNotes(GList *tmp, gint *pitchoff)
 {
         if ((((nstack *) &tmp->data)->pitch == pitchoff) &&
-               (((nstack *) &tmp->data)->duration == NULL))
+               (((nstack *) &tmp->data)->duration == (int) NULL))
                return 0;
         else
                return 1;
 }
 
-void NewFindChordTones(GList *list, midicallback *mididata){
-	if ((((nstack *) list->data)->timeon == ((nstack *) mididata->currentnote->data)->timeon) 
-		&& (((nstack *) list->data)->duration == ((nstack *) mididata->currentnote->data)->duration)
-		&& (((nstack *) list->data)->pitch != ((nstack *) mididata->currentnote->data)->pitch)){
-				mididata->chordnotes = g_list_append(mididata->chordnotes, (int *) ((nstack *) list->data)->pitch);			
-			//mididata->chordnotes = g_list_concat(mididata->chordnotes, list);
+void findchordtones(GList *list, midicallback *mididata){
+	/*list*/
+	gint *l_timeon = ((nstack *) list->data)->timeon;
+	gint *l_staffnum = ((nstack *) list->data)->staffnum;
+	gint *l_duration = ((nstack *) list->data)->duration;
+	gint *l_pitch = ((nstack *) list->data)->pitch;
+	/*current note*/
+	gint *c_timeon = ((nstack *) mididata->currentnote->data)->timeon;
+	gint *c_staffnum = ((nstack *) mididata->currentnote->data)->staffnum;
+	gint *c_duration = ((nstack *) mididata->currentnote->data)->duration;
+	gint *c_pitch = ((nstack *) mididata->currentnote->data)->pitch;
+        //1 2	
+	if ((int) l_timeon <= (int) c_timeon < ((int) l_timeon + (int) l_duration)){
+	  if ((l_timeon == c_timeon) /*check for notes that belong in the chord*/
+		&& (l_duration == c_duration)
+		&& (l_staffnum == c_staffnum)
+		&& (l_pitch != c_pitch)){
+				mididata->chordnotes = g_list_append(mididata->chordnotes, (int *) l_pitch);
+	  	printf("\n!!! Chord Tone !!!\n");
+	  }
+	  else if (((gint) l_timeon == (gint) c_timeon)
+			 && (l_pitch != c_pitch) 
+			 && ((gint) l_duration != (gint) c_duration)) 
+            printf("\n!!! possible voice !!!\n");
+	  else if (((gint) l_timeon < (gint) c_timeon)
+			 && ((((int) l_timeon + (int) l_duration) > (int) c_timeon)))
+		  printf("\n!!! possible voice !!!\n");
 	}
-	else 
-		g_printf("\nNo other chord notes\n");	
+
 }
 
-nstack *glist_to_note (GList *tmp) {
-	nstack *note;
-	  note->pitch = ((nstack *) &tmp->data)->pitch;
-	  note->measure = ((nstack *) &tmp->data)->measure;
-	  note->timeon = ((nstack *) &tmp->data)->timeon;
-	  note->duration = ((nstack *) &tmp->data)->duration;
-	return note;
+void findvoicesandchords(GList *list, midicallback *mididata){
+	/*list*/
+	gint *l_timeon = ((nstack *) list->data)->timeon;
+	gint *l_staffnum = ((nstack *) list->data)->staffnum;
+	gint *l_duration = ((nstack *) list->data)->duration;
+	gint *l_pitch = ((nstack *) list->data)->pitch;
+	/*current note*/
+	gint *c_timeon = ((nstack *) mididata->currentnote->data)->timeon;
+	gint *c_staffnum = ((nstack *) mididata->currentnote->data)->staffnum;
+	gint *c_duration = ((nstack *) mididata->currentnote->data)->duration;
+	gint *c_pitch = ((nstack *) mididata->currentnote->data)->pitch;
+	//0 3 / 1 4
 }
+
+
 /**
  * Process note off command
  */
 void
-donoteoff (midicallback *mididata, gint pitchoff, gint timeoff)
+donoteoff (midicallback *mididata, gint *pitchoff, gint *timeoff)
 {
   gint duration = 0;
   gint starttime = 0;
   GList *tmp = NULL;
-  tmp = g_list_find_custom (mididata->notestack, pitchoff, (GCompareFunc) CompareNotes);
+  tmp = g_list_find_custom (mididata->notestack, pitchoff, (GCompareFunc) (int) CompareNotes);
   if (tmp != NULL)
-      	starttime = ((nstack *) tmp->data)->timeon;
+      	starttime = (int) ((nstack *) tmp->data)->timeon;
       	duration = ((gint) timeoff) - starttime;
 #ifdef DEBUG
    	g_printf("\nduration = %i\n",duration);
 #endif
-  	((nstack *) tmp->data)->duration = duration;
-	((nstack *) tmp->data)->measure = mididata->si->currentmeasurenum;
+  	((nstack *) tmp->data)->duration = (int *) duration;
+	((nstack *) tmp->data)->measure = (int *) mididata->gui->si->currentmeasurenum;
+	((nstack *) tmp->data)->staffnum = (int *) mididata->track;
  	
 	mididata->final_list = g_list_append(mididata->final_list, tmp);
 	mididata->notestack = g_list_remove (mididata->notestack, tmp);
@@ -472,9 +473,13 @@ donoteoff (midicallback *mididata, gint pitchoff, gint timeoff)
 
 
 void ProcessNoteList(GList *list, midicallback *mididata) {
-	gint starttime = ((nstack *) list->data)->timeon;
+	DenemoScore *si = mididata->gui->si;
+	gint starttime = (int) ((nstack *) list->data)->timeon;
 	gint lastoff = mididata->lastoff;
-	gint duration = ((nstack *) list->data)->duration;
+	gint duration = (int) ((nstack *) list->data)->duration;
+	gint currentstaffnum = si->currentstaffnum; 
+	gint track = (int) ((nstack *) list->data)->staffnum;
+	gint pitch = (int) ((nstack *) list->data)->pitch;
 	nstack note;
 	DenemoObject *mudela_obj_new;
 
@@ -486,86 +491,87 @@ void ProcessNoteList(GList *list, midicallback *mididata) {
 	
 	/* call function to return a list of chord tones */
 	mididata->currentnote = g_list_copy(list);
-	g_list_foreach(mididata->final_list, NewFindChordTones, mididata);
-	//mididata->currentnote = g_list_remove_link(mididata->currentnote, g_list_first (mididata->currentnote));
+	g_list_foreach(mididata->final_list, (GFunc) findchordtones, mididata);
+
+	if ((track + 1) > currentstaffnum) /*if not first track add track */
+	  {
+	      si->currentstaffnum++;
+	      si->currentstaff = g_list_first (si->thescore); /* set the first track to be copied */
+	      newstaff (mididata->gui, ADDFROMLOAD, DENEMO_NONE); /* add track */
+	      si->currentstaff = g_list_last (si->thescore); /* make last staff the current */
+	      mididata->lastoff = 0;
+	      si->cursor_x = 0;
+	      mididata->bartime = 0;
+	      si->currentmeasurenum = 1;
+	  }
 
 	if (starttime == mididata->lastoff) {
 		notetype length = ConvertLength(duration, mididata);
 		mudela_obj_new = new_dnm_object (length,mididata->leftover);
-		struct harmonic enote = enharmonic ((int *) ((nstack *) list->data)->pitch,mididata->key);
-		dnm_addtone (mudela_obj_new, enote.pitch, enote.enshift, mididata->si->cursorclef);
+		struct harmonic enote = enharmonic ((int) ((nstack *) list->data)->pitch,mididata->key);
+		dnm_addtone (mudela_obj_new, enote.pitch, enote.enshift, mididata->gui->si->cursorclef);
 		while (mididata->chordnotes){
-			struct harmonic enote = enharmonic ((int *) 
-					((int) mididata->chordnotes->data),mididata->key);
-	                dnm_addtone (mudela_obj_new, enote.pitch, enote.enshift, mididata->si->cursorclef);		
-		   mididata->chordnotes = g_list_remove_link(mididata->chordnotes, g_list_first (mididata->chordnotes));
-			//mididata->chordnotes = g_list_next(mididata->chordnotes);	
+		  struct harmonic enote = enharmonic ((int) 
+				 mididata->chordnotes->data,mididata->key);
+	          dnm_addtone (mudela_obj_new, enote.pitch, enote.enshift, mididata->gui->si->cursorclef);		
+		  mididata->chordnotes = g_list_remove_link(mididata->chordnotes, g_list_first (mididata->chordnotes));
 		}
-	        //mididata->chordnotes = g_list_remove_link(mididata->chordnotes, g_list_first (mididata->chordnotes));	
 		mididata->lastoff = starttime + duration;
 		MeasureCheck(list,mididata);
-		object_insert (mididata->si, mudela_obj_new);
+		object_insert (mididata->gui, mudela_obj_new);
 		mididata->bartime = mididata->bartime + duration;
 	}
 	if ((starttime != mididata->lastoff) && (mididata->leftover !=0)){
 		mididata->leftover = 0;
-		((nstack *) list->data)->timeon = mididata->lastoff;
+		((nstack *) list->data)->timeon = (int *) mididata->lastoff;
 		MeasureCheck(list,mididata);
 		addnote(list, mididata);
 	}
-	
-	
-
-
+#ifdef DEBUG
+	printf("\nNotelist pitch = %i, timeon = %i, duration = %i, staffnum = %i\n", pitch, starttime, duration, track);
+#endif
 }
 
 void restcheck(GList *tmp, midicallback *mididata){
   gint rest = 0;
   gint PPQN = mididata->PPQN;
-  gint starttime = ((nstack *) tmp->data)->timeon;
-  gint duration = ((nstack *) tmp->data)->duration;
+  gint starttime = (int) ((nstack *) tmp->data)->timeon;
+  gint duration = (int) ((nstack *) tmp->data)->duration;
   DenemoObject *mudela_obj_new;
 
 	if (starttime > mididata->lastoff){
 		  rest = starttime - mididata->lastoff;
 		      /* Until I find a better way to pad the end of a measure */
-		  //if ((mididata->bartime + rest) > mididata->barlength){
-			//  //reduce rest and bartime 
-			  //mididata->leftover = ((nstack *) tmp->data)->timeon - mididata->barlength;
-			  //rest = mididata->barlength - mididata->lastoff;
-		 // }
-		  //mididata->bartime = mididata->bartime + rest;
-
+		
 			  if (rest > PPQN)
-			{
-			  if (rest % PPQN){
-				struct notetype length = ConvertLength((rest % PPQN), mididata);
-				mudela_obj_new = new_dnm_object(length,0);
-				object_insert (mididata->si, mudela_obj_new);
-				mididata->bartime = mididata->bartime + (rest % PPQN);
-				MeasureCheck(tmp,mididata);
-
-			  }
-			  rest = rest - (rest % PPQN);
-			  while (rest > 0)
-			 {
-				struct notetype length = ConvertLength(PPQN, mididata);
-				mudela_obj_new = new_dnm_object(length,0);
-				object_insert (mididata->si, mudela_obj_new);
-				rest = rest - PPQN;
-				mididata->bartime = mididata->bartime + (PPQN);
-				MeasureCheck(tmp,mididata);
-
+			  {
+			    if (rest % PPQN){
+			      struct notetype length = ConvertLength((rest % PPQN), mididata);
+			      mudela_obj_new = new_dnm_object(length,0);
+			      object_insert (mididata->gui, mudela_obj_new);
+			      mididata->bartime = mididata->bartime + (rest % PPQN);
+			      MeasureCheck(tmp,mididata);
 			    }
-			}
-		      if ((rest < PPQN) && (rest != 0)) {
-				struct notetype length = ConvertLength(rest, mididata);
-				mudela_obj_new = new_dnm_object(length,0);
-				object_insert (mididata->si, mudela_obj_new);
-				mididata->bartime = mididata->bartime + (rest);
-				MeasureCheck(tmp,mididata);
-				 
+			    
+			    rest = rest - (rest % PPQN);
+			    while (rest > 0){
+			      struct notetype length = ConvertLength(PPQN, mididata);
+			      mudela_obj_new = new_dnm_object(length,0);
+			      object_insert (mididata->gui, mudela_obj_new);
+			      rest = rest - PPQN;
+			      mididata->bartime = mididata->bartime + (PPQN);
+			      MeasureCheck(tmp,mididata);
+			    }
+			  }
+		          
+			  if ((rest < PPQN) && (rest != 0)) {
+			    struct notetype length = ConvertLength(rest, mididata);
+			    mudela_obj_new = new_dnm_object(length,0);
+			    object_insert (mididata->gui, mudela_obj_new);
+			    mididata->bartime = mididata->bartime + (rest);
+			    MeasureCheck(tmp,mididata);
 		      }
+
 		  if(0)// (((starttime - mididata->lastoff) > 0) && (mididata->leftover > 0))
 		  {
 			mididata->lastoff = mididata->bartime = mididata->barlength;
@@ -586,7 +592,7 @@ notetype ConvertLength(gint endnote, midicallback *mididata){
   gint dsq = (8 * endnote) / PPQN;
   
   switch (dsq)
-    {
+  {
     case 1:			/*demi-semi-quaver */
       {
 	notetype = 5;
@@ -689,58 +695,50 @@ notetype ConvertLength(gint endnote, midicallback *mididata){
 	notetype = 0;
 	break;
       }
-      /*default:
-         {
-         notetype = 2;
-         tied = endnote - PPQN;
-         break;
-         }  */
-    }
+  }
   gnotetype.notetype = notetype;
   gnotetype.tied = tied;
   return gnotetype;
 }
 
-
 /* all this stuff is just checking 
  * to see if we need to add a new measure 
  */
 void MeasureCheck(GList *list, midicallback *mididata){
   if (mididata->leftover == 0)
     {
-      if (mididata->bartime >= mididata->barlength)	/*(mididata->bartime == barlength) not entirely functional */
+      if (mididata->bartime >= mididata->barlength)	
 	{			/* mididata->bartime >= barlenth will be true if there are rests  or notes
 				   going over end of measures. */
-	  if (!mididata->si->currentmeasure->next)
+	  if (!mididata->gui->si->currentmeasure->next)
 	    /* Add a measure and make it currentmeasure */
-	    mididata->si->currentmeasure =
-	      dnm_addmeasures (mididata->si, mididata->si->currentmeasurenum, 1, 1);
+	    mididata->gui->si->currentmeasure =
+	      dnm_addmeasures (mididata->gui->si, mididata->gui->si->currentmeasurenum, 1, 1);
 	  else
-	    mididata->si->currentmeasure = mididata->si->currentmeasure->next;
+	    mididata->gui->si->currentmeasure = mididata->gui->si->currentmeasure->next;
 	  /* Now the stuff that needs to be done for each case */
-	  mididata->si->currentmeasurenum++;
-	  mididata->si->currentobject = NULL;
-	  mididata->si->cursor_x = 0;
-	  memcpy (mididata->si->cursoraccs, mididata->si->nextmeasureaccs, SEVENGINTS);
-	  memcpy (mididata->si->curmeasureaccs, mididata->si->nextmeasureaccs, SEVENGINTS);
-	  mididata->si->curmeasureclef = mididata->si->cursorclef;
+	  mididata->gui->si->currentmeasurenum++;
+	  mididata->gui->si->currentobject = NULL;
+	  mididata->gui->si->cursor_x = 0;
+	  memcpy (mididata->gui->si->cursoraccs, mididata->gui->si->nextmeasureaccs, SEVENGINTS);
+	  memcpy (mididata->gui->si->curmeasureaccs, mididata->gui->si->nextmeasureaccs, SEVENGINTS);
+	  mididata->gui->si->curmeasureclef = mididata->gui->si->cursorclef;
 	  mididata->bartime = 0;
 	}
 
-      //mididata->bartime = mididata->bartime + ((nstack *) list->data)->duration;
     }
 }
 
 void TiedNoteCheck(GList *list, midicallback *mididata){
   	restcheck(list,mididata);
-	gint endnote = ((nstack *) list->data)->duration;
-  	gint barlength = mididata->barlength;
+	gint endnote = (int) ((nstack *) list->data)->duration;
+  	gint barlength = (int) mididata->barlength;
   
 	if ((mididata->bartime + endnote) > barlength)
 	{
 	  	mididata->leftover = (int) (barlength - mididata->bartime);	/*tied over bar line */
 	  	endnote = endnote - (int) mididata->leftover;
-		((nstack *) list->data)->duration = endnote;
+		((nstack *) list->data)->duration = (int *) endnote;
 	}
       	else 
 		mididata->leftover = (int) 0;
@@ -752,9 +750,6 @@ void TiedNoteCheck(GList *list, midicallback *mididata){
 void
 addnote (GList *list, midicallback *mididata)
 {
-  /* check to see if a measure needs to be added */
-  //MeasureCheck(list, mididata);  
-
   /* check to see if notes are tied over */
   TiedNoteCheck(list, mididata);
    
