@@ -15,10 +15,11 @@ typedef struct notetype
 
 typedef struct nstack 
 {
-	gint pitch;
-   	gint measure;
-     	gint timeon;
-     	gint duration;  
+	gint *pitch;
+   	gint *measure;
+     	gint *timeon;
+     	gint *duration; 
+        gint *staffnum;	
 }nstack;
 
 typedef struct midicallback
@@ -27,7 +28,7 @@ typedef struct midicallback
 	GList *final_list;
 	GList *chordnotes;
 	GList *currentnote;
-	DenemoScore *si;
+	DenemoGUI *gui;
 	gint leftover;
 	gint PPQN;
 	gint bartime;
@@ -35,6 +36,7 @@ typedef struct midicallback
 	gint lastoff;
 	gint trackplus;
 	gint key;
+	gint track;
 }midicallback;
 
 typedef struct harmonic
@@ -65,9 +67,9 @@ void dotrackname(FILE* fp, midicallback *mididata, gint x);
 
 void doinstrname(FILE* fp, midicallback *mididata, gint x);
 
-void donoteon(midicallback *mididata, gint pitchon, gint attack, gint timeon);
+void donoteon(midicallback *mididata, gint *pitchon, gint *attack, gint *timeon);
 
-void donoteoff(midicallback *mididata, gint pitchoff, gint timeoff);
+void donoteoff(midicallback *mididata, gint *pitchoff, gint *timeoff);
 
 void restcheck(GList *tmp, midicallback *mididata);
 
