@@ -184,22 +184,41 @@ getXMLChild (xmlNodePtr parentElem, gchar * childElemName, xmlNsPtr childNs)
 static DenemoContext
 lookupContext (gchar * string)
 {
-  if (!strcmp (string, "Staff"))
+  if(string == NULL)
+    return 0;
+  static gboolean pianostaff=FALSE, groupstaff=FALSE, choirstaff=FALSE;
+  if (!strcmp (string, "PianoStaff"))//for backwards compatibility only
     {
-      return DENEMO_NONE;
+      pianostaff = !pianostaff;
+      if(pianostaff)
+	return DENEMO_PIANO_START;
+      else
+	return DENEMO_PIANO_END;
     }
-  else if (!strcmp (string, "PianoStaff"))
+  else if (!strcmp (string, "ChoirStaff"))//for backwards compatibility only
     {
-      return DENEMO_PIANO;
+      choirstaff = !choirstaff;
+      if(choirstaff)
+	return DENEMO_CHOIR_START;
+      else
+	return DENEMO_CHOIR_END;
     }
-  else if (!strcmp (string, "ChoirStaff"))
+  else if (!strcmp (string, "GroupStaff"))//for backwards compatibility only
     {
-      return DENEMO_CHOIR;
-    }
-  else if (!strcmp (string, "StaffGroup"))
-    {
-      return DENEMO_GROUP;
+      groupstaff = !groupstaff;
+      if(groupstaff)
+	return DENEMO_GROUP_START;
+      else
+	return DENEMO_GROUP_END;
     }
+#define LOOKUP(A,B)  else if (!strcmp (string, A)) {return B;}
+LOOKUP(PIANO_START_STRING,DENEMO_PIANO_START)
+LOOKUP(PIANO_END_STRING,DENEMO_PIANO_END)
+LOOKUP(CHOIR_START_STRING,DENEMO_CHOIR_START)
+LOOKUP(CHOIR_END_STRING,DENEMO_CHOIR_END)
+LOOKUP(GROUP_START_STRING,DENEMO_GROUP_START)
+LOOKUP(GROUP_END_STRING,DENEMO_GROUP_END)
+#undef LOOKUP
   else
     {
       return DENEMO_NONE;
@@ -617,7 +636,7 @@ parseFakechord (xmlNodePtr fakechordElem, DenemoObject * curobj)
 						  1);
   
   ((chord *) curobj->object)->is_fakechord = TRUE; 
-  seperate_fakechord_elements (fakechord, curobj); 
+  separate_fakechord_elements (fakechord, curobj); 
   
   //((chord *) curobj->object)->fakechord = g_string_new (fakechord);
 
@@ -637,7 +656,7 @@ parseNote (xmlNodePtr noteElem, xmlNsPtr ns,
   gint middleCOffset = 0, accidental = 0,
     noteHeadType = DENEMO_NORMAL_NOTEHEAD;
   gboolean showAccidental = FALSE;
-  gchar *accidentalName, *showAccidentalProp, *noteHeadName;
+  gchar *accidentalName, *showAccidentalProp, *noteHeadName, *directive = NULL;
 
   FOREACH_CHILD_ELEM (childElem, noteElem)
   {
@@ -697,6 +716,11 @@ parseNote (xmlNodePtr noteElem, xmlNsPtr ns,
 		g_free (showAccidentalProp);
 	      }
 	  }
+	else if (ELEM_NAME_EQ (childElem, "directive"))
+	  {
+	    directive = (gchar *) xmlNodeListGetString
+	      (childElem->doc, childElem->xmlChildrenNode, 1);	    
+	  }
 	else if (ELEM_NAME_EQ (childElem, "note-head"))
 	  {
 	    noteHeadName =
@@ -733,7 +757,9 @@ parseNote (xmlNodePtr noteElem, xmlNsPtr ns,
 
   /* Now actually construct the note object. */
 
-  addtone (chordObj, middleCOffset, accidental, currentClef);
+  note *newnote = addtone (chordObj, middleCOffset, accidental, currentClef);
+  if(newnote && directive)
+    newnote->directive = g_string_new(directive);
   if (noteHeadType != DENEMO_NORMAL_NOTEHEAD)
     {
       /* FIXME: Is this the right note in the chord? */
@@ -1275,12 +1301,12 @@ parseChord (xmlNodePtr chordElem, xmlNsPtr ns,
 	else if (ELEM_NAME_EQ (childElem, "figure"))
 	  {
 	    parseFigure (childElem, chordObj);
-	    si->has_figures = TRUE;
+	    si->has_figures = (gpointer) TRUE;
 	  }
 	else if (ELEM_NAME_EQ (childElem, "fakechord"))
 	  {
 	    parseFakechord (childElem, chordObj);
-	    si->has_fakechords = TRUE;
+	    si->has_fakechords = (gpointer) TRUE;
 	  }
 
 	else
@@ -1306,11 +1332,15 @@ parseChord (xmlNodePtr chordElem, xmlNsPtr ns,
 parseLilyDir (xmlNodePtr LilyDirectiveElem, xmlNsPtr ns, DenemoScore *si)
 {
   gchar *directive = (gchar *) xmlNodeListGetString (LilyDirectiveElem->doc,
-                     LilyDirectiveElem->
-                     xmlChildrenNode,
-                     1);
-
-  return lily_directive_new (directive);
+						     LilyDirectiveElem->
+						     xmlChildrenNode,
+						     1);
+  gchar *locked = (gchar *) xmlGetProp (LilyDirectiveElem, (xmlChar *) "locked");
+  DenemoObject *curobj = lily_directive_new (directive);
+  if(locked)
+    ((lilydirective*)curobj->object)->locked = !strcmp (locked, "true");
+  g_free(locked);
+  return curobj;
 }
 
 
@@ -1437,7 +1467,7 @@ parseGraceStart (xmlNodePtr graceStartElem, xmlNsPtr ns, DenemoScore * si)
  * @return 0 on succes, -1 on failure
  */
 static gint
-parseSetupInfo (xmlNodePtr editInfoElem, xmlNsPtr ns, DenemoScore * si)
+parseSetupInfo (xmlNodePtr editInfoElem, xmlNsPtr ns, DenemoGUI * gui)
 {
   xmlNodePtr childElem;
   gint cursorposition = 0;
@@ -1454,16 +1484,28 @@ parseSetupInfo (xmlNodePtr editInfoElem, xmlNsPtr ns, DenemoScore * si)
 						  xmlChildrenNode, 1);
 	    if (tmp != NULL)
 	      {
-		g_print ("lilypond version %s", tmp);
-		g_string_assign (si->sconfig->lilyversion, tmp);
+		//g_print ("lilypond version %s", tmp);
+		g_string_assign (gui->lilycontrol.lilyversion, tmp);
+		g_free (tmp);
+	      }
+	  }
+	if (ELEM_NAME_EQ (childElem, "lilypond"))
+	  {
+	    tmp = (gchar *) xmlNodeListGetString (childElem->doc,
+						  childElem->
+						  xmlChildrenNode, 1);
+	    if (tmp != NULL)
+	      {
+		//g_print ("lilypond directive for all music %s", tmp);
+		g_string_assign (gui->lilycontrol.lilypond, tmp);
 		g_free (tmp);
 	      }
 	  }
 	else if (ELEM_NAME_EQ (childElem, "fontsize"))
 	  {
 	    gint font = getXMLIntChild (childElem);
-	    si->sconfig->fontsize = font;
-	    g_print ("Font Size %d", font);
+	    gui->lilycontrol.fontsize = font;
+	    //g_print ("Font Size %d", font);
 	  }
 	else if (ELEM_NAME_EQ (childElem, "papersize"))
 	  {
@@ -1473,8 +1515,8 @@ parseSetupInfo (xmlNodePtr editInfoElem, xmlNsPtr ns, DenemoScore * si)
 						  xmlChildrenNode, 1);
 	    if (tmp != NULL)
 	      {
-		g_print ("Paper size %s\n", tmp);
-		g_string_assign (si->sconfig->papersize, tmp);
+		//g_print ("Paper size %s\n", tmp);
+		g_string_assign (gui->lilycontrol.papersize, tmp);
 		g_free (tmp);
 	      }
 	  }
@@ -1482,8 +1524,8 @@ parseSetupInfo (xmlNodePtr editInfoElem, xmlNsPtr ns, DenemoScore * si)
 	  {
 
 	    gint orientation = getXMLIntChild (childElem);
-	    si->sconfig->orientation = orientation;
-	    g_print ("Orientation %d\n", orientation);
+	    gui->lilycontrol.orientation = orientation;
+	    //g_print ("Orientation %d\n", orientation);
 	  }
       }
   }
@@ -1512,12 +1554,12 @@ parseEditInfo (xmlNodePtr editInfoElem, xmlNsPtr ns, DenemoScore * si,
 	if (ELEM_NAME_EQ (childElem, "staffno"))
 	  {
 	    *staffnoPtr = getXMLIntChild (childElem);
-	    g_print ("Staff no %d\t", *staffnoPtr);
+	    //g_print ("Staff no %d\t", *staffnoPtr);
 	  }
 	else if (ELEM_NAME_EQ (childElem, "cursorposition"))
 	  {
 	    cursorposition = getXMLIntChild (childElem);
-	    g_print ("Cursor Pos %d\n", cursorposition);
+	    //g_print ("Cursor Pos %d\n", cursorposition);
 	    si->cursor_x = cursorposition;
 	  }
       }
@@ -1541,7 +1583,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
   gint bpm;
   gchar *title, *subtitle, *composer, *poet, *meter, *arranger, *opus;
   gchar *instrument, *dedication, *piece, *head, *copyright, *footer,
-    *tagline;
+    *tagline, *mvmnt_header, *markup_before, *markup_after;
 
   FOREACH_CHILD_ELEM (childElem, scoreInfoElem)
   {
@@ -1589,7 +1631,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						    xmlChildrenNode, 1);
 	    if (title != NULL)
 	      {
-		g_string_assign (si->headerinfo->title, title);
+		g_string_assign (si->headerinfo.title, title);
 		g_free (title);
 	      }
 	  }
@@ -1600,7 +1642,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						       xmlChildrenNode, 1);
 	    if (subtitle != NULL)
 	      {
-		g_string_assign (si->headerinfo->subtitle, subtitle);
+		g_string_assign (si->headerinfo.subtitle, subtitle);
 		g_free (subtitle);
 	      }
 	  }
@@ -1611,7 +1653,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						       xmlChildrenNode, 1);
 	    if (composer != NULL)
 	      {
-		g_string_assign (si->headerinfo->composer, composer);
+		g_string_assign (si->headerinfo.composer, composer);
 		g_free (composer);
 	      }
 	  }
@@ -1622,7 +1664,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						   xmlChildrenNode, 1);
 	    if (poet != NULL)
 	      {
-		g_string_assign (si->headerinfo->poet, poet);
+		g_string_assign (si->headerinfo.poet, poet);
 		g_free (poet);
 	      }
 	  }
@@ -1633,7 +1675,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						    xmlChildrenNode, 1);
 	    if (meter != NULL)
 	      {
-		g_string_assign (si->headerinfo->meter, meter);
+		g_string_assign (si->headerinfo.meter, meter);
 		g_free (meter);
 	      }
 	  }
@@ -1644,7 +1686,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						   xmlChildrenNode, 1);
 	    if (opus != NULL)
 	      {
-		g_string_assign (si->headerinfo->opus, opus);
+		g_string_assign (si->headerinfo.opus, opus);
 		g_free (opus);
 	      }
 	  }
@@ -1655,7 +1697,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						       xmlChildrenNode, 1);
 	    if (arranger != NULL)
 	      {
-		g_string_assign (si->headerinfo->arranger, arranger);
+		g_string_assign (si->headerinfo.arranger, arranger);
 		g_free (arranger);
 	      }
 	  }
@@ -1666,7 +1708,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 							 xmlChildrenNode, 1);
 	    if (instrument != NULL)
 	      {
-		g_string_assign (si->headerinfo->instrument, instrument);
+		g_string_assign (si->headerinfo.instrument, instrument);
 		g_free (instrument);
 	      }
 	  }
@@ -1677,7 +1719,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 							 xmlChildrenNode, 1);
 	    if (dedication != NULL)
 	      {
-		g_string_assign (si->headerinfo->dedication, dedication);
+		g_string_assign (si->headerinfo.dedication, dedication);
 		g_free (dedication);
 	      }
 	  }
@@ -1688,7 +1730,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						    xmlChildrenNode, 1);
 	    if (piece != NULL)
 	      {
-		g_string_assign (si->headerinfo->piece, piece);
+		g_string_assign (si->headerinfo.piece, piece);
 		g_free (piece);
 	      }
 	  }
@@ -1699,7 +1741,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						   xmlChildrenNode, 1);
 	    if (head != NULL)
 	      {
-		g_string_assign (si->headerinfo->head, head);
+		g_string_assign (si->headerinfo.head, head);
 		g_free (head);
 	      }
 	  }
@@ -1710,7 +1752,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 							xmlChildrenNode, 1);
 	    if (copyright != NULL)
 	      {
-		g_string_assign (si->headerinfo->copyright, copyright);
+		g_string_assign (si->headerinfo.copyright, copyright);
 		g_free (copyright);
 	      }
 	  }
@@ -1721,7 +1763,7 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						     xmlChildrenNode, 1);
 	    if (footer != NULL)
 	      {
-		g_string_assign (si->headerinfo->footer, footer);
+		g_string_assign (si->headerinfo.footer, footer);
 		g_free (footer);
 	      }
 	  }
@@ -1732,10 +1774,47 @@ parseScoreInfo (xmlNodePtr scoreInfoElem, xmlNsPtr ns, DenemoScore * si)
 						      xmlChildrenNode, 1);
 	    if (tagline != NULL)
 	      {
-		g_string_assign (si->headerinfo->tagline, tagline);
+		g_string_assign (si->headerinfo.tagline, tagline);
 		g_free (tagline);
 	      }
 	  }
+	else if (ELEM_NAME_EQ (childElem, "extra"))
+	  {
+	    mvmnt_header = (gchar *) xmlNodeListGetString (childElem->doc,
+						      childElem->
+						      xmlChildrenNode, 1);
+	    if (mvmnt_header != NULL)
+	      {
+		g_string_assign (si->headerinfo.extra, mvmnt_header);
+		g_free (mvmnt_header);
+	      }
+	  }
+	else if (ELEM_NAME_EQ (childElem, "markup_before"))
+	  {
+	    markup_before = (gchar *) xmlNodeListGetString (childElem->doc,
+						      childElem->
+						      xmlChildrenNode, 1);
+	    if (markup_before != NULL)
+	      {
+		g_string_assign (si->headerinfo.lilypond_before, markup_before);
+		g_free (markup_before);
+	      }
+	  }
+	else if (ELEM_NAME_EQ (childElem, "markup_after"))
+	  {
+	    markup_after = (gchar *) xmlNodeListGetString (childElem->doc,
+						      childElem->
+						      xmlChildrenNode, 1);
+	    if (markup_after != NULL)
+	      {
+		g_string_assign (si->headerinfo.lilypond_after, markup_after);
+		g_free (markup_after);
+	      }
+	  }
+
+
+
+
       }
     else
       {
@@ -1779,6 +1858,18 @@ parseStaff (xmlNodePtr staffElem, xmlNsPtr ns, DenemoScore * si)
 		curStaff->no_of_lines = 5;
 	      }
 	  }
+        else if (ELEM_NAME_EQ (childElem, "midi_prognum_override"))
+	  {
+  	    curStaff->midi_prognum_override = getXMLIntChild (childElem);
+  	  }	    
+  	else if (ELEM_NAME_EQ (childElem, "midi_prognum"))
+	  {
+  	    curStaff->midi_prognum = getXMLIntChild (childElem);
+  	  }
+	else if (ELEM_NAME_EQ (childElem, "midi_channel"))
+	  {
+  	    curStaff->midi_channel = getXMLIntChild (childElem);
+  	  }
 	else if (ELEM_NAME_EQ (childElem, "transpose"))
 	  {
 	    curStaff->transposition = getXMLIntChild (childElem);
@@ -1795,7 +1886,17 @@ parseStaff (xmlNodePtr staffElem, xmlNsPtr ns, DenemoScore * si)
 	  {
 	    curStaff->hasfigures = getXMLIntChild (childElem);
 	    if(curStaff->hasfigures)
-	      si->has_figures=TRUE;
+	      si->has_figures = (gpointer) TRUE;
+	  }
+	else if (ELEM_NAME_EQ (childElem, "hasfakechords"))
+	  {
+	    curStaff->hasfakechords = getXMLIntChild (childElem);
+	    
+	  }
+	else if (ELEM_NAME_EQ (childElem, "haslyrics"))
+	  {
+	    curStaff->haslyrics = getXMLIntChild (childElem);
+	    
 	  }
 	else if (ELEM_NAME_EQ (childElem, "instrument"))
 	  {
@@ -1809,15 +1910,46 @@ parseStaff (xmlNodePtr staffElem, xmlNsPtr ns, DenemoScore * si)
 	    gchar *temp = NULL;
 	    temp = (gchar *) xmlNodeListGetString
 	      (childElem->doc, childElem->xmlChildrenNode, 1);
-	    curStaff->context = lookupContext (temp);
+	    curStaff->context |= lookupContext (temp);
 	    g_free (temp);
 	  }
+
+
 	else if (ELEM_NAME_EQ (childElem, "lilybefore"))
 	  {
-	    gchar *temp = NULL;
-	    temp = (gchar *) xmlNodeListGetString
+	    gchar *temp = (gchar *) xmlNodeListGetString
 	      (childElem->doc, childElem->xmlChildrenNode, 1);
-	    curStaff->lilybefore = temp;
+	    if(temp)
+	      g_warning("Ignoring the old-style string %s\nAdd this in LilyPond window if required\n", temp);
+	    g_free(temp);
+	  }
+	else if (ELEM_NAME_EQ (childElem, "staff-prolog"))
+	  {
+	    gchar *temp = 
+	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1);
+	    curStaff->staff_prolog = (temp?g_string_new(temp):NULL);
+	    g_free (temp);
+	  }
+	else if (ELEM_NAME_EQ (childElem, "lyrics-prolog"))
+	  {
+	    gchar *temp = 
+	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1);
+	    curStaff->lyrics_prolog = (temp?g_string_new(temp):NULL);
+	    g_free (temp);
+	  }
+	else if (ELEM_NAME_EQ (childElem, "figures-prolog"))
+	  {
+	    gchar *temp = 
+	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1);
+	    curStaff->figures_prolog = (temp?g_string_new(temp):NULL);
+	    g_free (temp);
+	  }
+	else if (ELEM_NAME_EQ (childElem, "fakechords-prolog"))
+	  {
+	    gchar *temp = 
+	      (gchar *) xmlNodeListGetString (childElem->doc, childElem->xmlChildrenNode, 1);
+	    curStaff->fakechords_prolog = (temp?g_string_new(temp):NULL);
+	    g_free (temp);
 	  }
 	else
 	  {
@@ -1967,8 +2099,8 @@ parseInitVoiceParams (xmlNodePtr initVoiceParamsElem, xmlNsPtr ns,
 	  {
 	    parseKeySignature (childElem, ns, &(curVoice->skey),
 			       &(curVoice->skey_isminor));
-	    //initkeyaccs (curVoice->skeyaccs, curVoice->skey);
-	    dnm_setinitialkeysig(curVoice, curVoice->skey, curVoice->skey_isminor);
+	    initkeyaccs (curVoice->skeyaccs, curVoice->skey);
+	    //dnm_setinitialkeysig(curVoice, curVoice->skey, curVoice->skey_isminor);
 	  }
 	else if (ELEM_NAME_EQ (childElem, "time-signature"))
 	  {
@@ -2066,6 +2198,15 @@ parseMeasures (xmlNodePtr measuresElem, xmlNsPtr ns, DenemoScore * si)
 				       &slurEndChordElems,
 				       &crescEndChordElems,
 				       &diminEndChordElems, &notesElem);
+		  /* old format files will not have haslyrics... fields of staff explicit
+		   so for backwards compatibility we reconstruct it here*/
+		  if(((chord *) curObj->object)->lyric)
+		    ((DenemoStaff *) si->currentstaff->data)->haslyrics = TRUE;
+		  if(((chord *) curObj->object)->figure)
+		    ((DenemoStaff *) si->currentstaff->data)->hasfigures = TRUE;
+		  if(((chord *) curObj->object)->fakechord)
+		    ((DenemoStaff *) si->currentstaff->data)->hasfakechords = TRUE;
+
 		  if (startedBeam)
 		    {
 		      curObj->isstart_beamgroup = TRUE;
@@ -2108,10 +2249,18 @@ parseMeasures (xmlNodePtr measuresElem, xmlNsPtr ns, DenemoScore * si)
 		}
 	      else if (ELEM_NAME_EQ (objElem, "key-signature"))
 		{
+#if 1
 		  parseKeySignature (objElem, ns, &keySig, &isMinor);
 		  curObj = dnm_newkeyobj (keySig, isMinor, 0);
+		  initkeyaccs (((keysig *) curObj->object)->accs, keySig);
+		  //dnm_setinitialkeysig(((keysig *) curObj->object), keySig, isMinor);
+#else
+		  //warningdialog("Dodgy code");
+		  parseKeySignature (objElem, ns, &keySig, &isMinor);
+		  //curObj = dnm_newkeyobj (keySig, isMinor, 0);
 		  //initkeyaccs (((keysig *) curObj->object)->accs, keySig);
-		  dnm_setinitialkeysig(((keysig *) curObj->object), keySig, isMinor);
+		  dnm_setinitialkeysig((DenemoStaff *) si->currentstaff->data, keySig, isMinor);
+#endif
 		}
 	      else if (ELEM_NAME_EQ (objElem, "measure-break"))
 		{
@@ -2239,15 +2388,16 @@ parseMeasures (xmlNodePtr measuresElem, xmlNsPtr ns, DenemoScore * si)
  * @return 0 on success, -1 on failure
  */
 static gint
-parseVoice (xmlNodePtr voiceElem, xmlNsPtr ns, DenemoScore * si)
+parseVoice (xmlNodePtr voiceElem, xmlNsPtr ns, DenemoGUI * gui)
 {
+  DenemoScore *si = gui->si;
   xmlNodePtr childElem;
   /*  gchar *id; */
 
   /* Create the staff structure. */
 
   si->currentstaffnum++;
-  newstaff (si, ADDFROMLOAD, DENEMO_NONE);
+  newstaff (gui, ADDFROMLOAD, DENEMO_NONE);
   si->currentstaff = g_list_last (si->thescore);
   si->currentmeasurenum = 1;
 
@@ -2285,14 +2435,12 @@ parseVoice (xmlNodePtr voiceElem, xmlNsPtr ns, DenemoScore * si)
  * @return 0 on success ,-1 on failure
  */
 static gint
-parseScore (xmlNodePtr scoreElem, xmlNsPtr ns, DenemoScore * si,
-	    gint * staffnoPtr)
+parseScore (xmlNodePtr scoreElem, xmlNsPtr ns, DenemoGUI * gui,
+	    gint * staffnoPtr, ImportType type)
 {
+  DenemoScore *si = gui->si;
   xmlNodePtr childElem, voiceElem;
 
-  childElem = getXMLChild (scoreElem, "paper-info", ns);
-  if (childElem != 0)
-    parseSetupInfo (childElem, ns, si);
 
   childElem = getXMLChild (scoreElem, "edit-info", ns);
 
@@ -2303,8 +2451,9 @@ parseScore (xmlNodePtr scoreElem, xmlNsPtr ns, DenemoScore * si,
 
   childElem = getXMLChild (scoreElem, "score-info", ns);
   RETURN_IF_ELEM_NOT_FOUND ("score", childElem, "score-info");
-  if (parseScoreInfo (childElem, ns, si) != 0)
-    return -1;
+  if(type==REPLACE_SCORE)
+    if ( parseScoreInfo (childElem, ns, si) != 0)
+      return -1;
 
   /*
    * Note: We don't currently care about <staves>, but we will need to
@@ -2318,7 +2467,7 @@ parseScore (xmlNodePtr scoreElem, xmlNsPtr ns, DenemoScore * si,
   {
     if (ELEM_NAME_EQ (voiceElem, "voice") && (voiceElem->ns == ns))
       {
-	if (parseVoice (voiceElem, ns, si) != 0)
+	if (parseVoice (voiceElem, ns, gui) != 0)
 	  return -1;
       }
     else
@@ -2340,25 +2489,48 @@ parseScore (xmlNodePtr scoreElem, xmlNsPtr ns, DenemoScore * si,
 }
 
 
+/* parse the movement (ie DenemoScore) from childElem */
+static gint   parseMovement(xmlNodePtr childElem, xmlNsPtr ns, DenemoGUI *gui, ImportType type) {
+  int ret= 0, staffno = 0;
+
+   DenemoScore *si = gui->si;
+   if(type!=ADD_STAFFS)
+     gui->movements = g_list_append(gui->movements, gui->si);
+   si->currentstaffnum = 0;
+   sPrevStaffElem = NULL;
+   ret = parseScore (childElem, ns, gui, &staffno, type);
+   sPrevStaffElem = NULL;
+   gui->si->currentstaffnum = staffno;
+   updatescoreinfo (gui);
+   set_rightmeasurenum (gui->si);
+   set_bottom_staff (gui);
+   set_width_to_work_with(gui);
+   return ret;
+}
+
+
+
+
+
+
 /**
  * Import the given (possibly zlib-compressed) Denemo "native" XML file into
  * the given score.
  * 
  * @param filename the file to importxml
- * @param si the DenemoScore structure to populate
+ * @param  gui DenemoGUI to hold the score
  * @return 0 on success, -1 on failure
  */
 gint
-importXML (gchar * filename, DenemoScore * si)
+importXML (gchar * filename, DenemoGUI *gui, ImportType type)
 {
-  gint ret = 0, staffno = 0;
+  gint ret = 0;
   xmlDocPtr doc = NULL;
   xmlNsPtr ns;
   /*  xmlNodePtr rootElem, childElem; */
   xmlNodePtr rootElem;
   gchar *version = NULL;
-  /*tmp variable to hold readonly status while score is freed */
-  int tmpreadonly = si->readonly;
+
   /* Try to parse the file. */
 
   doc = xmlParseFile (filename);
@@ -2395,12 +2567,16 @@ importXML (gchar * filename, DenemoScore * si)
       ret = -1;
       goto cleanup;
     }
-  if (strcmp (version, "1.0") != 0)
+  gint version_number = 2;
+  if (strcmp (version, "2.0") != 0)
     {
+      if(strcmp (version, "1.0") != 0) {
       g_warning ("Denemo XML file version %s found, but we can only handle "
-		 "version 1.0", version);
+		 "versions 1.0 and 2.0", version);
       ret = -1;
       goto cleanup;
+      } else
+	version_number = 1;
     }
 
   /*
@@ -2410,19 +2586,99 @@ importXML (gchar * filename, DenemoScore * si)
    */
 
   buildXMLIDToElemMap (doc);
-
   /* Then, parse the score. */
-
-  free_score (si);
-  si->currentstaffnum = 0;
-  si->readonly = tmpreadonly;
-  sPrevStaffElem = NULL;
-  ret = parseScore (rootElem, ns, si, &staffno);
-  sPrevStaffElem = NULL;
-  si->currentstaffnum = staffno;
-
-cleanup:
-
+  if(version_number==2) {
+    xmlNodePtr childElem;
+    switch(type) {
+    case ADD_STAFFS:
+      FOREACH_CHILD_ELEM(childElem, rootElem){
+	if (ELEM_NAME_EQ (childElem, "lilycontrol") 
+	    || ELEM_NAME_EQ (childElem, "custom_scoreblock")
+	    || ELEM_NAME_EQ (childElem, "visible_scoreblock")){
+	  continue;
+	} else
+	ret |=  parseMovement(childElem, ns, gui, type);
+	//g_print("parsed more staffs breaking now\n");
+	break;//Note: we only adds staffs from first movement
+      }
+      break;
+    case ADD_MOVEMENTS:
+      FOREACH_CHILD_ELEM(childElem, rootElem){
+	if (ELEM_NAME_EQ (childElem, "lilycontrol") 
+	    || ELEM_NAME_EQ (childElem, "custom_scoreblock")
+	    || ELEM_NAME_EQ (childElem, "visible_scoreblock")){
+	  continue;/* do not change the header when adding movements parseScoreInfo(childElem, ns, gui);*/
+	} else {
+	  new_empty_score(gui);
+	  ret |=  parseMovement(childElem, ns, gui, type);
+	  //g_print("parsed movement\n");
+	}
+      }
+      break;
+    case REPLACE_SCORE:
+      free_gui(gui);
+      /* this is dependent on the order of elements, which is not strictly correct */
+      FOREACH_CHILD_ELEM(childElem, rootElem){
+	if (ELEM_NAME_EQ (childElem, "lilycontrol")){
+	  parseSetupInfo(childElem, ns, gui);
+	} else
+	if (ELEM_NAME_EQ (childElem, "custom_scoreblock")){
+	    gchar *tmp = (gchar *) xmlNodeListGetString (childElem->doc,
+						  childElem->
+						  xmlChildrenNode, 1);
+	    if (tmp != NULL)
+	      {
+		DenemoScoreblock *sb = g_malloc0 (sizeof(DenemoScoreblock));
+		sb->scoreblock = g_string_new(tmp);
+		gui->custom_scoreblocks = g_list_prepend(gui->custom_scoreblocks, sb);
+		g_free (tmp);
+	      }
+	  } else
+	    if (ELEM_NAME_EQ (childElem, "visible_scoreblock")){
+	      {
+		if (gui->custom_scoreblocks)
+		  {
+		    DenemoScoreblock *sb =  (DenemoScoreblock*)gui->custom_scoreblocks->data;
+		    sb->visible = TRUE;
+		  }
+	      }
+	    }
+	    else {
+	      new_empty_score(gui);
+	      ret |=  parseMovement(childElem, ns, gui, type);
+	    }
+      }
+      break;
+    default:
+      warningdialog("Erroneous call");
+      goto cleanup;
+    } 
+  } else {//version 1
+    switch(type) {
+    case ADD_STAFFS:
+      ret +=  parseMovement(rootElem, ns, gui, type);
+      break;
+    case ADD_MOVEMENTS:
+      new_empty_score(gui);
+      ret =  parseMovement(rootElem, ns, gui, type);
+      break;
+    case REPLACE_SCORE:
+      free_score (gui);
+      if(gui->movements)
+	g_list_free(gui->movements);/*FIXME free all the other si */
+      gui->movements = NULL;
+      init_score(gui->si, gui);
+      gui->si->currentstaffnum = 0;
+      ret =  parseMovement(rootElem, ns, gui, type);
+      break;
+    default:
+      warningdialog("Erroneous call");
+      goto cleanup;
+    }
+  }
+  //gui->si = gui->movements->data;
+ cleanup:
+  
   if (version != NULL)
     g_free (version);
   if (doc != NULL)
@@ -2431,5 +2687,9 @@ cleanup:
   g_hash_table_destroy (sXMLIDToElemMap);
   sXMLIDToElemMap = NULL;
 
+
+  //g_print("Number of movements %d\n", g_list_length(gui->movements));
   return ret;
 }
+
+
