@@ -1,6 +1,4 @@
 /* keyresponses.c
- * switching station for processing user keypresses
- *
  * for Denemo, a gtk+ frontend to GNU Lilypond
  * (c) 1999-2005 Matthew Hiller
  */
@@ -17,69 +15,36 @@
  * keypress event callback 
  * looks up the key press and executes the correct function
  */
+
 int
 scorearea_keypress_event (GtkWidget * widget, GdkEventKey * event,
 			  gpointer data)
 {
   DenemoGUI *gui = (DenemoGUI *) data;
-  KeybindingInfo *ki;
-
+  keymap *the_keymap = Denemo.prefs.the_keymap;
+#if 0
   if (gui->textview && GTK_WIDGET_IS_SENSITIVE (gui->textview))
     {
       /*    printf("Focus is on text window\n"); */
       gtk_widget_queue_draw (gui->textview);
       return 0;
     }
+#endif
   /* Look up the keystroke in the keymap and execute the appropriate
    * function */
-
-  if ((ki = lookup_keybinding (Denemo.prefs.the_keymap, event->keyval,
-			       event->state)))
-    {
-
-      /* in insert mode when a duration use the singleton rhythm pattern that
-	 is just this one duration */
-      if(gui->mode&INPUTINSERT) {
-	if(ki->func.nocallback == (gpointer)insert_chord_0key) {
-	  highlight_duration(gui, 0);
-	  return 0;
-	}
-	if(ki->func.nocallback == (gpointer)insert_chord_1key) {
-	  highlight_duration(gui, 1);
-	  return 0;
-	}
-	if(ki->func.nocallback == (gpointer)insert_chord_2key) {
-	  highlight_duration(gui, 2);
-	  return 0;
-	}
-	if(ki->func.nocallback == (gpointer)insert_chord_3key) {
-	  highlight_duration(gui, 3);
-	  return 0;
-	}
-	if(ki->func.nocallback == (gpointer)insert_chord_4key) {
-	  highlight_duration(gui, 4);
-	  return 0;
-	}
-	if(ki->func.nocallback == (gpointer)insert_chord_5key) {
-	  highlight_duration(gui, 5);
-	  return 0;
-	}
-	if(ki->func.nocallback == (gpointer)insert_chord_6key) {
-	  highlight_duration(gui, 6);
-	  return 0;
-	}
-
-	  }
-      
-
-      if (ki->callback_action == -1)
-	ki->func.nocallback (gui);
-      else
-	ki->func.callback (NULL, gui);
-      displayhelper (gui);
-      gtk_widget_draw (gui->scorearea, NULL);
-      return 1;
-    }
+  gint command_idx = lookup_keybinding (the_keymap, event->keyval,
+			       dnm_sanitize_key_state(event));
+  if (command_idx != -1) {
+      const gchar *command_name =
+          lookup_name_from_idx (the_keymap, command_idx);
+      if (execute_callback_from_idx(the_keymap, command_idx, gui)) {
+          displayhelper (gui);
+          gtk_widget_draw (gui->scorearea, NULL);
+          return 1;
+      } else {
+          g_warning("No callback for action %s", command_name);
+      }
+  }
   return 0;
 }
 
@@ -135,7 +100,7 @@ unset_selection_key (DenemoGUI * gui)
 void
 insert_chord_key (DenemoGUI * gui)
 {
-  dnm_insertchord (gui->si, 2, gui->mode, FALSE);
+  dnm_insertchord (gui, 2, gui->mode, FALSE);
 }
 
 /**
@@ -329,7 +294,7 @@ void
 insert_chord_0key (DenemoGUI * gui)
 {
 
-    dnm_insertchord (gui->si, 0, gui->mode, FALSE);
+    dnm_insertchord (gui, 0, gui->mode, FALSE);
 
 }
 
@@ -337,99 +302,99 @@ void
 insert_chord_1key (DenemoGUI * gui)
 {
 
-    dnm_insertchord (gui->si, 1, gui->mode, FALSE);
+    dnm_insertchord (gui, 1, gui->mode, FALSE);
 }
 
 void
 insert_chord_2key (DenemoGUI * gui)
 {
 
-    dnm_insertchord (gui->si, 2, gui->mode, FALSE);
+    dnm_insertchord (gui, 2, gui->mode, FALSE);
 }
 
 void
 insert_chord_3key (DenemoGUI * gui)
 {
 
-    dnm_insertchord (gui->si, 3, gui->mode, FALSE);
+    dnm_insertchord (gui, 3, gui->mode, FALSE);
 }
 
 void
 insert_chord_4key (DenemoGUI * gui)
 {
 
-    dnm_insertchord (gui->si, 4, gui->mode, FALSE);
+    dnm_insertchord (gui, 4, gui->mode, FALSE);
 }
 
 void
 insert_chord_5key (DenemoGUI * gui)
 {
 
-    dnm_insertchord (gui->si, 5, gui->mode, FALSE);
+    dnm_insertchord (gui, 5, gui->mode, FALSE);
 }
 
 void
 insert_chord_6key (DenemoGUI * gui)
 {
 
-    dnm_insertchord (gui->si, 6, gui->mode, FALSE);
+    dnm_insertchord (gui, 6, gui->mode, FALSE);
 }
 
 void
 insert_rest_0key (DenemoGUI * gui)
 {
-  dnm_insertchord (gui->si, 0, INPUTREST, TRUE);
+  dnm_insertchord (gui, 0, INPUTREST, TRUE);
 
 }
 
 void
 insert_rest_1key (DenemoGUI * gui)
 {
-  dnm_insertchord (gui->si, 1, INPUTREST, TRUE);
+  dnm_insertchord (gui, 1, INPUTREST, TRUE);
 }
 
 void
 insert_rest_2key (DenemoGUI * gui)
 {
-  dnm_insertchord (gui->si, 2, INPUTREST, TRUE);
+  dnm_insertchord (gui, 2, INPUTREST, TRUE);
 }
 
 void
 insert_rest_3key (DenemoGUI * gui)
 {
-  dnm_insertchord (gui->si, 3, INPUTREST, TRUE);
+  dnm_insertchord (gui, 3, INPUTREST, TRUE);
 }
 
 void
 insert_rest_4key (DenemoGUI * gui)
 {
-  dnm_insertchord (gui->si, 4, INPUTREST, TRUE);
+  dnm_insertchord (gui, 4, INPUTREST, TRUE);
 }
 
 void
 insert_rest_5key (DenemoGUI * gui)
 {
-  dnm_insertchord (gui->si, 5, INPUTREST, TRUE);
+  dnm_insertchord (gui, 5, INPUTREST, TRUE);
 }
 
 void
 insert_rest_6key (DenemoGUI * gui)
 {
-  dnm_insertchord (gui->si, 6, INPUTREST, TRUE);
+  dnm_insertchord (gui, 6, INPUTREST, TRUE);
 }
 
 
 void
 insert_blankchord_0key (DenemoGUI * gui)
 {
-  dnm_insertchord (gui->si, 0, INPUTBLANK, FALSE);
+  dnm_insertchord (gui, 0, INPUTBLANK, FALSE);
 }
 
 void
 insert_blankchord_1key (DenemoGUI * gui)
 {
 
-  dnm_insertchord (gui->si, 1, INPUTBLANK, FALSE);
+  dnm_insertchord (gui, 1, INPUTBLANK, FALSE);
 
 }
 
@@ -437,7 +402,7 @@ void
 insert_blankchord_2key (DenemoGUI * gui)
 {
 
-  dnm_insertchord (gui->si, 2, INPUTBLANK, FALSE);
+  dnm_insertchord (gui, 2, INPUTBLANK, FALSE);
 
 }
 
@@ -445,7 +410,7 @@ void
 insert_blankchord_3key (DenemoGUI * gui)
 {
 
-  dnm_insertchord (gui->si, 3, INPUTBLANK, FALSE);
+  dnm_insertchord (gui, 3, INPUTBLANK, FALSE);
 
 }
 
@@ -453,7 +418,7 @@ void
 insert_blankchord_4key (DenemoGUI * gui)
 {
 
-  dnm_insertchord (gui->si, 4, INPUTBLANK, FALSE);
+  dnm_insertchord (gui, 4, INPUTBLANK, FALSE);
 
 }
 
@@ -461,7 +426,7 @@ void
 insert_blankchord_5key (DenemoGUI * gui)
 {
 
-  dnm_insertchord (gui->si, 5, INPUTBLANK, FALSE);
+  dnm_insertchord (gui, 5, INPUTBLANK, FALSE);
 
 }
 
@@ -469,7 +434,7 @@ void
 insert_blankchord_6key (DenemoGUI * gui)
 {
 
-  dnm_insertchord (gui->si, 6, INPUTBLANK, FALSE);
+  dnm_insertchord (gui, 6, INPUTBLANK, FALSE);
 
 }
 
@@ -478,48 +443,48 @@ insert_blankchord_6key (DenemoGUI * gui)
 void
 insert_duplet (DenemoGUI * gui)
 {
-  dnm_inserttuplet (gui->si, DUPLET);
+  dnm_inserttuplet (gui, DUPLET);
 }
 
 void
 insert_triplet (DenemoGUI * gui)
 {
-  dnm_inserttuplet (gui->si, TRIPLET);
+  dnm_inserttuplet (gui, TRIPLET);
 }
 
 void
 start_triplet (DenemoGUI * gui)
 {
-  nextmeasure (gui->si, FALSE);
-  object_insert (gui->si, newtupopen(2,3));
+  insertion_point (gui->si);
+  object_insert (gui, newtupopen(2,3));
 }
 void
 end_tuplet (DenemoGUI * gui)
 {
-  object_insert (gui->si, newtupclose ());
+  object_insert (gui, newtupclose ());
 }
 void
 insert_quadtuplet (DenemoGUI * gui)
 {
-  dnm_inserttuplet (gui->si, QUADTUPLET);
+  dnm_inserttuplet (gui, QUADTUPLET);
 }
 
 void
 insert_quintuplet (DenemoGUI * gui)
 {
-  dnm_inserttuplet (gui->si, QUINTUPLET);
+  dnm_inserttuplet (gui, QUINTUPLET);
 }
 
 void
 insert_sextuplet (DenemoGUI * gui)
 {
-  dnm_inserttuplet (gui->si, SEXTUPLET);
+  dnm_inserttuplet (gui, SEXTUPLET);
 }
 
 void
 insert_septuplet (DenemoGUI * gui)
 {
-  dnm_inserttuplet (gui->si, SEPTUPLET);
+  dnm_inserttuplet (gui, SEPTUPLET);
 }
 
 
@@ -588,7 +553,7 @@ sharpen_key (DenemoGUI * gui)
   if (curmudelaobj && curmudelaobj->type == STEMDIRECTIVE)
     change_stem_directive (gui->si, DENEMO_STEMUP);
   else
-    incrementenshift (gui->si, 1);
+    incrementenshift (gui, 1);
 }
 
 void
@@ -600,7 +565,7 @@ flatten_key (DenemoGUI * gui)
   if (curmudelaobj && curmudelaobj->type == STEMDIRECTIVE)
     change_stem_directive (gui->si, DENEMO_STEMDOWN);
   else
-    incrementenshift (gui->si, -1);
+    incrementenshift (gui, -1);
 }
 /* insert a duplicate note and tie to it */
 void
@@ -613,8 +578,8 @@ tie_notes_key (DenemoGUI * gui)
   if (curmudelaobj && curmudelaobj->type == CHORD &&
       ((chord *) curmudelaobj->object)->notes)
     {
-      nextmeasure (gui->si, FALSE);
-      object_insert (gui->si, dnm_clone_object (curmudelaobj));
+      insertion_point (gui->si);
+      object_insert (gui, dnm_clone_object (curmudelaobj));
       ((chord *) curmudelaobj->object)->is_tied = TRUE;
     }
 }
@@ -852,7 +817,7 @@ add_arpeggio (DenemoGUI * gui)
 void
 set_grace (DenemoGUI * gui)
 {
-  insertgrace (gui->si);
+  insertgrace (gui);
 }
 
 void
@@ -885,31 +850,31 @@ change_pitch (DenemoGUI * gui)
 void
 insert_doublebar (DenemoGUI * gui)
 {
-  object_insert (gui->si, newbarline (barlinefromname (_("Double"))));
+  object_insert (gui, newbarline (barlinefromname (_("Double"))));
 }
 
 void
 insert_endbar (DenemoGUI * gui)
 {
-  object_insert (gui->si, newbarline (barlinefromname (_("End"))));
+  object_insert (gui, newbarline (barlinefromname (_("End"))));
 }
 
 void
 insert_openrepeat (DenemoGUI * gui)
 {
-  object_insert (gui->si, newbarline (barlinefromname (_("Open Repeat"))));
+  object_insert (gui, newbarline (barlinefromname (_("Open Repeat"))));
 }
 
 void
 insert_closerepeat (DenemoGUI * gui)
 {
-  object_insert (gui->si, newbarline (barlinefromname (_("Close Repeat"))));
+  object_insert (gui, newbarline (barlinefromname (_("Close Repeat"))));
 }
 
 void
 insert_opencloserepeat (DenemoGUI * gui)
 {
-  object_insert (gui->si, newbarline (barlinefromname (_("Double"))));
+  object_insert (gui, newbarline (barlinefromname (_("Double"))));
 }
 
 
@@ -919,7 +884,7 @@ insert_opencloserepeat (DenemoGUI * gui)
 void
 newcleftreble (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newclefobj (DENEMO_TREBLE_CLEF));
+  object_insert (gui, dnm_newclefobj (DENEMO_TREBLE_CLEF));
 }
 
 /**
@@ -928,7 +893,7 @@ newcleftreble (DenemoGUI * gui)
 void
 newclefbass (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newclefobj (DENEMO_BASS_CLEF));
+  object_insert (gui, dnm_newclefobj (DENEMO_BASS_CLEF));
 }
 
 /**
@@ -937,7 +902,7 @@ newclefbass (DenemoGUI * gui)
 void
 newclefalto (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newclefobj (DENEMO_ALTO_CLEF));
+  object_insert (gui, dnm_newclefobj (DENEMO_ALTO_CLEF));
 }
 
 /**
@@ -946,7 +911,7 @@ newclefalto (DenemoGUI * gui)
 void
 newclefg8 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newclefobj (DENEMO_G_8_CLEF));
+  object_insert (gui, dnm_newclefobj (DENEMO_G_8_CLEF));
 }
 
 /**
@@ -955,7 +920,7 @@ newclefg8 (DenemoGUI * gui)
 void
 newcleftenor (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newclefobj (DENEMO_TENOR_CLEF));
+  object_insert (gui, dnm_newclefobj (DENEMO_TENOR_CLEF));
 }
 
 /**
@@ -964,7 +929,7 @@ newcleftenor (DenemoGUI * gui)
 void
 newclefsoprano (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newclefobj (DENEMO_SOPRANO_CLEF));
+  object_insert (gui, dnm_newclefobj (DENEMO_SOPRANO_CLEF));
 }
 
 /**
@@ -973,7 +938,7 @@ newclefsoprano (DenemoGUI * gui)
 void
 newtimesig44 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (4, 4));
+  object_insert (gui, dnm_newtimesigobj (4, 4));
 }
 
 /**
@@ -982,7 +947,7 @@ newtimesig44 (DenemoGUI * gui)
 void
 newtimesig24 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (2, 4));
+  object_insert (gui, dnm_newtimesigobj (2, 4));
 }
 
 /**
@@ -991,7 +956,7 @@ newtimesig24 (DenemoGUI * gui)
 void
 newtimesig34 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (3, 4));
+  object_insert (gui, dnm_newtimesigobj (3, 4));
 }
 
 /**
@@ -1000,7 +965,7 @@ newtimesig34 (DenemoGUI * gui)
 void
 newtimesig64 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (6, 4));
+  object_insert (gui, dnm_newtimesigobj (6, 4));
 }
 
 /**
@@ -1009,7 +974,7 @@ newtimesig64 (DenemoGUI * gui)
 void
 newtimesig54 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (5, 4));
+  object_insert (gui, dnm_newtimesigobj (5, 4));
 }
 
 /**
@@ -1018,7 +983,7 @@ newtimesig54 (DenemoGUI * gui)
 void
 newtimesig38 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (3, 8));
+  object_insert (gui, dnm_newtimesigobj (3, 8));
 }
 
 /**
@@ -1027,7 +992,7 @@ newtimesig38 (DenemoGUI * gui)
 void
 newtimesig68 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (6, 8));
+  object_insert (gui, dnm_newtimesigobj (6, 8));
 }
 
 /**
@@ -1036,7 +1001,7 @@ newtimesig68 (DenemoGUI * gui)
 void
 newtimesig98 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (9, 8));
+  object_insert (gui, dnm_newtimesigobj (9, 8));
 }
 
 /**
@@ -1045,7 +1010,7 @@ newtimesig98 (DenemoGUI * gui)
 void
 newtimesig128 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (12, 8));
+  object_insert (gui, dnm_newtimesigobj (12, 8));
 }
 
 /**
@@ -1054,7 +1019,7 @@ newtimesig128 (DenemoGUI * gui)
 void
 newtimesig22 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (2, 2));
+  object_insert (gui, dnm_newtimesigobj (2, 2));
 }
 
 /**
@@ -1063,7 +1028,7 @@ newtimesig22 (DenemoGUI * gui)
 void
 newtimesig32 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (3, 2));
+  object_insert (gui, dnm_newtimesigobj (3, 2));
 }
 
 /**
@@ -1072,7 +1037,7 @@ newtimesig32 (DenemoGUI * gui)
 void
 newtimesig42 (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newtimesigobj (4, 2));
+  object_insert (gui, dnm_newtimesigobj (4, 2));
 }
 
 
@@ -1080,181 +1045,181 @@ newtimesig42 (DenemoGUI * gui)
 void
 newkeysigcmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (0, 1, 0));
+  object_insert (gui, dnm_newkeyobj (0, 1, 0));
 }
 
 void
 newkeysiggmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (1, 1, 0));
+  object_insert (gui, dnm_newkeyobj (1, 1, 0));
 }
 
 void
 newkeysigdmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (2, 1, 0));
+  object_insert (gui, dnm_newkeyobj (2, 1, 0));
 }
 
 void
 newkeysigamaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (3, 1, 0));
+  object_insert (gui, dnm_newkeyobj (3, 1, 0));
 }
 
 void
 newkeysigemaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (4, 1, 0));
+  object_insert (gui, dnm_newkeyobj (4, 1, 0));
 }
 
 void
 newkeysigbmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (5, 1, 0));
+  object_insert (gui, dnm_newkeyobj (5, 1, 0));
 }
 
 void
 newkeysigfsharpmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (6, 1, 0));
+  object_insert (gui, dnm_newkeyobj (6, 1, 0));
 }
 
 void
 newkeysigcsharpmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (7, 1, 0));
+  object_insert (gui, dnm_newkeyobj (7, 1, 0));
 }
 
 void
 newkeysigfmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-1, 1, 0));
+  object_insert (gui, dnm_newkeyobj (-1, 1, 0));
 }
 
 void
 newkeysigbflatmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-2, 1, 0));
+  object_insert (gui, dnm_newkeyobj (-2, 1, 0));
 }
 
 void
 newkeysigeflatmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-3, 1, 0));
+  object_insert (gui, dnm_newkeyobj (-3, 1, 0));
 }
 
 void
 newkeysigaflatmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-4, 1, 0));
+  object_insert (gui, dnm_newkeyobj (-4, 1, 0));
 }
 
 void
 newkeysigdflatmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-5, 1, 0));
+  object_insert (gui, dnm_newkeyobj (-5, 1, 0));
 }
 
 void
 newkeysiggflatmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-6, 1, 0));
+  object_insert (gui, dnm_newkeyobj (-6, 1, 0));
 }
 
 void
 newkeysigcflatmaj (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-7, 1, 0));
+  object_insert (gui, dnm_newkeyobj (-7, 1, 0));
 }
 
 void
 newkeysigamin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (0, 0, 0));
+  object_insert (gui, dnm_newkeyobj (0, 0, 0));
 }
 
 void
 newkeysigemin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (1, 0, 0));
+  object_insert (gui, dnm_newkeyobj (1, 0, 0));
 }
 
 void
 newkeysigbmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (2, 0, 0));
+  object_insert (gui, dnm_newkeyobj (2, 0, 0));
 }
 
 void
 newkeysigfsharpmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (3, 0, 0));
+  object_insert (gui, dnm_newkeyobj (3, 0, 0));
 }
 
 void
 newkeysigcsharpmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (4, 0, 0));
+  object_insert (gui, dnm_newkeyobj (4, 0, 0));
 }
 
 void
 newkeysiggsharpmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (5, 0, 0));
+  object_insert (gui, dnm_newkeyobj (5, 0, 0));
 }
 
 void
 newkeysigdsharpmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (6, 0, 0));
+  object_insert (gui, dnm_newkeyobj (6, 0, 0));
 }
 
 void
 newkeysigasharpmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (7, 0, 0));
+  object_insert (gui, dnm_newkeyobj (7, 0, 0));
 }
 
 void
 newkeysigdmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-1, 0, 0));
+  object_insert (gui, dnm_newkeyobj (-1, 0, 0));
 }
 
 void
 newkeysiggmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-2, 0, 0));
+  object_insert (gui, dnm_newkeyobj (-2, 0, 0));
 }
 
 void
 newkeysigcmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-3, 0, 0));
+  object_insert (gui, dnm_newkeyobj (-3, 0, 0));
 }
 
 void
 newkeysigfmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-4, 0, 0));
+  object_insert (gui, dnm_newkeyobj (-4, 0, 0));
 }
 
 void
 newkeysigbflatmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-5, 0, 0));
+  object_insert (gui, dnm_newkeyobj (-5, 0, 0));
 }
 
 void
 newkeysigeflatmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-6, 0, 0));
+  object_insert (gui, dnm_newkeyobj (-6, 0, 0));
 }
 
 void
 newkeysigaflatmin (DenemoGUI * gui)
 {
-  object_insert (gui->si, dnm_newkeyobj (-7, 0, 0));
+  object_insert (gui, dnm_newkeyobj (-7, 0, 0));
 }
 
 
