@@ -42,7 +42,7 @@ newlyric (gint baseduration, gint numdots, gchar * lys)
 
 struct callbackdata
 {
-  DenemoScore *si;
+  DenemoGUI *gui;
   GtkWidget *entry;
   GtkWidget *extender;
   GtkWidget *center;
@@ -58,7 +58,8 @@ void
 insertlyric (gpointer data)
 {
   struct callbackdata *cbdata = (struct callbackdata *) data;
-  DenemoScore *si = (DenemoScore *) cbdata->si;
+  DenemoGUI *gui = cbdata->gui;
+  DenemoScore *si = gui->si;
   DenemoObject *curObj = (DenemoObject *) (si->currentobject ?
 					   si->currentobject->data : NULL);
   gchar *lyric = (gchar *) gtk_entry_get_text (GTK_ENTRY (cbdata->entry));
@@ -74,9 +75,20 @@ insertlyric (gpointer data)
       ((chord *) curObj->object)->center_lyric =
 	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cbdata->center));
     }
-
-  si->haschanged = TRUE;
-
+    do
+      {
+	if (si->currentobject->next)
+	  cursorright (gui);
+	else if (gui->si->currentmeasure->next)
+	  measureright (gui);
+	else 
+	  break;
+	curObj =
+	  si->currentobject ? (DenemoObject *) si->currentobject->data : NULL;
+      }
+    while ((curObj != NULL) && 
+	   ((curObj->type != CHORD)||
+	     ((curObj->type == CHORD)&& (((chord *) curObj->object)->notes==NULL))));
 }
 
 /**
@@ -134,7 +146,7 @@ lyric_insert (GtkAction * action, DenemoGUI * gui)
   gtk_widget_show (center);
 
 
-  cbdata.si = si;
+  cbdata.gui = gui;
   cbdata.entry = entry;
   cbdata.extender = extender;
   cbdata.center = center;
@@ -149,6 +161,8 @@ lyric_insert (GtkAction * action, DenemoGUI * gui)
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
     {
       insertlyric (&cbdata);
+      ((DenemoStaff*)si->currentstaff->data)->haslyrics=TRUE;
+      score_status(gui, TRUE);
       displayhelper (gui);
     }
   gtk_widget_destroy (dialog);
