@@ -88,33 +88,7 @@ static const GtkStockItem denemo_stock_items[] = {
   {"denemo-upprall", N_("Upprall"), (GdkModifierType) 0, 0, NULL},
   {"denemo-arpeggio", N_("Arpeggio"), (GdkModifierType) 0, 0, NULL}
 };
-static void
-load_accels () {
-#define STANDARD_ACCELS "standard.accels"
-  gchar * dotdenemo = locatedotdenemo ();
-  gchar *filename = dotdenemo?g_build_filename(locatedotdenemo(), STANDARD_ACCELS, NULL):NULL;
-  gchar *systemwide = g_build_filename (get_data_dir (), STANDARD_ACCELS, NULL);
-#undef STANDARD_ACCELS
-  if(filename) {
-    if(g_file_test(filename,  G_FILE_TEST_EXISTS))
-      gtk_accel_map_load (filename);
-    else {
-      gtk_accel_map_load (systemwide);
-      //gtk_accel_map_save (filename);
-    }
-  } else
-    gtk_accel_map_load (systemwide);
-   if(filename) 
-     g_free(filename);
 
-   filename = dotdenemo?g_build_filename(locatedotdenemo(), EXTRA_ACCELS, NULL):NULL;
-   if(filename) {
-     if(g_file_test(filename,  G_FILE_TEST_EXISTS))
-       gtk_accel_map_load (filename);
-   }
-   if(filename) 
-     g_free(filename);
-}
 static void
 register_stock_icon (GtkIconFactory * icon_factory, const gchar * stock_id,
                      const gchar * file)
@@ -142,6 +116,40 @@ register_stock_icon (GtkIconFactory * icon_factory, const gchar * stock_id,
     gtk_icon_factory_add (icon_factory, stock_id, icon_set);
     g_free (path);
   } 
+  //FIXME  path may leak
+}
+
+// removes the accel of a standard stock item. We use this because we want to
+// handle all the keybindings in denemo's keymap.
+// Also, removes the _ in the label of the accel so that no mnemonic is defined
+static void
+clean_stock_item(const gchar *stock_id)
+{
+    gint i, j;
+    GtkStockItem stock;
+    gchar *label;
+    if (gtk_stock_lookup(stock_id, &stock)) {
+        label = g_strdup(stock.label);
+        if (stock.label != NULL) {
+            i = 0;
+            j = 0;
+            while (stock.label[i]) {
+                if (stock.label[i] == '_')
+                    i++;
+                else {
+                    label[j] = stock.label[i];
+                    i++;
+                    j++;
+                }
+            }
+            label[j] = '\0';
+        }
+        stock.label = label;
+        stock.keyval = 0;
+        stock.modifier = 0;
+        gtk_stock_add(&stock, 1);
+        g_free(label);
+    }
 }
 
 static void
@@ -149,10 +157,11 @@ register_stock_items ()
 {
   GtkIconFactory *icon_factory;
 
-  /* Load stock items (icons) */
+  /* Load stock items that denemo defines*/
   gtk_stock_add_static (denemo_stock_items,
                         G_N_ELEMENTS (denemo_stock_items));
 
+  /* Load stock icons for the new stock items*/
   icon_factory = gtk_icon_factory_new ();
   gtk_icon_factory_add_default (icon_factory);
   register_stock_icon (icon_factory, "denemo-staccato", "staccato.svg");
@@ -213,6 +222,22 @@ register_stock_items ()
                        "feta26-scripts-upprall.xbm");
   register_stock_icon (icon_factory, "denemo-arpeggio",
                        "feta26-scripts-arpeggio.xbm");
+
+  //remove accelerators from some gtk standard stock items
+  clean_stock_item(GTK_STOCK_NEW);
+  clean_stock_item(GTK_STOCK_OPEN);
+  clean_stock_item(GTK_STOCK_SAVE);
+  clean_stock_item(GTK_STOCK_CLOSE);
+  clean_stock_item(GTK_STOCK_QUIT);
+  clean_stock_item(GTK_STOCK_CUT);
+  clean_stock_item(GTK_STOCK_COPY);
+  clean_stock_item(GTK_STOCK_PASTE);
+  clean_stock_item(GTK_STOCK_GOTO_FIRST);
+  clean_stock_item(GTK_STOCK_GOTO_LAST);
+  clean_stock_item(GTK_STOCK_ADD);
+  clean_stock_item(GTK_STOCK_REMOVE);
+  clean_stock_item(GTK_STOCK_FIND);
+
   g_object_unref (icon_factory);
 }
 
@@ -322,9 +347,9 @@ denemo_signal_handler (int sig)
       g_debug ("si is %p", gui);
       gchar *filename = g_build_filename(locatedotdenemo (), 
                                          "crashrecovery.denemo", NULL);
-
+      gui->si->markstaffnum = 0;
       if (gui->si->lily_file)
-        exportlilypond (filename, gui, 0, 0, 1);
+        exportlilypond (filename, gui, TRUE);
       else
         exportXML (filename, gui, 0, 0);
 
@@ -342,8 +367,9 @@ denemo_signal_handler (int sig)
           sprintf (t, "%d", i);
           strncat (filename, t, strlen (t));
           strcat (filename, ".denemo");
+	  gui->si->markstaffnum = 0;
           if (gui->si->lily_file)
-            exportlilypond (filename, gui, 0, 0, 1);
+            exportlilypond (filename, gui, TRUE);
           else
             exportXML (filename, gui, 0, 0);
           i++;
@@ -464,9 +490,19 @@ COPYING for details.\n\n");
 
   /* gtk initialization */
   gtk_init (&argc, &argv);
-  load_accels();
+  /* adapt stock items to denemo */
   register_stock_items ();
-
+  /* Following calls were made previously in newview. However I think they are
+   * global, and should be done once and for all when the application opens
+   */
+  /* Initialize preferences */
+  initprefs();
+  /* read history file */
+  readHistory();
+  /* Set up the keymap */
+  init_keymap();
+  
+  /* create the first window */
   newview ();
 
 
