@@ -123,35 +123,29 @@ freeobjlist (gpointer data, gpointer user_data)
 void
 staffremovemeasures (staffnode * curstaff, guint pos)
 {
-  g_print ("In Staffremovemeasures\n");
-
+  //g_print ("In Staffremovemeasures\n");
   measurenode *firstmeasure;
   measurenode *delmeasure;
-
   if (((DenemoStaff *) curstaff->data)->is_parasite)
     {
       ((DenemoStaff *) curstaff->data)->measures =
 	*((DenemoStaff *) curstaff->data)->is_parasite;
     }
-
-
   firstmeasure = firstmeasurenode (curstaff);
   delmeasure = g_list_nth (firstmeasure, pos);
+  if(delmeasure) {
 #ifdef DEBUG
-  g_print ("Firstmeasure %x\t DelMeasure %x \t Position\n",
-	   firstmeasure, delmeasure, pos);
+    g_print ("Firstmeasure %x\t DelMeasure %x \t Position\n",
+	     firstmeasure, delmeasure, pos);
 #endif
-
-  freeobjlist (delmeasure->data, NULL);
-  ((DenemoStaff *) curstaff->data)->measures =
-    g_list_remove_link (firstmeasure, delmeasure);
-  g_list_free_1 (delmeasure);
-
-  ((DenemoStaff *) curstaff->data)->nummeasures--;
-
-
-
-
+    
+    freeobjlist (delmeasure->data, NULL);
+    ((DenemoStaff *) curstaff->data)->measures =
+      g_list_remove_link (firstmeasure, delmeasure);
+    g_list_free_1 (delmeasure);
+    
+    ((DenemoStaff *) curstaff->data)->nummeasures--;
+  }
 }
 
 /**
@@ -177,19 +171,22 @@ removemeasures (DenemoScore * si, guint pos, guint nummeasures, gboolean all)
       g_list_length (firstmeasurenode ((staffnode *) si->currentstaff)) - pos)
   {
 		for (i = 0; i < nummeasures; i++)
-	  {
-		  if (all)
 		  {
-			  for (curstaff = si->thescore; curstaff;
-				  curstaff = curstaff->next)
-			  {
-				  staffremovemeasures (curstaff, pos);
-			  }
-		  }
-		  else
-		  {
-			  staffremovemeasures ((staffnode *) si->currentstaff, pos);
-		  }
+		    
+		    
+		    for (curstaff = si->thescore; curstaff;
+			 curstaff = curstaff->next)
+		      {
+			if (curstaff==si->currentstaff || all) {
+			  staffremovemeasures (curstaff, pos);
+			  if(!firstmeasurenode(curstaff)) {
+			    ((DenemoStaff *) curstaff->data)->measures =
+			      g_list_append (NULL, NULL);
+			    ((DenemoStaff *) curstaff->data)->nummeasures=1;
+			  } 	    
+			}
+		      }
+		 
 		  
 			for(curstaff = si->thescore; curstaff; curstaff = curstaff->next)
 			{
@@ -211,14 +208,9 @@ removemeasures (DenemoScore * si, guint pos, guint nummeasures, gboolean all)
                    than exist.  Junking request."));
       return si->currentmeasure;
     }
-  if (!(firstmeasure = firstmeasurenode (si->currentstaff)))
-    {
-      /* We've deleted all the measures - add a new one again */
-      return addmeasures (si, 0, 1, TRUE);
-    }
-
-  else if (pos ==
-	   g_list_length (firstmeasurenode ((staffnode *) si->currentstaff)))
+  firstmeasure = firstmeasurenode (si->currentstaff);
+  if (pos ==
+      g_list_length (firstmeasurenode ((staffnode *) si->currentstaff)))
     {
       /* That is, we deleted the last measure */
       si->currentmeasurenum--;
@@ -666,6 +658,9 @@ showwhichaccidentals (objnode * theobjs, gint initialnum, gint * initialaccs)
 		/* A new accidental not present in the original chord */
 		thetone->showaccidental =
 		  ((chord *) theobj->object)->hasanacc = TRUE;
+	      else if (thetone->directive && (*thetone->directive->str=='!'|| *thetone->directive->str=='?'))
+		thetone->showaccidental =
+		  ((chord *) theobj->object)->hasanacc = TRUE;
 	      else
 		thetone->showaccidental = FALSE;
 	    }			/* End second loop through chord */
