@@ -13,23 +13,25 @@
 #include "staffops.h"
 #include <string.h>
 
+void
+initkeyaccs (gint * accs, gint number);
 /**
  * Free the given object
- * @param mudobj the mudelaobject to free
+ * @param mudobj the DenemoObject to free
  */
 void
-freeobject (mudelaobject * mudobj)
+freeobject (DenemoObject * mudobj)
 {
   if (mudobj)
     switch (mudobj->type)
-    {
-    	case CHORD:
-				freechord (mudobj);	/* Which also frees mudobj itself */
-				break;
+      {
+      case CHORD:
+	freechord (mudobj);	/* Which also frees mudobj itself */
+	break;
       default:
-				g_free (mudobj);
-				break;
-    }
+	g_free (mudobj);
+	break;
+      }
 }
 
 /**
@@ -38,12 +40,12 @@ freeobject (mudelaobject * mudobj)
  * @param time2 denominator of the time signature
  * @return the timesignature
  */
-mudelaobject *
-newtimesigobj (gint time1, gint time2)
+DenemoObject *
+dnm_newtimesigobj (gint time1, gint time2)
 {
-  mudelaobject *ret;
+  DenemoObject *ret;
   timesig *newtimesig = (timesig *) g_malloc (sizeof (timesig));
-  ret = (mudelaobject *) g_malloc (sizeof (mudelaobject));
+  ret = (DenemoObject *) g_malloc (sizeof (DenemoObject));
   ret->type = TIMESIG;
   newtimesig->time1 = time1;
   newtimesig->time2 = time2;
@@ -61,12 +63,12 @@ newtimesigobj (gint time1, gint time2)
  *
  * @return the clef 
  */
-mudelaobject *
-newclefobj (enum clefs type)
+DenemoObject *
+dnm_newclefobj (enum clefs type)
 {
-  mudelaobject *ret;
+  DenemoObject *ret;
   clef *newclef = (clef *) g_malloc (sizeof (clef));
-  ret = (mudelaobject *) g_malloc (sizeof (mudelaobject));
+  ret = (DenemoObject *) g_malloc (sizeof (DenemoObject));
   ret->type = CLEF;
   ret->isinvisible = FALSE;
   newclef->type = type;
@@ -77,36 +79,38 @@ newclefobj (enum clefs type)
 }
 
 
-void set_modeaccs(gint *accs, gint number, gint mode)
-{ 
-	
-	g_print("Mode %d : %d \n", number, mode);
-	if(mode == 0)
+void
+set_modeaccs (gint * accs, gint number, gint mode)
+{
+
+  g_print ("Mode %d : %d \n", number, mode);
+  if (mode == 0)
+    {
+      switch (number)
 	{
-	  switch (number) 
-	  {
-	  case 11:
-	  	number -= 7;
-	  	break;
-	  }
-	  initkeyaccs(accs,number);
-	  
+	case 11:
+	  number -= 7;
+	  break;
 	}
-	else if(mode == 2)
-	  initkeyaccs(accs,0);
-	else if(number == 1 && mode == 1)
-	  initkeyaccs(accs,0);
-	else if(number == 1 && mode == 1)
-	  initkeyaccs(accs,0);
-	else if(number == 1 && mode == 1)	
-	  initkeyaccs(accs,0);
-	else if(number == 1 && mode == 1)
-	  initkeyaccs(accs,0);
-	else if(number == 1 && mode == 1)
-	  initkeyaccs(accs,0);
-	else if(number == 1 && mode == 1)
-	  initkeyaccs(accs,0);
+      initkeyaccs (accs, number);
+
+    }
+  else if (mode == 2)
+    initkeyaccs (accs, 0);
+  else if (number == 1 && mode == 1)
+    initkeyaccs (accs, 0);
+  else if (number == 1 && mode == 1)
+    initkeyaccs (accs, 0);
+  else if (number == 1 && mode == 1)
+    initkeyaccs (accs, 0);
+  else if (number == 1 && mode == 1)
+    initkeyaccs (accs, 0);
+  else if (number == 1 && mode == 1)
+    initkeyaccs (accs, 0);
+  else if (number == 1 && mode == 1)
+    initkeyaccs (accs, 0);
 }
+
 /** 
  * This function initializes the accidental-context array associated with
  * a key signature or a staff to that appropriate for _number_ 
@@ -123,11 +127,7 @@ initkeyaccs (gint * accs, gint number)
   else if (number < 0)
     for (index = 6; number; number++, index = (index + 3) % 7)
       accs[index] = -1;
-#ifdef DEBUG
-  for(i=0; i < 7; i++)
- 	g_print("[%d]", accs[i]);
-  g_print("\n");
-#endif
+
 }
 
 /**
@@ -135,26 +135,29 @@ initkeyaccs (gint * accs, gint number)
  * 
  * @param number number of keysignature
  * @param isminor signifies if the key sig should be minor
+ * @param mode    description of the keys mode
  * @return the key signature
  */
-mudelaobject *
-newkeyobj (gint number, gint isminor, gint mode)
+DenemoObject *
+dnm_newkeyobj (gint number, gint isminor, gint mode)
 {
-  mudelaobject *ret;
+  DenemoObject *ret;
   keysig *newkeysig = (keysig *) g_malloc (sizeof (keysig));
-  ret = (mudelaobject *) g_malloc (sizeof (mudelaobject));
+  ret = (DenemoObject *) g_malloc (sizeof (DenemoObject));
   ret->type = KEYSIG;
   ret->isinvisible = FALSE;
-  
-  g_print("Number %d \t IsMinor %d \t Mode %d\n", number, isminor, mode);
+#ifdef DEBUG
+  g_print ("Number %d \t IsMinor %d \t Mode %d\n", number, isminor, mode);
+#endif
+
   newkeysig->mode = mode;
   newkeysig->number = number;
   newkeysig->isminor = isminor;
-  
-  
-  
-  if(isminor == 2)
-    set_modeaccs(newkeysig->accs, number, mode);
+
+
+
+  if (isminor == 2)
+    set_modeaccs (newkeysig->accs, number, mode);
   else
     initkeyaccs (newkeysig->accs, number);
 
@@ -167,37 +170,55 @@ newkeyobj (gint number, gint isminor, gint mode)
 /**
  * Create a new measure break object
  * @return the measurebreak
- */ 
-mudelaobject *
+ */
+DenemoObject *
 newmeasurebreakobject ()
 {
-  mudelaobject *ret;
+  DenemoObject *ret;
 
-  ret = (mudelaobject *) g_malloc (sizeof (mudelaobject));
+  ret = (DenemoObject *) g_malloc (sizeof (DenemoObject));
   ret->type = MEASUREBREAK;
   return ret;
 }
 
+/**
+ * Create a new staff break object
+ * @return the staffbreak
+ */
+DenemoObject *
+newstaffbreakobject ()
+{
+  DenemoObject *ret;
+  ret = (DenemoObject *) g_malloc (sizeof (DenemoObject));
+  ret->type = STAFFBREAK;
+  return ret;
+}
+
+
 /**
  * Create a clone of the given object
  * @param orig the object to clone
  * @return the cloned object
  */
-mudelaobject *
-clone_object (mudelaobject * orig)
+DenemoObject *
+dnm_clone_object (DenemoObject * orig)
 {
-  mudelaobject *ret;
-
-  switch (orig->type)
-    {
-    case CHORD:
-      ret = clone_chord (orig);
-      break;
-    default:
-      ret = (mudelaobject *) g_malloc (sizeof (mudelaobject));
-      memcpy (ret, orig, sizeof (mudelaobject));
-      break;
-    }
+  DenemoObject *ret;
+  if (orig != NULL) {
+	  switch (orig->type)
+	    {
+	    case CHORD:
+	      ret = clone_chord (orig);
+	      break;
+	    case LILYDIRECTIVE:
+	      ret = lily_directive_new(((lilydirective*)orig->object)->directive->str);
+	      break;
+	    default:
+	      ret = (DenemoObject *) g_malloc (sizeof (DenemoObject));
+	      memcpy (ret, orig, sizeof (DenemoObject));//dangerous ret->object is duplicated pointer
+	      break;
+	    }
+  }
   return ret;
 }
 
@@ -206,13 +227,13 @@ clone_object (mudelaobject * orig)
  *  @param type the stem directive type
  *  @return the stem directive
  */
-mudelaobject *
-stem_directive_new (enum stemdirections type)
+DenemoObject *
+dnm_stem_directive_new (enum stemdirections type)
 {
-  mudelaobject *ret;
+  DenemoObject *ret;
   stemdirective *newstemdir =
     (stemdirective *) g_malloc (sizeof (stemdirective));
-  ret = (mudelaobject *) g_malloc (sizeof (mudelaobject));
+  ret = (DenemoObject *) g_malloc (sizeof (DenemoObject));
   ret->type = STEMDIRECTIVE;
   ret->isinvisible = FALSE;
   newstemdir->type = type;
@@ -228,13 +249,14 @@ stem_directive_new (enum stemdirections type)
  * @param type the lilypond directive body
  * @return the lilypond directive
  *
-mudelaobject *
+*/
+DenemoObject *
 lily_directive_new (gchar * type)
 {
-  mudelaobject *ret;
+  DenemoObject *ret;
   lilydirective *newlily =
     (lilydirective *) g_malloc (sizeof (lilydirective));
-  ret = (mudelaobject *) g_malloc (sizeof (mudelaobject));
+  ret = (DenemoObject *) g_malloc (sizeof (DenemoObject));
   ret->type = LILYDIRECTIVE;
   newlily->directive = g_string_new (type);
   ret->object = newlily;
@@ -242,18 +264,19 @@ lily_directive_new (gchar * type)
   setpixelmin (ret);
   return ret;
 }
-*/
+
+
 /**
  * Create a new dynamic object
  * @param type the dynamic to create
  * @return the dynamic
   */
-mudelaobject *
+DenemoObject *
 dynamic_new (gchar * type)
 {
-  mudelaobject *ret;
+  DenemoObject *ret;
   dynamic *newdyn = (dynamic *) g_malloc (sizeof (dynamic));
-  ret = (mudelaobject *) g_malloc (sizeof (mudelaobject));
+  ret = (DenemoObject *) g_malloc (sizeof (DenemoObject));
   ret->type = DYNAMIC;
   ret->isinvisible = FALSE;
   newdyn->type = g_string_new (type);
@@ -270,12 +293,12 @@ dynamic_new (gchar * type)
  * @param syllable whether it is a syllable
  * @return the dynamic
   */
-mudelaobject *
-lyric_new (gchar * type, gint position, gboolean syllable)
+DenemoObject *
+dnm_lyric_new (gchar * type, gint position, gboolean syllable)
 {
-  mudelaobject *ret;
+  DenemoObject *ret;
   lyric *newlyric = (lyric *) g_malloc (sizeof (lyric));
-  ret = (mudelaobject *) g_malloc (sizeof (mudelaobject));
+  ret = (DenemoObject *) g_malloc (sizeof (DenemoObject));
   ret->type = LYRIC;
   ret->isinvisible = FALSE;
 
@@ -290,43 +313,40 @@ lyric_new (gchar * type, gint position, gboolean syllable)
 }
 
 /**
- *  Create a mudelaobject
- * @param type mudelaobject type 
- * @return the mudelaobject
+ *  Create a DenemoObject
+ * @param type DenemoObject type 
+ * @return the DenemoObject
  */
 
-mudelaobject *
-newobj (enum objtype type)
+DenemoObject *
+dnm_newobj (DenemoObjType type)
 {
-  mudelaobject *ret = (mudelaobject *) g_malloc0 (sizeof (mudelaobject));;
+  DenemoObject *ret = (DenemoObject *) g_malloc0 (sizeof (DenemoObject));;
   ret->type = type;
   set_basic_numticks (ret);
-  setpixelmin (ret);		/* these do nothing at present - but if we introduce
-				   a show markers option then we will want to allot some space
-				 */
+  setpixelmin (ret);  /* these do nothing at present - but if we introduce
+			 a show markers option then we will want to allot 
+			 some space
+		      */
   return ret;
 }
 
-
-
-
-
-
-
-
-
 /**
  * Set the key signature into the score
- * @param si the score
+ * @param curstaff the current staff
  * @param tokey the key sig to insert
  * @param type  major/minor/mode
  */
-void setinitialkeysig(staff *curstaff, gint tokey, gint type)
+void
+dnm_setinitialkeysig (DenemoStaff * curstaff, gint tokey, gint type)
 {
-	
-	curstaff->skey = tokey;
-	curstaff->skey_isminor = type;
-	initkeyaccs(curstaff->skeyaccs, tokey);
-   showwhichaccidentalswholestaff (curstaff);
-	
+
+  curstaff->skey = tokey;
+  curstaff->skey_isminor = type;
+
+  initkeyaccs (curstaff->leftmost_keyaccs, tokey);
+  memcpy (curstaff->skeyaccs, curstaff->leftmost_keyaccs, SEVENGINTS);
+
+  showwhichaccidentalswholestaff (curstaff);
+
 }
