@@ -110,7 +110,36 @@ static temperament Equal = {
   }
 };
 
- 
+static temperament Lehman = {
+  "Lehman Temperament", 8,3,
+    { 
+      {262.37*1     , {0, 0}},
+      {262.37*1.0590, {0, 1}},
+      {262.37*1.1203, {1, 0}},
+      {262.37*1.1889, {2, -1}},
+      {262.37*1.2551, {2, 0}},
+      {262.37*1.3360, {3, 0}},
+      {262.37*1.4120, {3, 1}},
+      {262.37*1.4968, {4, 0}},
+      {262.37*1.5869, {4, 1}},
+      {262.37*1.6770, {5, 0}},
+      {262.37*1.7816, {6, -1}},
+      {262.37*1.8827, {6, 0}}
+  }
+};
+
+		
+ 	 	
+	 	
+ 	 	
+	 	
+	 	
+ 	 	
+	 	
+ 	 	
+	 	
+ 	 	
+	 	
 
 static temperament Meantone = {
   "Quarter comma meantone", 8,3,
@@ -151,7 +180,7 @@ static temperament WerckmeisterIV = {
 
 static temperament *PR_temperament; /* the currently used temperament */
 
-static temperament *temperaments[] = {&Equal, &Meantone, &WerckmeisterIV};
+static temperament *temperaments[] = {&Equal, &Meantone, &WerckmeisterIV, &Lehman};
 
 static void switch_back_to_main_window(void) {
   if(PR_gui)
@@ -336,9 +365,8 @@ static gboolean  apply_tones(DenemoScore *si) {
 	  if(thetone==NULL || store_el==NULL)
 	    ((chord*)theobj->object)->tone_node = NULL;
 	  else {
-	    find_leftmost_staffcontext (curstaff, si);
-	    int dclef = curstaff->leftmost_clefcontext;	    
-	    int mid_c_offset = thetone->step;
+	    gint dclef  = find_prevailing_clef(si);
+	    gint mid_c_offset = thetone->step;
 	    ((chord*)theobj->object)->tone_node = store_el;
 	    modify_note((chord*)theobj->object, mid_c_offset, thetone->enshift, dclef);
 	    tone_stored=TRUE;
@@ -693,9 +721,11 @@ static void stop_PR_controls(void) {
   PR_window = NULL;
 }
 
-/* stop the pitch recognition subsystem and optionally destroy the window
-   that controls it */
-static int stop_pitch_recognition_and_window(gboolean stop_window) {
+
+/* stop_pitch_recognition
+   stop audio and aubio
+ */
+int stop_pitch_recognition(void) {
   if(PR_gui==NULL)// when closing a second gui after the first has dismissed PR
     return;
   if(PR_timer)
@@ -707,19 +737,24 @@ static int stop_pitch_recognition_and_window(gboolean stop_window) {
   if(PR_leave) 
     g_signal_handler_disconnect (PR_gui->scorearea, PR_leave);
    PR_timer = PR_enter = PR_leave = 0;
-  if(stop_window)
-    stop_PR_controls();// which calls terminate_pitch_recognition 
-  else
-    terminate_pitch_recognition();
-  PR_window = NULL;
 
+   terminate_pitch_recognition();
+   if(PR_window) { 
+     GtkWidget *temp = PR_window; PR_window = NULL, gtk_widget_destroy(temp);
+   }
   PR_gui = NULL;
 
   return 0;
 }
 
-static stop_pitch_recognition_callback(){
-  stop_pitch_recognition_and_window(FALSE);
+static gboolean stop_pitch_recognition_callback(GtkWidget *unused, DenemoGUI *gui){
+if(PR_gui==NULL)
+    return FALSE;
+GtkWidget * widget = gtk_ui_manager_get_widget (gui->ui_manager, "/RhythmToolBar/TogglePitchRecognition");
+ PR_window=NULL;
+ gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON(widget),  FALSE);
+ clear_tone_store(NULL, gui);
+ return FALSE;
 }
 
 static stop_tuning_callback(){
@@ -839,7 +874,7 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
 
   gtk_window_set_title (GTK_WINDOW (PR_window), "Pitch Recognition Control");
   g_signal_connect (G_OBJECT (PR_window), "destroy",
-		    G_CALLBACK (stop_pitch_recognition_callback), NULL); 
+		    G_CALLBACK (stop_pitch_recognition_callback), gui); 
   GtkWidget *main_vbox = gtk_vbox_new (FALSE, 1);
   gtk_container_border_width (GTK_CONTAINER (main_vbox), 1);
   gtk_container_add (GTK_CONTAINER (PR_window), main_vbox);
@@ -1164,15 +1199,6 @@ gint setup_pitch_recognition(DenemoGUI *gui){
 
 
 
-/* stop_pitch_recognition
-   stop audio and aubio
-   param STOP_WINDOW means destroy the PR_window, otherwise it is
-   being called from the destroy callback of PR_window itself */
-int stop_pitch_recognition(void) {
-  return stop_pitch_recognition_and_window(TRUE);
-
-}
-
 static void 
 scorearea_set_active(GtkWidget *widget, GdkEventCrossing *event, DenemoGUI *gui) {
   PR_enable = TRUE; 
