@@ -10,18 +10,19 @@
 #include <string.h>
 
 void
-silentconversion (char *file_in, DenemoScore * si)
+silentconversion (char *file_in, DenemoGUI *gui)
 {
   int result;
   char *file_out;
+  DenemoScore *si=gui->si;
 
   if (strcmp (file_in + strlen (file_in) - 7, ".denemo") == 0)
     {
       printf ("\nConverting file %s", file_in);
-      result = (int) importXML (file_in, si);
+      result = (int) importXML (file_in, gui, REPLACE_SCORE);
     }
 
   file_out = "silentconversion.ly";
-  exportlilypond ((gchar *) file_out, si, 0, 0);
+  exportlilypond ((gchar *) file_out, si, TRUE);
   printf ("\nWritten to %s\n\n", file_out);
 };
