@@ -6,6 +6,7 @@
 
 #include <denemo/denemo.h>
 #include <stdio.h>
+#include "config.h"
 #include "staffops.h"
 #include "scoreops.h"
 #include "prefops.h"
@@ -34,7 +35,7 @@ void
 new_score (DenemoGUI *gui)
 {
   new_empty_score(gui);
-  newstaff (gui->si, INITIAL, DENEMO_NONE);
+  newstaff (gui, INITIAL, DENEMO_NONE);
 }
 
 
@@ -70,11 +71,13 @@ insert_movement_after(GtkAction * action, DenemoGUI *gui) {
 
 void
 delete_movement(GtkAction * action, DenemoGUI *gui) {
+  if(!confirm_deletestaff_custom_scoreblock(gui))
+    return;
   GString *primary = g_string_new(""), *secondary = g_string_new("");
   if(gui->movements==NULL || (g_list_length(gui->movements)==1)) {
     g_string_assign(primary, N_("This is the only movement"));
     g_string_assign(secondary, N_("Delete it and start over?"));
-    if(confirm(gui, primary->str, secondary->str)) {
+    if(confirm(primary->str, secondary->str)) {
       gchar *name = g_strdup(gui->filename->str);
       deletescore (NULL, gui);
       g_string_assign(gui->filename, name);
@@ -84,8 +87,8 @@ delete_movement(GtkAction * action, DenemoGUI *gui) {
     gint num = g_list_index(gui->movements, gui->si);
     g_string_printf(primary,N_("This is movement #%d, entitled %s"), num+1, gui->si->headerinfo.piece->str);
     g_string_assign(secondary,N_("Delete entire movement?"));
-    if( confirm(gui, primary->str, secondary->str)) {
-      free_score(gui->si);
+    if( confirm(primary->str, secondary->str)) {
+      free_score(gui);
       DenemoScore *si= gui->si;
       GList *g = g_list_find(gui->movements, si)->next;
       if(g==NULL)
@@ -100,7 +103,47 @@ delete_movement(GtkAction * action, DenemoGUI *gui) {
   score_status(gui, TRUE);
 }
 
+/* go to the movement voice measure staff and object numbers (starting from 1) */
+gboolean
+goto_movement_staff_obj (DenemoGUI * gui, gint movementnum, gint staffnum, gint measurenum, gint objnum)
+{
+  GList *this = g_list_nth(gui->movements, movementnum-1);
 
+  if(this==NULL){
+    warningdialog(N_("No such movement"));
+    return FALSE;
+  }
+  gui->si = this->data;
+  if(!set_currentstaffnum (gui, staffnum))
+    {
+      warningdialog(N_("No such voice"));
+      return FALSE;
+    }
+  if(!set_currentmeasurenum(gui, measurenum))
+  {
+    warningdialog(N_("No such measure"));
+    return FALSE;
+  }
+  while(--objnum>0 && gui->si->currentobject->next) {
+    gui->si->currentobject = gui->si->currentobject->next;
+    gui->si->cursor_x++;
+  }
+  if(objnum)
+    return FALSE;
+#if 0
+  //something bad here re-sets the staff to 1
+  updatescoreinfo (gui);
+  set_rightmeasurenum (gui->si);
+  find_leftmost_allcontexts (gui->si);
+  set_bottom_staff (gui);
+#endif
+  update_hscrollbar (gui);
+  update_vscrollbar (gui);
+  gtk_signal_emit_by_name (GTK_OBJECT (gui->hadjustment), "changed");
+  gtk_signal_emit_by_name (GTK_OBJECT (gui->vadjustment), "changed");
+  gtk_widget_queue_draw (gui->scorearea);
+  return TRUE;
+}
 
 
 /**
@@ -218,11 +261,12 @@ init_score (DenemoScore * si, DenemoGUI *gui)
   HEADER(head);
   HEADER(copyright);
   HEADER(footer);
-  HEADER(tagline);
+
+  si->headerinfo.tagline = g_string_new ("Generated by Denemo Version "VERSION", go to http://www.denemo.org");
 
   si->headerinfo.extra = g_string_new ("breakbefore = ##f");
-  HEADER(markup_before);
-  HEADER(markup_after);
+  HEADER(lilypond_before);
+  HEADER(lilypond_after);
 #undef HEADER
   if(gui->filename==NULL) {
     gui->filename = g_string_new ("");
@@ -253,13 +297,14 @@ init_score (DenemoScore * si, DenemoGUI *gui)
   //gui->movements = g_list_append(gui->movements, si);
 }
 
-static delete_all_staffs(DenemoScore * si) {
+static delete_all_staffs(DenemoGUI * gui) {
+  DenemoScore *si=gui->si;
   gint i;
   for (i = g_list_length (si->thescore); i>0; i--)
     {
       si->currentstaffnum = i;
       si->currentstaff = g_list_nth (si->thescore, i - 1);
-      deletestaff (si);
+      deletestaff (gui, FALSE);
     }
 }
 
@@ -269,10 +314,10 @@ static delete_all_staffs(DenemoScore * si) {
  * @param si pointer to the scoreinfo structure to free
  */
 void
-free_score (DenemoScore * si)
+free_score (DenemoGUI * gui)
 {
-  delete_all_staffs(si);
-#define  HEADER(field)   g_string_free(si->headerinfo.field, TRUE)
+  delete_all_staffs(gui);
+#define  HEADER(field)   g_string_free(gui->si->headerinfo.field, TRUE)
   HEADER(title);
   HEADER(subtitle);
   HEADER(poet);
@@ -287,12 +332,12 @@ free_score (DenemoScore * si)
   HEADER(copyright);
   HEADER(footer);
   HEADER(tagline);
-  HEADER(markup_before);
-  HEADER(markup_after);
+  HEADER(lilypond_before);
+  HEADER(lilypond_after);
   HEADER(extra);
 #undef HEADER
 
-  g_queue_free (si->undodata);
-  g_queue_free (si->redodata);
+  g_queue_free (gui->si->undodata);
+  g_queue_free (gui->si->redodata);
 }
 
