@@ -13,11 +13,13 @@ void new_empty_score(DenemoGUI * gui);
 void new_score(DenemoGUI * gui);
 void init_score (DenemoScore *si, DenemoGUI * gui);
 
-void free_score (DenemoScore *si);
+void free_score (DenemoGUI *gui);
 void next_movement (GtkAction * action, DenemoGUI * gui);
 void prev_movement (GtkAction * action, DenemoGUI * gui);
 void insert_movement_before (GtkAction * action, DenemoGUI *gui);
 void insert_movement_after (GtkAction * action, DenemoGUI *gui);
 void delete_movement (GtkAction * action, DenemoGUI * gui);
 void movement_props_dialog (GtkAction * action, DenemoGUI * gui);
+gboolean
+goto_movement_staff_obj (DenemoGUI * gui, gint movementnum, gint staffnum, gint measurenum, gint objnum);
 #endif
