@@ -14,6 +14,8 @@
 /**
  * Instrument properties
  */
+
+
 typedef struct instproperties
 {
 	gint *clef;
@@ -67,7 +69,7 @@ static gchar *modes[7] =
  *
  */
 
-static void create_staff_instruments(wizarddata *wdata, gpointer data,
+static void create_staff_instruments(wizarddata *wdata, gchar *instrument_name,
 				     instproperties *inst)
 {
   gint numstaffsforinstrument = 1; //(gint) inst->numstaffs;
@@ -76,13 +78,11 @@ static void create_staff_instruments(wizarddata *wdata, gpointer data,
   DenemoStaff *curstaffstruct = 
     (DenemoStaff *) wdata->gui->si->currentstaff->data;
   //DenemoStaff *curstaffstruct  = g_list_last(wdata->gui->si->currentstaff);
-  gchar *instrument = (gchar *)data;
+
   //g_printf("clef # in create_staff_instrument = %i\n", inst->clef);
   gtk_widget_queue_draw (wdata->gui->scorearea);
-  printf("\nlocation before switch currentstaffnum = %i\n", 
-	 wdata->gui->si->currentstaffnum);
-  
-  GString *temp = g_string_new(instrument);
+  /* printf("\nlocation before switch currentstaffnum = %i\n",  */
+/* 	 wdata->gui->si->currentstaffnum); */
   switch((gint) inst->numstaffs)
     {
     case 1:
@@ -92,11 +92,11 @@ static void create_staff_instruments(wizarddata *wdata, gpointer data,
 	  //problem is it is not creating staves for multiple instruments
 	  {
 	    wdata->gui->si->currentstaffnum++;
-	    newstaff ((DenemoScore *) wdata->gui->si, 
+	    newstaff (wdata->gui, 
 		      ADDFROMLOAD, DENEMO_NONE);
 	    
-	    printf("\nlocation b before switch currentstaffnum = %i\n", 
-		   wdata->gui->si->currentstaffnum);
+/* 	    printf("\nlocation b before switch currentstaffnum = %i\n",  */
+/* 		   wdata->gui->si->currentstaffnum); */
 	    update_vscrollbar (wdata->gui);
 	    //staffdown(wdata->gui);
 	    displayhelper(wdata->gui);
@@ -107,11 +107,11 @@ static void create_staff_instruments(wizarddata *wdata, gpointer data,
 			   (int) ((instproperties *) inst)->clef);
 	//strcpy(curstaffstruct->denemo_name->str, instrument);
 
-	curstaffstruct->denemo_name->str = g_strdup(instrument);
+	g_string_assign(curstaffstruct->denemo_name, instrument_name);
 
 	set_lily_name (curstaffstruct->denemo_name,curstaffstruct->lily_name);
 	//strcpy(curstaffstruct->midi_instrument->str, instrument);
-	curstaffstruct->midi_instrument->str = g_strdup(instrument);
+	g_string_assign(curstaffstruct->midi_instrument, instrument_name);
 
 	curstaffstruct->transposition = (int) inst->transposition;
 	
@@ -128,7 +128,7 @@ static void create_staff_instruments(wizarddata *wdata, gpointer data,
 	      (numstaffsforinstrument == 2) || (wdata->currentstaffnumber > 1))
 	    {
 	      wdata->gui->si->currentstaffnum++;
-	      newstaff ((DenemoScore *) wdata->gui->si, ADDFROMLOAD, 
+	      newstaff (wdata->gui, ADDFROMLOAD, 
 			DENEMO_NONE);
 	      printf("\ninside case 2 first if currentstaffnum = %i\n", 
 		     wdata->gui->si->currentstaffnum); 
@@ -148,35 +148,30 @@ static void create_staff_instruments(wizarddata *wdata, gpointer data,
 	      (DenemoStaff *) wdata->gui->si->currentstaff->data;
 	    dnm_setinitialclef(wdata->gui->si, curstaffstruct, 
 			       DENEMO_TREBLE_CLEF);
-	    curstaffstruct->context = DENEMO_PIANO;
-	    
-	    g_string_assign(temp, instrument);
-	    gchar *pianobasslh = "rh";
-	    g_string_append(temp,pianobasslh);
-	    //strcpy(curstaffstruct->denemo_name->str, temp->str);
-	    curstaffstruct->denemo_name->str = g_strdup(temp->str);
+	    curstaffstruct->context = DENEMO_PIANO_START;
+
+	    g_string_printf(curstaffstruct->denemo_name, "%s%s", instrument_name, "rh");
+
 
 	    set_lily_name (curstaffstruct->denemo_name,
 			   curstaffstruct->lily_name);
 	    //strcpy(curstaffstruct->midi_instrument->str, instrument);
-	    curstaffstruct->midi_instrument->str = g_strdup(instrument);
+	    g_string_assign(curstaffstruct->midi_instrument, instrument_name);
 	  }
 	  if (numstaffsforinstrument == 2){
 	    wdata->gui->si->currentstaff = 
 	      g_list_last (wdata->gui->si->thescore);
 	    curstaffstruct = 
 	      (DenemoStaff *) wdata->gui->si->currentstaff->data;
-	    printf("\ninside case 2 third if currentstaffnum = %i\n", 
-		   wdata->gui->si->currentstaffnum);
+/* 	    printf("\ninside case 2 third if currentstaffnum = %i\n",  */
+/* 		   wdata->gui->si->currentstaffnum); */
 	    dnm_setinitialclef(wdata->gui->si, curstaffstruct, 
 			       DENEMO_BASS_CLEF);
-	    curstaffstruct->context = DENEMO_PIANO;
-	    g_string_assign(temp, instrument);
-	    gchar *pianobassrh = "lh";
-	    g_string_append(temp,pianobassrh);
-	    // strcpy(curstaffstruct->denemo_name->str, temp->str);
+	    curstaffstruct->context = DENEMO_PIANO_END;
+
+	    g_string_printf(curstaffstruct->denemo_name, "%s%s", instrument_name, "lh");
+
 
-	    curstaffstruct->denemo_name->str = g_strdup(temp->str);
 	    set_lily_name (curstaffstruct->denemo_name,
 			   curstaffstruct->lily_name);
 	    //strcpy(curstaffstruct->midi_instrument->str, instrument);
@@ -184,7 +179,7 @@ static void create_staff_instruments(wizarddata *wdata, gpointer data,
 	  }
 	  numstaffsforinstrument++;
 	}
-	wdata->staffnumber--;
+	wdata->staffnumber--;//FIXME should be a for loop
 	break;
       }
     case 3:
@@ -192,7 +187,7 @@ static void create_staff_instruments(wizarddata *wdata, gpointer data,
 	if (wdata->staffnumber != 1)
 	  {
 	    wdata->gui->si->currentstaffnum++;
-	    newstaff ((DenemoScore *) wdata->gui->si, 
+	    newstaff (wdata->gui, 
 		      ADDFROMLOAD, DENEMO_NONE);
 	    update_vscrollbar(wdata->gui);
 	    wdata->gui->si->currentstaff = 
@@ -207,9 +202,9 @@ static void create_staff_instruments(wizarddata *wdata, gpointer data,
       }
     };
   
-  printf("\nlocation c currentstaffnum = %i\n", 
-	 wdata->gui->si->currentstaffnum);
-  printf("\nstaffnumger = %i\n",wdata->staffnumber);
+/*   printf("\nlocation c currentstaffnum = %i\n",  */
+/* 	 wdata->gui->si->currentstaffnum); */
+/*   printf("\nstaffnumger = %i\n",wdata->staffnumber); */
 }
 
 
@@ -225,9 +220,10 @@ static void addinstrument(GtkButton *button, wizarddata *data)
   data->sdata->selection =
     gtk_tree_view_get_selection(GTK_TREE_VIEW(data->instrument_list));
   GtkTreeIter iter;
-  gtk_tree_selection_get_selected(GTK_TREE_SELECTION(data->sdata->selection), 
+  if(!gtk_tree_selection_get_selected(GTK_TREE_SELECTION(data->sdata->selection), 
 				  (GtkTreeModel **)&data->sdata->list_store, 
-				  &iter);
+				     &iter))
+    return;
   gchar *name;
 
   gtk_tree_model_get(GTK_TREE_MODEL(data->sdata->list_store), &iter, 0, 
@@ -318,7 +314,7 @@ cell_clicked(GtkTreeSelection *selection,
 {
 
   gboolean ret = FALSE;
-  g_print("\nIn cell_clicked callback\n");
+  //g_print("\nIn cell_clicked callback\n");
   wizarddata *wdata = (wizarddata *)user_data;
   instsdata *cbdata = (instsdata *) wdata->cbdata;
   GList *instruments = wdata->icbdata->instruments;
@@ -479,7 +475,7 @@ papersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook)
   gint i;
 
   papersetupcb *setup = (papersetupcb *) g_malloc0(sizeof(papersetupcb));
-  GtkWidget *table = gtk_table_new(3, 3 , FALSE);
+  GtkWidget *table = gtk_table_new(3, 4 , FALSE);
   gtk_container_set_border_width (GTK_CONTAINER (table), 12);
   gtk_table_set_row_spacings (GTK_TABLE (table), 8);
   gtk_table_set_col_spacings (GTK_TABLE (table), 8);
@@ -495,7 +491,7 @@ papersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook)
     }
 
   gtk_entry_set_text(GTK_ENTRY(GTK_BIN(papersize)->child), 
-		     gui->si->sconfig->papersize->str);
+		     gui->lilycontrol.papersize->str);
   gtk_table_attach(GTK_TABLE(table), papersize,1,2,0,1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
@@ -510,9 +506,10 @@ papersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook)
     {
       gtk_combo_box_append_text(GTK_COMBO_BOX(fontsize), fontsizes[i]);
     }
-	 gchar tmp[10];
-	 sprintf(tmp, "%d", gui->si->sconfig->fontsize);
+  gchar *tmp;
+  tmp = g_strdup_printf( "%d", gui->lilycontrol.fontsize);
   gtk_entry_set_text (GTK_ENTRY (GTK_BIN(fontsize)->child), tmp);
+  g_free(tmp);
   gtk_table_attach(GTK_TABLE(table), fontsize, 1,2,1,2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
@@ -528,7 +525,19 @@ papersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook)
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
   gtk_entry_set_text(GTK_ENTRY(lilyversion), 
-		     gui->si->sconfig->lilyversion->str);
+		     gui->lilycontrol.lilyversion->str);
+
+  label = gtk_label_new(_("LilyPond Control"));
+  gtk_table_attach(GTK_TABLE(table), label, 0,1,3,4,
+                   (GtkAttachOptions) (GTK_FILL),
+                   (GtkAttachOptions) (0), 0, 0);
+
+  GtkWidget *lilypond = gtk_entry_new();
+  gtk_table_attach(GTK_TABLE(table), lilypond, 1,2,3,4,
+                   (GtkAttachOptions) (GTK_FILL),
+                   (GtkAttachOptions) (0), 0, 0);
+  gtk_entry_set_text(GTK_ENTRY(lilypond), 
+		     gui->lilycontrol.lilypond->str);
 
 
   GtkWidget *vbox = gtk_vbox_new(FALSE,0);
@@ -541,7 +550,7 @@ papersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook)
     (gtk_radio_button_group (GTK_RADIO_BUTTON (portraitradio)),_("Landscape"));
   gtk_box_pack_start(GTK_BOX(vbox), landscaperadio, TRUE, TRUE,0);
 
-  if(gui->si->sconfig->orientation)
+  if(gui->lilycontrol.orientation)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(portraitradio), TRUE);
   else
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(landscaperadio), TRUE);
@@ -564,7 +573,8 @@ papersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook)
   setup->fontsize = fontsize;
   setup->portrait = portraitradio;
   setup->lilyversion = lilyversion;
-
+  setup->lilypond = lilypond;
+  score_status(gui, TRUE);
   return setup;
 }
 
@@ -618,7 +628,7 @@ timekeysig(wizarddata *wdata, gboolean isnotebook)
   gtk_table_attach(GTK_TABLE(table), tempo, 1,2,0,1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
-  //gtk_entry_set_text(GTK_ENTRY(lilyversion), gui->si->sconfig->lilyversion->str);
+  //gtk_entry_set_text(GTK_ENTRY(lilyversion), gui->lilycontrol.lilyversion->str);
   /*timesig*/
   label = gtk_label_new (_("Enter desired time signature:"));
   gtk_table_attach(GTK_TABLE(table), label, 0,1,1,2,
@@ -725,7 +735,7 @@ timekeysig(wizarddata *wdata, gboolean isnotebook)
   timekeysigdata->numerator = numerator;
   timekeysigdata->denominator = denominator;
 
-  cbdata->si = gui->si;
+  cbdata->gui = gui;
   cbdata->curstaffstruct = curstaffstruct;
   cbdata->combobox = combobox;
   cbdata->radiobutton2 = radiobutton2;
@@ -738,25 +748,25 @@ timekeysig(wizarddata *wdata, gboolean isnotebook)
 }
 
 void
-applyheader_settings(headersetupcb *hsetup, DenemoScore *si)
+applyheader_settings(headersetupcb *hsetup, DenemoGUI *gui)
 {
   //DenemoScore *si = wdata->gui->si;
   //headersetupcb *hsetup = wdata->hsetup;
-  g_string_assign(si->headerinfo->title, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->title)));
-  g_string_assign(si->headerinfo->subtitle, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->subtitle)));
-  g_string_assign(si->headerinfo->poet, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->poet)));
-  g_string_assign(si->headerinfo->composer, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->composer)));
-  g_string_assign(si->headerinfo->meter, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->meter)));
-  g_string_assign(si->headerinfo->opus, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->opus)));
-  g_string_assign(si->headerinfo->arranger, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->arranger)));
-  g_string_assign(si->headerinfo->instrument, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->instrument)));
-  g_string_assign(si->headerinfo->dedication, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->dedication)));
-  g_string_assign(si->headerinfo->piece, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->piece)));
-  g_string_assign(si->headerinfo->head, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->head)));
-  g_string_assign(si->headerinfo->copyright, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->copyright)));
-  g_string_assign(si->headerinfo->footer, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->footer)));
-  g_string_assign(si->headerinfo->tagline, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->tagline)));
-
+  g_string_assign(gui->si->headerinfo.title, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->title)));
+  g_string_assign(gui->si->headerinfo.subtitle, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->subtitle)));
+  g_string_assign(gui->si->headerinfo.poet, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->poet)));
+  g_string_assign(gui->si->headerinfo.composer, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->composer)));
+  g_string_assign(gui->si->headerinfo.meter, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->meter)));
+  g_string_assign(gui->si->headerinfo.opus, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->opus)));
+  g_string_assign(gui->si->headerinfo.arranger, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->arranger)));
+  g_string_assign(gui->si->headerinfo.instrument, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->instrument)));
+  g_string_assign(gui->si->headerinfo.dedication, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->dedication)));
+  g_string_assign(gui->si->headerinfo.piece, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->piece)));
+  g_string_assign(gui->si->headerinfo.head, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->head)));
+  g_string_assign(gui->si->headerinfo.copyright, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->copyright)));
+  g_string_assign(gui->si->headerinfo.footer, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->footer)));
+  g_string_assign(gui->si->headerinfo.tagline, (gchar *) gtk_entry_get_text (GTK_ENTRY (hsetup->tagline)));
+  score_status(gui, TRUE);
 }
 
 headersetupcb *
@@ -773,7 +783,7 @@ headersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook)
   gtk_table_set_col_spacings (GTK_TABLE (table), 8);
 
   /** title **/
-#define SET_TEXT(label) gtk_entry_set_text (GTK_ENTRY (label), gui->si->headerinfo->label->str);
+#define SET_TEXT(label) gtk_entry_set_text (GTK_ENTRY (label), gui->si->headerinfo.label->str);
   label = gtk_label_new(_("Title"));
   gtk_table_attach(GTK_TABLE(table), label, 0,1,1,2,
                    (GtkAttachOptions) (GTK_FILL),
@@ -877,14 +887,13 @@ headersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook)
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
 
-  /** Piece **/
   label = gtk_label_new(_("Piece"));
   gtk_table_attach(GTK_TABLE(table), label, 0,1,10,11,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
 
   piece = gtk_entry_new();
-  SET_TEXT(piece);
+  gtk_entry_set_text (GTK_ENTRY (piece), gui->si->headerinfo.piece->str);
   gtk_table_attach(GTK_TABLE(table), piece, 1,2,10,11,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
@@ -963,6 +972,7 @@ headersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook)
   setup->copyright = copyright;
   setup->footer = footer;
   setup->tagline = tagline;
+  score_status(gui, TRUE);
   return setup;
 }
 
@@ -970,33 +980,37 @@ headersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook)
  * Function to set the printed score parameters
  *
  */
-void setpaperconfig(papersetupcb *cbdata, DenemoScore *si)
+void setpaperconfig(papersetupcb *cbdata, DenemoGUI *gui)
 {
-  g_string_assign(si->sconfig->papersize, 
+  g_string_assign(gui->lilycontrol.papersize, 
 		  (gchar *) gtk_entry_get_text 
 		  (GTK_ENTRY (GTK_BIN (cbdata->papersize)->child)));
-  g_string_assign(si->sconfig->lilyversion, 
+  g_string_assign(gui->lilycontrol.lilyversion, 
 		  (gchar *)gtk_entry_get_text 
 		  (GTK_ENTRY (cbdata->lilyversion)));
-  si->sconfig->fontsize = 
+  g_string_assign(gui->lilycontrol.lilypond, 
+		  (gchar *)gtk_entry_get_text 
+		  (GTK_ENTRY (cbdata->lilypond)));
+
+  gui->lilycontrol.fontsize = 
     atoi((gchar *) gtk_entry_get_text 
 	 (GTK_ENTRY (GTK_BIN (cbdata->fontsize)->child)));
   if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(cbdata->portrait)))
     {
-      si->sconfig->orientation = TRUE;
+      gui->lilycontrol.orientation = TRUE;
     }
   else
-    si->sconfig->orientation = FALSE;
-
-  //g_print(" %s %s %d %d \n", si->sconfig->papersize->str, si->sconfig->lilyversion->str, si->sconfig->fontsize, si->sconfig->orientation);
+    gui->lilycontrol.orientation = FALSE;
 
+  //g_print(" %s %s %d %d \n", gui->lilycontrol.papersize->str, gui->lilycontrol.lilyversion->str, gui->lilycontrol.fontsize, gui->lilycontrol.orientation);
+  score_status(gui, TRUE);
 }
 
 /*
- * Create new staff's and label them with instrument names. 
+ * Create new staffs and label them with instrument names. 
  * 
  */
-void setstaffconfig(wizarddata *wdata, DenemoScore *si)
+static void setstaffconfig(wizarddata *wdata, DenemoScore *si)
 {
   wdata->staffnumber = gtk_tree_model_iter_n_children 
     (GTK_TREE_MODEL(wdata->sdata->score_list), NULL);
@@ -1004,7 +1018,7 @@ void setstaffconfig(wizarddata *wdata, DenemoScore *si)
 			 foreach_func, wdata);
 }
 
-void applykeytimesig_settings(wizarddata *wdata)
+static void applykeytimesig_settings(wizarddata *wdata)
 {
   DenemoScore *si = wdata->gui->si;
   timekeysigcb *tsetup = wdata->tsetup;
@@ -1019,7 +1033,7 @@ void applykeytimesig_settings(wizarddata *wdata)
   curstaffstruct->stime2 = 
     gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tsetup->denominator));
 
-  si->haschanged = TRUE;
+  // wdata->gui->haschanged = TRUE;???????? why was this set???
   //barlength = PPQN * 4 * curstaffstruct->stime1 / curstaffstruct->stime2;
 
   gint tokey, mode;
@@ -1034,7 +1048,7 @@ void applykeytimesig_settings(wizarddata *wdata)
     2 : 0;
   tokey = findkey (tsetup->cbdata->combobox, tsetup->cbdata->mdata, isminor);
  
-  printf("\nisminor = %i tokey = %i\n",isminor,tokey);
+  //printf("\nisminor = %i tokey = %i\n",isminor,tokey);
   if (isminor == 2)
     mode = findmode (tsetup->cbdata->mode, tsetup->cbdata->mdata);
   dnm_setinitialkeysig (curstaffstruct, tokey - mode, isminor);
@@ -1050,10 +1064,11 @@ void applykeytimesig_settings(wizarddata *wdata)
 
 static void setupscore(GtkButton *button, wizarddata *wdata)
 {
-  setpaperconfig(wdata->paper, wdata->gui->si);
-  applyheader_settings(wdata->hsetup,wdata->gui->si);
+  setpaperconfig(wdata->paper, wdata->gui);
+  applyheader_settings(wdata->hsetup,wdata->gui);
   applykeytimesig_settings(wdata);
   setstaffconfig(wdata, wdata->gui->si);
+  score_status(wdata->gui, TRUE);
   set_bottom_staff(wdata->gui);      
   displayhelper(wdata->gui);
   /*free memory*/
@@ -1063,7 +1078,7 @@ static void setupscore(GtkButton *button, wizarddata *wdata)
   g_free(wdata->sdata);
   g_free(wdata->icbdata);
   g_free(wdata);
-
+  
 }
 
 void notebook_page(gpointer data)
@@ -1093,7 +1108,7 @@ void notebook_page(gpointer data)
  */
 void scorewizard(GtkAction *action, DenemoGUI *gui)
 {
-  if (gui->si->haschanged)
+  if (gui->changecount)
     {
       if (confirmbox (gui))
         deletescore (NULL, gui);
@@ -1168,6 +1183,7 @@ void scorewizard(GtkAction *action, DenemoGUI *gui)
   gtk_widget_show_all(dialog);
      
   g_list_free(wdata->instruments);
+  score_status(gui, FALSE);/* clear the change count */
 }
 
 
