@@ -45,6 +45,7 @@ typedef struct papersetupcb
 	GtkWidget *lilyversion;
 	GtkWidget *portrait;
 	GtkWidget *fontsize;
+	GtkWidget *lilypond;
 }papersetupcb;
 
 
@@ -114,8 +115,8 @@ typedef struct wizarddata
 
 
 void scorewizard(GtkAction *action, DenemoGUI *gui);
-void setpaperconfig(papersetupcb *cbdata, DenemoScore *si);
-void applyheader_settings(headersetupcb *hsetup, DenemoScore *si);
+void setpaperconfig(papersetupcb *cbdata, DenemoGUI *gui);
+void applyheader_settings(headersetupcb *hsetup, DenemoGUI *gui);
 papersetupcb *papersetup(GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook); /* isnotebook should defaults to TRUE */
 headersetupcb *headersetup (GtkWidget *notebook, DenemoGUI *gui, gboolean isnotebook);
 #endif
