@@ -19,7 +19,6 @@
 #include "staffops.h"
 #include "prefops.h"
 /*For save selection function*/
-#include "frogio.h"
 #include "utils.h"
 
 /**
@@ -323,7 +322,7 @@ cuttobuffer (DenemoScore * si)
  * pastefrombuffer
  * Pastes the current buffer to the score
  *
- * @param si pointer to the score information structure
+ * @param gui the DenemoGUI to be used.
  *
  * The updates that are done towards the bottom of this function -
  * beamsandstemdirswholestaff, find_xes_in_all_measures, etc. - are
@@ -331,9 +330,10 @@ cuttobuffer (DenemoScore * si)
  * but additional lines-of-code required for implementation. 
  */
 
-void
-pastefrombuffer (DenemoScore * si)
+static void
+pastefrombuffer (DenemoGUI * gui)
 {
+  DenemoScore *si = gui->si;
   staffnode *curstaff;
   measurenode *curmeasure;
   GList *curbuffernode = copybuffer;
@@ -400,8 +400,8 @@ pastefrombuffer (DenemoScore * si)
 	      
 	      g_debug ("offend %d start of next note %d\n", 
 		       si->cursoroffend, clonedobject->starttick);
-	      nextmeasure (si, FALSE);
-	      object_insert(si, clonedobject);
+	      insertion_point (si);
+	      object_insert(gui, clonedobject);
 	      /*	      curmeasure->data =
 	      g_list_insert ((objnode *) curmeasure->data, clonedobject,
 	      insertat);*/
@@ -410,7 +410,7 @@ pastefrombuffer (DenemoScore * si)
 	      si->cursoroffend = (si->currentobject?
 		((DenemoObject *)si->currentobject->data)->starttickofnextnote
 				  >= (WHOLE_NUMTICKS * si->cursortime1 / si->cursortime2): 0);   // guess  
-	      /*object_insert(si, clonedobject); */
+
 	      insertat++;
 	    }
 	}			/* End bufferobj loop */
@@ -418,7 +418,7 @@ pastefrombuffer (DenemoScore * si)
       beamsandstemdirswholestaff ((DenemoStaff *) curstaff->data);
     }				/* End staff loop */
 
-  si->haschanged = TRUE;
+
   si->currentmeasure = g_list_nth (firstmeasurenode (si->currentstaff),
 				   si->currentmeasurenum - 1);
   si->currentobject = g_list_nth ((objnode *) si->currentmeasure->data,
@@ -498,7 +498,8 @@ cutwrapper (GtkAction * action, DenemoGUI * gui)
 void
 pastewrapper (GtkAction * action, DenemoGUI * gui)
 {
-  pastefrombuffer (gui->si);
+  pastefrombuffer (gui);
+  score_status(gui, TRUE);
   displayhelper (gui);
 }
 
@@ -650,7 +651,7 @@ undo (DenemoGUI * gui)
   unre_data *undo=NULL;
   unre_data *redo = (unre_data *) g_malloc (sizeof (unre_data));//FIXME memory leak
 
-  if (gui->si->haschanged && g_queue_get_length (gui->si->undodata) > 0)
+  if (gui->changecount /* consider has the changed been to this movement?? */&& g_queue_get_length (gui->si->undodata) > 0)
     {
       gui->si->undo_redo_mode = UNDO;
       undo = (unre_data *) g_queue_pop_head (gui->si->undodata);
@@ -676,7 +677,7 @@ undo (DenemoGUI * gui)
       else if (undo->action == ACTION_DELETE)
 	{
 
-	  object_insert (gui->si, undo->object);
+	  object_insert (gui, undo->object);
 
 	  g_debug ("UNDO Delete Object %d\n",
 		   ((DenemoObject *) undo->object)->type);
@@ -711,7 +712,7 @@ redo (DenemoGUI * gui)
   unre_data *redo = NULL;
   
   DenemoScore *si = gui->si;
-  if (si->haschanged && g_queue_get_length (si->redodata) > 0)
+  if (gui->changecount && g_queue_get_length (si->redodata) > 0)
     {
       si->undo_redo_mode = REDO;
       redo = (unre_data *) g_queue_pop_head (si->redodata);
@@ -741,7 +742,7 @@ redo (DenemoGUI * gui)
 	  si->currentmeasurenum = redo->measurenum;
 	  si->cursor_x = redo->position;
 	  setcurrents (si);
-	  object_insert (si, (DenemoObject *) redo->object);
+	  object_insert (gui, (DenemoObject *) redo->object);
 
 	  redo->action = ACTION_DELETE;
 	}
@@ -766,7 +767,6 @@ update_undo_info (DenemoScore * si, unre_data * undo)
 
   g_debug ("Undo structure: Action %d, Position %d,  Staff %d, Measure %d\n",
 	   undo->action, undo->position, undo->staffnum, undo->measurenum); 
-  si->haschanged = TRUE;
 
   if (g_queue_get_length (si->undodata) == MAX_UNDOS)
     {
