@@ -174,9 +174,10 @@ insert_staff (DenemoScore * si, DenemoStaff * thestaffstruct,
  * @return none
  */
 void
-newstaff (DenemoScore * si, enum newstaffcallbackaction action,
+newstaff (DenemoGUI * gui, enum newstaffcallbackaction action,
 	  DenemoContext context)
 {
+  DenemoScore *si = gui->si;
   g_assert (si != NULL);
   /* What gets added */
   gint ret = -1;
@@ -273,6 +274,11 @@ newstaff (DenemoScore * si, enum newstaffcallbackaction action,
   thestaffstruct->measures = themeasures;
   thestaffstruct->denemo_name = g_string_new (NULL);
   thestaffstruct->lily_name = g_string_new (NULL);
+
+  thestaffstruct->staff_prolog = NULL;
+  thestaffstruct->lyrics_prolog = NULL;
+  thestaffstruct->figures_prolog = NULL;
+  thestaffstruct->fakechords_prolog = NULL;
   thestaffstruct->context = context;
   if (action == NEWVOICE)
     g_string_sprintf (thestaffstruct->denemo_name, _("poly voice %d"),
@@ -313,7 +319,7 @@ newstaff (DenemoScore * si, enum newstaffcallbackaction action,
       addat = 1;
       break;
     }
-
+  itp.gui = gui;
   itp.staff = thestaffstruct;
   itp.addat = addat;
   if (action != INITIAL && action != ADDFROMLOAD)
@@ -356,29 +362,55 @@ newstaff (DenemoScore * si, enum newstaffcallbackaction action,
   else // is INITIAL or ADDFROMLOAD
     {
       insert_staff (si, thestaffstruct, action, addat);
+      si->leftmeasurenum = 1;
     }
 
   //si->haschanged = TRUE;
 }
 
-/**
- * Remove the si->currentstaff from the score si and resets si->currentstaff
- * @param si the scoreinfo structure
 
- * @return TRUE if a staff was deleted
+gboolean confirm_insertstaff_custom_scoreblock(DenemoGUI *gui) {
+  if(gui->custom_scoreblocks)
+    return confirm("Custom LilyPond Score Block", "You will need to edit the LilyPond text to make this change show in the print-out. Proceed?");
+  return TRUE;
+}
+
+gboolean confirm_deletestaff_custom_scoreblock(DenemoGUI *gui) {
+  if(gui->custom_scoreblocks)
+    return confirm("Custom LilyPond Score Block", "You will need to edit the LilyPond text to delete any reference to the deleted staff(s) in the scoreblock. Proceed?");
+  return TRUE;
+}
+
+
+/**
+ * Remove the gui->si->currentstaff from the piece gui and reset si->currentstaff
+ * if only one staff, inserts a new empty one
+ * if interactive checks for custom_scoreblock
+ * if a staff is deleted, updates the changecount
+ * @param gui the DenemoGUI structure
+ * @return nothing
  */
-gboolean
-deletestaff (DenemoScore * si)
+void
+deletestaff (DenemoGUI * gui, gboolean interactive)
 {
-  gboolean ret=FALSE;
+  DenemoScore *si = gui->si;
+  if(interactive && !confirm_deletestaff_custom_scoreblock(gui))
+    return;
   if(si->currentstaff==NULL)
-    return ret;
+    return;
   DenemoStaff *curstaffstruct = si->currentstaff->data;
+
+  gboolean give_info=FALSE;//give info about removing matching context
+  if(interactive && (curstaffstruct->context!=DENEMO_NONE) &&
+     (!confirm("A context is set on this staff", "You will need to alter/delete the matching staff; Proceed?")))
+    return;
+  if(interactive &&  (curstaffstruct->context!=DENEMO_NONE))
+    give_info = TRUE;
   gboolean isprimary = ((int) curstaffstruct->voicenumber == 1);
   //FIXME free_staff()
   g_list_foreach (curstaffstruct->measures, freeobjlist, NULL);
   g_list_free (curstaffstruct->measures);
-  g_string_free (curstaffstruct->denemo_name, FALSE);
+  g_string_free (curstaffstruct->denemo_name, FALSE);//FIXME these should all be TRUE??
   g_string_free (curstaffstruct->lily_name, FALSE);
   g_string_free (curstaffstruct->midi_instrument, FALSE);
   g_free (curstaffstruct);
@@ -386,10 +418,11 @@ deletestaff (DenemoScore * si)
     si->currentstaffnum--;//deleting the last, so the currentstaffnum must decrease
   si->thescore = g_list_delete_link (si->thescore, si->currentstaff);
   if(si->thescore==NULL) {
-    ret = FALSE;
-    newstaff (si, INITIAL, DENEMO_NONE);
+    newstaff (gui, INITIAL, DENEMO_NONE);
   }
   si->currentstaff = g_list_nth (si->thescore, si->currentstaffnum - 1);
+
+
   if (isprimary) // we deleted the primary, so the next one must become the primary
     {
       ((DenemoStaff *) si->currentstaff->data)->voicenumber = 1;
@@ -397,7 +430,17 @@ deletestaff (DenemoScore * si)
     } else {
       setcurrentprimarystaff (si);
     }
-  return ret;
+
+  //FIXME none of this works to get the current measure stem direction correct
+      setcurrents (si);
+      find_xes_in_all_measures (si);
+      beamsandstemdirswholestaff ((DenemoStaff *) si->currentstaff->data);
+      si->markstaffnum = 0;
+      displayhelper (gui);
+  score_status(gui,TRUE);
+  if(give_info)
+    infodialog ("The staff deleted had a start/end context; if you still have the staff with the matching end/start context\n then you should remove it (or its context) now.\nSee Staff->properties->context\nYou will not be able to print with miss-matched contexts.");
+  return;
 }	
 
 /**
@@ -488,6 +531,8 @@ fixnoteheights (DenemoStaff * thestaff)
   beamsandstemdirswholestaff (thestaff);
 }
 
+
+
 /**
  * Callback function to insert a staff in the initial position
  * @param action a Gtk Action
@@ -511,9 +556,10 @@ newstaffinitial (GtkAction * action, DenemoGUI * gui)
 void
 newstaffbefore (GtkAction * action, DenemoGUI * gui)
 {
-  //  g_print ("New Staff before: score info %X\n", data);
+  if(!confirm_insertstaff_custom_scoreblock(gui))
+    return;
   tohome(NULL, gui);
-  newstaff (gui->si, BEFORE, DENEMO_NONE);
+  newstaff (gui, BEFORE, DENEMO_NONE);
   if(gui->si->currentstaffnum>= gui->si->top_staff)
     gui->si->top_staff++;
   gui->si->currentstaffnum++;
@@ -535,8 +581,10 @@ newstaffbefore (GtkAction * action, DenemoGUI * gui)
 void
 dnm_newstaffafter (GtkAction * action, DenemoGUI * gui)
 {
+  if(!confirm_insertstaff_custom_scoreblock(gui))
+    return;
   tohome(NULL, gui);
-  newstaff (gui->si, AFTER, DENEMO_NONE);
+  newstaff (gui, AFTER, DENEMO_NONE);
   set_bottom_staff (gui);
   update_vscrollbar (gui);
   staffdown (gui);
@@ -566,7 +614,7 @@ newstafflast (GtkAction * action, DenemoGUI * gui)
 void
 dnm_newstaffvoice (GtkAction * action, DenemoGUI * gui)
 {
-  newstaff (gui->si, NEWVOICE, DENEMO_NONE);
+  newstaff (gui, NEWVOICE, DENEMO_NONE);
   set_bottom_staff (gui);
   update_vscrollbar (gui);
   setcurrents(gui->si);//staffdown (gui);
@@ -582,7 +630,7 @@ dnm_newstaffvoice (GtkAction * action, DenemoGUI * gui)
 void
 newstafflyric (GtkAction * action, DenemoGUI * gui)
 {
-  newstaff (gui->si, LYRICSTAFF, DENEMO_NONE);
+  newstaff (gui, LYRICSTAFF, DENEMO_NONE);
   set_bottom_staff (gui);
   update_vscrollbar (gui);
   staffdown (gui);
@@ -599,7 +647,7 @@ newstafflyric (GtkAction * action, DenemoGUI * gui)
 void
 dnm_newstafffigured (GtkAction * action, DenemoGUI * gui)
 {
-  newstaff (gui->si, FIGURESTAFF, DENEMO_NONE);
+  newstaff (gui, FIGURESTAFF, DENEMO_NONE);
   set_bottom_staff (gui);
   update_vscrollbar (gui);
   staffdown (gui);
@@ -618,7 +666,7 @@ void
 dnm_newstaffchords (GtkAction * action, DenemoGUI * gui)
 {
   printf("\n ok I am going to add chords to the staff now\n");
-  newstaff (gui->si, CHORDSTAFF, DENEMO_NONE);
+  newstaff (gui, CHORDSTAFF, DENEMO_NONE);
   //set_bottom_staff (gui);
   update_vscrollbar (gui);
   //staffdown (gui);
