@@ -22,7 +22,7 @@
 #endif
 
 /**
- * Displays warning to screen
+ * Pops up a warning dialog and blocks until it is dismissed
  *  @param msg warning message to display
  * @return none
  */
@@ -39,6 +39,22 @@ warningdialog (gchar * msg)
   gtk_widget_destroy (dialog);
 }
 
+/**
+ * Displays information message to screen, not blocking.
+ * User can destroy window when no longer needed.
+ * @param msg message to display
+ * @return none
+ */
+void
+infodialog (gchar * msg)
+{
+  GtkWidget *dialog;
+  dialog = gtk_message_dialog_new (NULL,
+				   GTK_DIALOG_DESTROY_WITH_PARENT,
+				   GTK_MESSAGE_INFO, GTK_BUTTONS_NONE, msg);
+  gtk_widget_show_all(dialog);
+}
+
 
 /**
  *  Draws the given bitmap mask on to the pixmap using the given 
@@ -525,6 +541,23 @@ get_data_dir ()
   return datadir;
 }
 
+const gchar *
+get_conf_dir ()
+{
+  static gchar *confdir = NULL;
+  if (confdir == NULL)
+  {
+#ifdef G_OS_WIN32
+  gchar *rootdir = g_win32_get_package_installation_directory (NULL,  NULL);
+  confdir = g_build_filename (rootdir, "etc", "denemo", NULL);
+  g_free (rootdir);
+#else /* not G_OS_WIN32 */
+  confdir = gbr_find_etc_dir(SYSCONFDIR);
+#endif /* not G_OS_WIN32 */
+  }
+  return confdir;
+}
+
 const gchar *
 get_plugin_dir ()
 {
@@ -598,14 +631,19 @@ void  set_title_bar(DenemoGUI *gui) {
     title = gui->filename->str;
   else
     title = "(Untitled)";
-  title = g_strdup_printf("Denemo - %s%c", title, gui->haschanged?'*':' ');   
+  title = g_strdup_printf("Denemo - %s%c", title, gui->changecount?'*':' ');   
   gtk_window_set_title (GTK_WINDOW (gui->window), title);
   g_free(title);
 }
 
-void score_status(DenemoGUI *gui, gboolean haschanged) {
-  if(gui->haschanged != haschanged) {
-    gui->haschanged = haschanged;
+void score_status(DenemoGUI *gui, gboolean change) {
+  if(change) { 
+    gui->changecount++;
+    if(gui->changecount==1)//just changed
+      set_title_bar(gui);
+
+  } else {
+    gui->changecount=0;
     set_title_bar(gui);
   }
   write_status(gui);
@@ -637,3 +675,78 @@ confirm (gchar *primary, gchar *secondary)
   gtk_widget_destroy (dialog);
   return r;
 }
+
+/* free a GString and the string it holds, and set the pointer to it to NULL */
+void nullify_gstring (GString **s) {
+  if(*s)
+    g_string_free(*s, TRUE);
+*s = NULL;
+}
+
+/**
+ * Pops up a dialog box that has a text entry box and ok/cancel buttons
+ * title is a title for the box.
+ * initial_value is for the text entry box, or NULL if none.
+ * instruction is a prompt for the user.
+ * Returns a new value on Ok and NULL if cancelled.
+ * The returned value should be freed by the caller.
+ *
+ */
+
+gchar *
+string_dialog_entry (DenemoGUI *gui, gchar *title, gchar *instruction, gchar *initial_value)
+{
+  string_dialog_entry_with_widget (gui, title, instruction, initial_value, NULL);
+}
+
+/* as string_dialog_entry() but with extra widget */
+gchar *
+string_dialog_entry_with_widget (DenemoGUI *gui, gchar *wlabel, gchar *direction, gchar *PreValue, GtkWidget *widget)
+{
+
+ 	GtkWidget *dialog;
+	GtkWidget *entry;
+	GtkWidget *label;
+	gchar *entry_string;
+	GString *string;
+	entry = gtk_entry_new ();
+
+	dialog = gtk_dialog_new_with_buttons (wlabel,
+                                        GTK_WINDOW (gui->window),
+                                        (GtkDialogFlags) (GTK_DIALOG_MODAL |
+                                                       GTK_DIALOG_DESTROY_WITH_PARENT),
+                                        GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+                                        GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+                                        NULL);
+
+	label = gtk_label_new (direction);
+  	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label,
+		                        TRUE, TRUE, 0);
+	if(widget)
+	  	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), widget,
+		                        TRUE, TRUE, 0);
+	if (PreValue != NULL) {
+            gtk_entry_set_text (GTK_ENTRY (entry), (gchar *) PreValue);
+        }
+  
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), entry,
+		                        TRUE, TRUE, 0);
+    	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+      	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+        gtk_widget_grab_focus (entry);
+  	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+	gtk_widget_show_all (dialog);
+
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){ 
+		entry_string = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
+		string = g_string_new(entry_string);
+		gtk_widget_destroy (dialog);
+		return g_string_free(string, FALSE);
+	}
+	else {  
+		gtk_widget_destroy (dialog);
+		return NULL;
+	}
+
+}
