@@ -92,6 +92,8 @@ freeit (gpointer data, gpointer user_data);
 
 void
 warningdialog(gchar *msg);
+void
+infodialog(gchar *msg);
 
 
 
@@ -103,6 +105,7 @@ headerfields(GtkWidget *dialog, GtkListStore *list_store,
 
 void initdir ();
 const gchar *get_data_dir ();
+const gchar *get_conf_dir ();
 const gchar *get_plugin_dir ();
 const gchar *get_locale_dir ();
 
@@ -111,7 +114,18 @@ void kill_process (GPid pid);
 gchar * music_font(gchar *str);
 
 void  set_title_bar(DenemoGUI *gui);
-void score_status(DenemoGUI *gui, gboolean haschanged);
+void score_status(DenemoGUI *gui, gboolean change);
 gboolean
 confirm (gchar *primary, gchar *secondary);
+void 
+nullify_gstring (GString **s);
+
+gchar *
+string_dialog_entry (DenemoGUI *gui, gchar *title, gchar *instruction, gchar *initial_value);
+
+gchar *
+string_dialog_entry_with_widget (DenemoGUI *gui, gchar *title, gchar *instruction, gchar *initial_value, GtkWidget *extra_widget);
+
+	
+
 #endif /* UTILS_H */
