@@ -14,7 +14,7 @@
 #include "draw.h"		/* Which includes gtk.h */
 #include <denemo/denemo.h>
 #include "dialogs.h"
-#include "exportmudela.h"
+#include "exportlilypond.h"
 #include "file.h"
 #include "gcs.h"
 #include "kbd-custom.h"
@@ -33,31 +33,34 @@
 #include "dynamic.h"
 #include "lyric.h"
 #include "figure.h"
+#include "fakechord.h"
 #include "changenotehead.h"
 #include "articulations.h"
-#include "csoundplayback.h"
+//#include "csoundplayback.h"
 #include "print.h"
 #include "barline.h"
 #include "plugin.h"
 #include "runsilent.h"
 
+#define EXTRA_ACCELS "extra.accels"
 
-
+void init_keymap(void);
 void newview(void);
-void createview(GtkAction *action,gpointer data);
-void closeit (GtkWidget * widget, gpointer data);
-void close_application (GtkWidget * widget, GdkEvent * event, gpointer data);
-void closewrapper (GtkAction *action,gpointer data);
-void openinnew(GtkAction *action,gpointer data);
-void close_view( GtkWidget *widget, gpointer data);
-void closeview(GtkAction *action,gpointer data);
-void addhistorymenuitem(gpointer data, gpointer user_data);
+void free_gui(DenemoGUI *gui);/* frees all movement data from gui, leaving gui interface intact */
+void addhistorymenuitem(gchar *filename, DenemoGUI *newgui);
 
 //TODO Fix for automatic updating during operation
 //
-struct cbd
-{
-	scoreinfo *si;
-	GtkWidget *menu;
-};
+
+/* tool_buttons cannot be visually separated from the toolbar, which is ok for icons but not so good for pure text; to separate them we put the label of the tool_button into an event box and decorate that. This macro gets the label out. */
+#define LABEL(a) (gtk_bin_get_child(GTK_BIN(gtk_tool_button_get_label_widget((a)))))
+
+void	  highlight_rhythm(RhythmPattern *r);
+
+
+void	  unhighlight_rhythm(RhythmPattern *r);
+void	  highlight_duration(DenemoGUI *gui, gint dur);
+void	  highlight_rest(DenemoGUI *gui, gint dur);
+void      write_status(DenemoGUI *gui);
+
 #endif
